import * as i0 from '@angular/core';
import { input, viewChild, Component, CUSTOM_ELEMENTS_SCHEMA, ChangeDetectionStrategy, computed, effect, untracked, inject, output, DestroyRef } from '@angular/core';
import * as i1 from 'angular-three';
import { omit, extend, injectBeforeRender, injectStore, pick, checkNeedsUpdate, NgtArgs, getLocalState, resolveRef, NgtObjectEvents } from 'angular-three';
import { mergeInputs } from 'ngxtension/inject-inputs';
import { Group, Quaternion, Vector3, Vector4, Vector2, Color, CatmullRomCurve3, CubicBezierCurve3, EdgesGeometry, CanvasTexture, BackSide, Plane, Mesh, PlaneGeometry, Object3D, Shape, ExtrudeGeometry, QuadraticBezierCurve3 } from 'three';
import { LineSegments2, Line2, LineMaterial, LineSegmentsGeometry, LineGeometry, toCreasedNormals, TextGeometry, mergeVertices } from 'three-stdlib';
import { DOCUMENT } from '@angular/common';
import { GridMaterial } from 'angular-three-soba/shaders';
import { assertInjector } from 'ngxtension/assert-injector';
import { Text, preloadFont } from 'troika-three-text';
import { injectFont } from 'angular-three-soba/loaders';

const defaultOptions$b = {
    follow: true,
    lockX: false,
    lockY: false,
    lockZ: false,
};
class NgtsBillboard {
    constructor() {
        this.options = input(defaultOptions$b, { transform: mergeInputs(defaultOptions$b) });
        this.parameters = omit(this.options, ['follow', 'lockX', 'lockY', 'lockZ']);
        this.groupRef = viewChild.required('group');
        this.innerRef = viewChild.required('inner');
        extend({ Group });
        const q = new Quaternion();
        injectBeforeRender(({ camera }) => {
            const [{ follow, lockX, lockY, lockZ }, group, inner] = [
                this.options(),
                this.groupRef().nativeElement,
                this.innerRef().nativeElement,
            ];
            if (!follow || !group)
                return;
            // save previous rotation in case we're locking an axis
            const prevRotation = group.rotation.clone();
            // always face the camera
            group.updateMatrix();
            group.updateWorldMatrix(false, false);
            group.getWorldQuaternion(q);
            camera.getWorldQuaternion(inner.quaternion).premultiply(q.invert());
            // readjust any axis that is locked
            if (lockX)
                group.rotation.x = prevRotation.x;
            if (lockY)
                group.rotation.y = prevRotation.y;
            if (lockZ)
                group.rotation.z = prevRotation.z;
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsBillboard, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsBillboard, isStandalone: true, selector: "ngts-billboard", inputs: { options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "groupRef", first: true, predicate: ["group"], descendants: true, isSignal: true }, { propertyName: "innerRef", first: true, predicate: ["inner"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-group #group [parameters]="parameters()">
			<ngt-group #inner>
				<ng-content />
			</ngt-group>
		</ngt-group>
	`, isInline: true, changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsBillboard, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-billboard',
                    template: `
		<ngt-group #group [parameters]="parameters()">
			<ngt-group #inner>
				<ng-content />
			</ngt-group>
		</ngt-group>
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }], ctorParameters: () => [] });

const defaultOptions$a = {
    lineWidth: 1,
    segments: false,
    color: 0xffffff,
};
class NgtsLine {
    // NOTE: use attached event to call computeLineDistances to ensure the geometry has been attached
    onGeometryAttached({ parent }) {
        parent.computeLineDistances();
    }
    constructor() {
        this.points = input.required();
        this.options = input(defaultOptions$a, { transform: mergeInputs(defaultOptions$a) });
        this.parameters = omit(this.options, ['color', 'vertexColors', 'lineWidth', 'segments', 'linewidth', 'dashed']);
        this.lineRef = viewChild('line');
        this.store = injectStore();
        this.size = this.store.select('size');
        this.segments = pick(this.options, 'segments');
        this.vertexColors = pick(this.options, 'vertexColors');
        this.dashed = pick(this.options, 'dashed');
        this.color = pick(this.options, 'color');
        this.vertex = computed(() => Boolean(this.vertexColors()));
        this.resolution = computed(() => [this.size().width, this.size().height]);
        this.lineWidth = pick(this.options, 'lineWidth');
        this.linewidth = pick(this.options, 'linewidth');
        this.line2 = computed(() => (this.segments() ? new LineSegments2() : new Line2()));
        this.lineMaterial = new LineMaterial();
        this.actualLineWidth = computed(() => this.linewidth() ?? this.lineWidth() ?? 1);
        this.itemSize = computed(() => (this.vertexColors()?.[0]?.length === 4 ? 4 : 3));
        this.lineGeometry = computed(() => {
            const geom = this.segments() ? new LineSegmentsGeometry() : new LineGeometry();
            const pValues = this.points().map((p) => {
                const isArray = Array.isArray(p);
                return p instanceof Vector3 || p instanceof Vector4
                    ? [p.x, p.y, p.z]
                    : p instanceof Vector2
                        ? [p.x, p.y, 0]
                        : isArray && p.length === 3
                            ? [p[0], p[1], p[2]]
                            : isArray && p.length === 2
                                ? [p[0], p[1], 0]
                                : p;
            });
            geom.setPositions(pValues.flat());
            const vertexColors = this.vertexColors();
            if (vertexColors) {
                const cValues = vertexColors.map((c) => (c instanceof Color ? c.toArray() : c));
                // @ts-expect-error - flat() isn't defined
                geom.setColors(cValues.flat(), this.itemSize());
            }
            return geom;
        });
        effect(() => {
            const [lineMaterial, dashed] = [this.lineMaterial, this.dashed()];
            if (dashed) {
                lineMaterial.defines['USE_DASH'] = '';
            }
            else {
                delete lineMaterial.defines['USE_DASH'];
            }
            checkNeedsUpdate(lineMaterial);
        });
        effect((onCleanup) => {
            const [lineGeometry, lineMaterial] = [this.lineGeometry(), this.lineMaterial];
            onCleanup(() => {
                lineGeometry.dispose();
                lineMaterial.dispose();
            });
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsLine, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsLine, isStandalone: true, selector: "ngts-line", inputs: { points: { classPropertyName: "points", publicName: "points", isSignal: true, isRequired: true, transformFunction: null }, options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "lineRef", first: true, predicate: ["line"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-primitive *args="[line2()]" #line [parameters]="parameters()">
			<ngt-primitive *args="[lineGeometry()]" attach="geometry" (attached)="onGeometryAttached($any($event))" />
			<ngt-primitive
				*args="[lineMaterial]"
				attach="material"
				[color]="color()"
				[vertexColors]="vertex()"
				[resolution]="resolution()"
				[linewidth]="actualLineWidth()"
				[dashed]="dashed()"
				[transparent]="itemSize() === 4"
				[parameters]="parameters()"
			/>
			<ng-content />
		</ngt-primitive>
	`, isInline: true, dependencies: [{ kind: "directive", type: NgtArgs, selector: "ng-template[args]", inputs: ["args"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsLine, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-line',
                    template: `
		<ngt-primitive *args="[line2()]" #line [parameters]="parameters()">
			<ngt-primitive *args="[lineGeometry()]" attach="geometry" (attached)="onGeometryAttached($any($event))" />
			<ngt-primitive
				*args="[lineMaterial]"
				attach="material"
				[color]="color()"
				[vertexColors]="vertex()"
				[resolution]="resolution()"
				[linewidth]="actualLineWidth()"
				[dashed]="dashed()"
				[transparent]="itemSize() === 4"
				[parameters]="parameters()"
			/>
			<ng-content />
		</ngt-primitive>
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    imports: [NgtArgs],
                }]
        }], ctorParameters: () => [] });

const defaultOptions$9 = {
    curveType: 'centripetal',
    tension: 0.5,
    segments: 20,
    closed: false,
    lineWidth: 1,
};
class NgtsCatmullRomLine {
    constructor() {
        this.points = input.required();
        this.options = input(defaultOptions$9, { transform: mergeInputs(defaultOptions$9) });
        this.parameters = omit(this.options, ['curveType', 'tension', 'segments', 'closed', 'vertexColors']);
        this.line = viewChild.required(NgtsLine);
        this.closed = pick(this.options, 'closed');
        this.curveType = pick(this.options, 'curveType');
        this.tension = pick(this.options, 'tension');
        this.segments = pick(this.options, 'segments');
        this.vertexColors = pick(this.options, 'vertexColors');
        this.curve = computed(() => {
            const [points, closed, curveType, tension] = [this.points(), this.closed(), this.curveType(), this.tension()];
            const mappedPoints = points.map((pt) => pt instanceof Vector3 ? pt : new Vector3(...pt));
            return new CatmullRomCurve3(mappedPoints, closed, curveType, tension);
        });
        this.segmentedPoints = computed(() => {
            const [curve, segments] = [this.curve(), this.segments()];
            return curve.getPoints(segments);
        });
        this.interpolatedVertexColors = computed(() => {
            const [vertexColors, segments] = [this.vertexColors(), this.segments()];
            if (!vertexColors || vertexColors.length < 2)
                return undefined;
            if (vertexColors.length === segments + 1)
                return vertexColors;
            const mappedColors = vertexColors.map((color) => color instanceof Color ? color : new Color(...color));
            if (untracked(this.closed))
                mappedColors.push(mappedColors[0].clone());
            const iColors = [mappedColors[0]];
            const divisions = segments / (mappedColors.length - 1);
            for (let i = 1; i < segments; i++) {
                const alpha = (i % divisions) / divisions;
                const colorIndex = Math.floor(i / divisions);
                iColors.push(mappedColors[colorIndex].clone().lerp(mappedColors[colorIndex + 1], alpha));
            }
            iColors.push(mappedColors[mappedColors.length - 1]);
            return iColors;
        });
        this.lineOptions = computed(() => ({ ...this.parameters(), vertexColors: this.interpolatedVertexColors() }));
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsCatmullRomLine, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsCatmullRomLine, isStandalone: true, selector: "ngts-catmull-rom-line", inputs: { points: { classPropertyName: "points", publicName: "points", isSignal: true, isRequired: true, transformFunction: null }, options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "line", first: true, predicate: NgtsLine, descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngts-line [points]="segmentedPoints()" [options]="lineOptions()">
			<ng-content />
		</ngts-line>
	`, isInline: true, dependencies: [{ kind: "component", type: NgtsLine, selector: "ngts-line", inputs: ["points", "options"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsCatmullRomLine, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-catmull-rom-line',
                    template: `
		<ngts-line [points]="segmentedPoints()" [options]="lineOptions()">
			<ng-content />
		</ngts-line>
	`,
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    imports: [NgtsLine],
                }]
        }] });

const defaultOptions$8 = {
    lineWidth: 1,
    segments: 20,
};
class NgtsCubicBezierLine {
    constructor() {
        this.start = input.required();
        this.end = input.required();
        this.midA = input.required();
        this.midB = input.required();
        this.options = input(defaultOptions$8, { transform: mergeInputs(defaultOptions$8) });
        this.parameters = omit(this.options, ['segments']);
        this.line = viewChild.required(NgtsLine);
        this.segments = pick(this.options, 'segments');
        this.points = computed(() => {
            const [start, end, midA, midB, segments] = [this.start(), this.end(), this.midA(), this.midB(), this.segments()];
            const startV = start instanceof Vector3 ? start : new Vector3(...start);
            const endV = end instanceof Vector3 ? end : new Vector3(...end);
            const midAV = midA instanceof Vector3 ? midA : new Vector3(...midA);
            const midBV = midB instanceof Vector3 ? midB : new Vector3(...midB);
            return new CubicBezierCurve3(startV, midAV, midBV, endV).getPoints(segments);
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsCubicBezierLine, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsCubicBezierLine, isStandalone: true, selector: "ngts-cubic-bezier-line", inputs: { start: { classPropertyName: "start", publicName: "start", isSignal: true, isRequired: true, transformFunction: null }, end: { classPropertyName: "end", publicName: "end", isSignal: true, isRequired: true, transformFunction: null }, midA: { classPropertyName: "midA", publicName: "midA", isSignal: true, isRequired: true, transformFunction: null }, midB: { classPropertyName: "midB", publicName: "midB", isSignal: true, isRequired: true, transformFunction: null }, options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "line", first: true, predicate: NgtsLine, descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngts-line [points]="points()" [options]="parameters()">
			<ng-content />
		</ngts-line>
	`, isInline: true, dependencies: [{ kind: "component", type: NgtsLine, selector: "ngts-line", inputs: ["points", "options"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsCubicBezierLine, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-cubic-bezier-line',
                    template: `
		<ngts-line [points]="points()" [options]="parameters()">
			<ng-content />
		</ngts-line>
	`,
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    imports: [NgtsLine],
                }]
        }] });

const defaultOptions$7 = {
    lineWidth: 1,
    threshold: 15,
};
class NgtsEdges {
    constructor() {
        this.options = input(defaultOptions$7, { transform: mergeInputs(defaultOptions$7) });
        this.parameters = omit(this.options, ['threshold', 'geometry']);
        this.lineOptions = computed(() => ({ ...this.parameters(), raycast: () => null }));
        this.tmpPoints = [0, 0, 0, 1, 0, 0];
        this.line = viewChild.required(NgtsLine);
        effect(() => {
            const line = this.line().lineRef()?.nativeElement;
            if (!line)
                return;
            const lS = getLocalState(line);
            if (!lS)
                return;
            const parent = lS.parent();
            if (!parent)
                return;
            const { geometry: explicitGeometry, threshold } = this.options();
            const geometry = explicitGeometry ?? parent.geometry;
            if (!geometry)
                return;
            const cached = this.memoizedGeometry === geometry && this.memoizedThreshold === threshold;
            if (cached)
                return;
            this.memoizedGeometry = geometry;
            this.memoizedThreshold = threshold;
            const points = new EdgesGeometry(geometry, threshold).attributes['position']
                .array;
            line.geometry.setPositions(points);
            checkNeedsUpdate(line.geometry.attributes['instanceStart']);
            checkNeedsUpdate(line.geometry.attributes['instanceEnd']);
            line.computeLineDistances();
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsEdges, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsEdges, isStandalone: true, selector: "ngts-edges", inputs: { options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "line", first: true, predicate: NgtsLine, descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngts-line [points]="tmpPoints" [options]="lineOptions()">
			<ng-content />
		</ngts-line>
	`, isInline: true, dependencies: [{ kind: "component", type: NgtsLine, selector: "ngts-line", inputs: ["points", "options"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsEdges, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-edges',
                    template: `
		<ngts-line [points]="tmpPoints" [options]="lineOptions()">
			<ng-content />
		</ngts-line>
	`,
                    imports: [NgtsLine],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }], ctorParameters: () => [] });

const defaultOptions$6 = {
    size: 1024,
    width: 16,
    type: 'linear',
    innerCircleRadius: 0,
    outerCircleRadius: 'auto',
};
class NgtsGradientTexture {
    constructor() {
        this.attach = input('map');
        this.stops = input.required();
        this.colors = input.required();
        this.options = input(defaultOptions$6, { transform: mergeInputs(defaultOptions$6) });
        this.parameters = omit(this.options, ['size', 'width', 'type', 'innerCircleRadius', 'outerCircleRadius']);
        this.store = injectStore();
        this.outputColorSpace = this.store.select('gl', 'outputColorSpace');
        this.document = inject(DOCUMENT);
        this.canvas = computed(() => {
            const canvas = this.document.createElement('canvas');
            const context = canvas.getContext('2d');
            const [{ width, size, type, outerCircleRadius, innerCircleRadius }, stops, colors] = [
                untracked(this.options),
                this.stops(),
                this.colors(),
            ];
            canvas.width = width;
            canvas.height = size;
            let gradient;
            if (type === 'linear') {
                gradient = context.createLinearGradient(0, 0, 0, size);
            }
            else {
                const canvasCenterX = canvas.width / 2;
                const canvasCenterY = canvas.height / 2;
                const radius = outerCircleRadius !== 'auto'
                    ? Math.abs(Number(outerCircleRadius))
                    : Math.sqrt(canvasCenterX ** 2 + canvasCenterY ** 2);
                gradient = context.createRadialGradient(canvasCenterX, canvasCenterY, Math.abs(innerCircleRadius), canvasCenterX, canvasCenterY, radius);
            }
            const tempColor = new Color(); // reuse instance for performance
            let i = stops.length;
            while (i--) {
                gradient.addColorStop(stops[i], tempColor.set(colors[i]).getStyle());
            }
            context.save();
            context.fillStyle = gradient;
            context.fillRect(0, 0, width, size);
            context.restore();
            return canvas;
        });
        extend({ CanvasTexture });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsGradientTexture, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.1.0", version: "19.1.4", type: NgtsGradientTexture, isStandalone: true, selector: "ngts-gradient-texture", inputs: { attach: { classPropertyName: "attach", publicName: "attach", isSignal: true, isRequired: false, transformFunction: null }, stops: { classPropertyName: "stops", publicName: "stops", isSignal: true, isRequired: true, transformFunction: null }, colors: { classPropertyName: "colors", publicName: "colors", isSignal: true, isRequired: true, transformFunction: null }, options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, ngImport: i0, template: `
		<ngt-canvas-texture
			*args="[canvas()]"
			[attach]="attach()"
			[colorSpace]="outputColorSpace()"
			[parameters]="parameters()"
		>
			<ng-content />
		</ngt-canvas-texture>
	`, isInline: true, dependencies: [{ kind: "directive", type: NgtArgs, selector: "ng-template[args]", inputs: ["args"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsGradientTexture, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-gradient-texture',
                    template: `
		<ngt-canvas-texture
			*args="[canvas()]"
			[attach]="attach()"
			[colorSpace]="outputColorSpace()"
			[parameters]="parameters()"
		>
			<ng-content />
		</ngt-canvas-texture>
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    imports: [NgtArgs],
                }]
        }], ctorParameters: () => [] });

const defaultOptions$5 = {
    planeArgs: [],
    cellSize: 0.5,
    cellThickness: 0.5,
    cellColor: 'black',
    sectionSize: 1,
    sectionThickness: 1,
    sectionColor: '#2080ff',
    infiniteGrid: false,
    followCamera: false,
    fadeDistance: 100,
    fadeStrength: 1,
    fadeFrom: 1,
    side: BackSide,
};
class NgtsGrid {
    constructor() {
        this.options = input(defaultOptions$5, { transform: mergeInputs(defaultOptions$5) });
        this.parameters = omit(this.options, [
            'planeArgs',
            'cellSize',
            'cellThickness',
            'cellColor',
            'sectionSize',
            'sectionThickness',
            'sectionColor',
            'infiniteGrid',
            'followCamera',
            'fadeDistance',
            'fadeStrength',
            'fadeFrom',
            'side',
        ]);
        this.meshRef = viewChild.required('mesh');
        this.plane = new Plane();
        this.upVector = new Vector3(0, 1, 0);
        this.zeroVector = new Vector3(0, 0, 0);
        this.planeArgs = pick(this.options, 'planeArgs');
        this.side = pick(this.options, 'side');
        this.uniforms = computed(() => {
            const { cellSize, sectionSize, cellColor, sectionColor, cellThickness, sectionThickness, fadeDistance, fadeStrength, fadeFrom, infiniteGrid, followCamera, } = this.options();
            return {
                cellSize,
                sectionSize,
                cellColor,
                sectionColor,
                cellThickness,
                sectionThickness,
                fadeDistance,
                fadeStrength,
                fadeFrom,
                infiniteGrid,
                followCamera,
            };
        });
        extend({ Mesh, PlaneGeometry, GridMaterial });
        injectBeforeRender(({ camera }) => {
            const mesh = this.meshRef().nativeElement;
            this.plane.setFromNormalAndCoplanarPoint(this.upVector, this.zeroVector).applyMatrix4(mesh.matrixWorld);
            const gridMaterial = mesh.material;
            const worldCamProjPosition = gridMaterial.uniforms['worldCamProjPosition'];
            const worldPlanePosition = gridMaterial.uniforms['worldPlanePosition'];
            this.plane.projectPoint(camera.position, worldCamProjPosition.value);
            worldPlanePosition.value.set(0, 0, 0).applyMatrix4(mesh.matrixWorld);
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsGrid, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsGrid, isStandalone: true, selector: "ngts-grid", inputs: { options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "meshRef", first: true, predicate: ["mesh"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-mesh #mesh [frustumCulled]="false" [parameters]="parameters()">
			<ngt-plane-geometry *args="planeArgs()" />
			<ngt-grid-material [transparent]="true" [side]="side()" [parameters]="uniforms()">
				<ngt-value attach="extensions.derivatives" [rawValue]="true" />
			</ngt-grid-material>
			<ng-content />
		</ngt-mesh>
	`, isInline: true, dependencies: [{ kind: "directive", type: NgtArgs, selector: "ng-template[args]", inputs: ["args"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsGrid, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-grid',
                    template: `
		<ngt-mesh #mesh [frustumCulled]="false" [parameters]="parameters()">
			<ngt-plane-geometry *args="planeArgs()" />
			<ngt-grid-material [transparent]="true" [side]="side()" [parameters]="uniforms()">
				<ngt-value attach="extensions.derivatives" [rawValue]="true" />
			</ngt-grid-material>
			<ng-content />
		</ngt-mesh>
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    imports: [NgtArgs],
                }]
        }], ctorParameters: () => [] });

function injectHelper(object, helperConstructor, { injector, args = () => [], } = {}) {
    return assertInjector(injectHelper, injector, () => {
        const store = injectStore();
        const scene = store.select('scene');
        const helper = computed(() => {
            const maybeObject3D = object();
            if (!maybeObject3D)
                return null;
            const object3D = resolveRef(maybeObject3D);
            if (!object3D)
                return null;
            const currentHelper = new (helperConstructor())(object3D, ...args());
            currentHelper.traverse((child) => (child.raycast = () => null));
            return currentHelper;
        });
        effect((onCleanup) => {
            const currentHelper = helper();
            if (!currentHelper)
                return;
            const _scene = scene();
            _scene.add(currentHelper);
            onCleanup(() => {
                _scene.remove(currentHelper);
                currentHelper.dispose?.();
            });
        });
        injectBeforeRender(() => {
            const currentHelper = helper();
            if (currentHelper)
                currentHelper.update();
        });
        return helper;
    });
}
class NgtsHelper {
    constructor() {
        this.type = input.required();
        this.options = input([]);
        this.helperRef = viewChild.required('helper');
        this.parent = computed(() => {
            const helper = this.helperRef().nativeElement;
            if (!helper)
                return;
            const localState = getLocalState(helper);
            if (!localState)
                return;
            return localState.parent();
        });
        this.helper = injectHelper(this.parent, this.type, { args: this.options });
        extend({ Object3D });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsHelper, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsHelper, isStandalone: true, selector: "ngts-helper", inputs: { type: { classPropertyName: "type", publicName: "type", isSignal: true, isRequired: true, transformFunction: null }, options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "helperRef", first: true, predicate: ["helper"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-object3D #helper />
	`, isInline: true, changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsHelper, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-helper',
                    template: `
		<ngt-object3D #helper />
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }], ctorParameters: () => [] });

const defaultOptions$4 = {
    height: 1,
    bevelEnabled: false,
};
class NgtsPrismGeometry {
    constructor() {
        this.attach = input('geometry');
        this.vertices = input.required();
        this.options = input(defaultOptions$4, { transform: mergeInputs(defaultOptions$4) });
        this.parameters = computed(() => ({ ...this.options(), depth: this.options().height }));
        this.shape = computed(() => {
            const vertices = this.vertices();
            const interpolatedVertices = vertices.map((v) => (v instanceof Vector2 ? v : new Vector2(...v)));
            return new Shape(interpolatedVertices);
        });
        this.geometryRef = viewChild('geometry');
        extend({ ExtrudeGeometry });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsPrismGeometry, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsPrismGeometry, isStandalone: true, selector: "ngts-prism-geometry", inputs: { attach: { classPropertyName: "attach", publicName: "attach", isSignal: true, isRequired: false, transformFunction: null }, vertices: { classPropertyName: "vertices", publicName: "vertices", isSignal: true, isRequired: true, transformFunction: null }, options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "geometryRef", first: true, predicate: ["geometry"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-extrude-geometry #geometry *args="[shape(), parameters()]" [attach]="attach()">
			<ng-content />
		</ngt-extrude-geometry>
	`, isInline: true, dependencies: [{ kind: "directive", type: NgtArgs, selector: "ng-template[args]", inputs: ["args"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsPrismGeometry, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-prism-geometry',
                    template: `
		<ngt-extrude-geometry #geometry *args="[shape(), parameters()]" [attach]="attach()">
			<ng-content />
		</ngt-extrude-geometry>
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    imports: [NgtArgs],
                }]
        }], ctorParameters: () => [] });

const defaultOptions$3 = {
    lineWidth: 1,
    segments: 20,
};
class NgtsQuadraticBezierLine {
    constructor() {
        this.start = input([0, 0, 0]);
        this.end = input([0, 0, 0]);
        this.mid = input();
        this.options = input(defaultOptions$3, { transform: mergeInputs(defaultOptions$3) });
        this.parameters = omit(this.options, ['segments']);
        this.segments = pick(this.options, 'segments');
        this.line = viewChild.required(NgtsLine);
        this.points = computed(() => this.getPoints(this.start(), this.end(), this.mid(), this.segments()));
        this.curve = new QuadraticBezierCurve3();
    }
    setPoints(start, end, mid) {
        const points = this.getPoints(start, end, mid);
        const geometry = this.line().lineGeometry();
        if (geometry)
            geometry.setPositions(points.map((p) => p.toArray()).flat());
    }
    getPoints(start, end, mid, segments = 20) {
        if (start instanceof Vector3)
            this.curve.v0.copy(start);
        else
            this.curve.v0.set(...start);
        if (end instanceof Vector3)
            this.curve.v2.copy(end);
        else
            this.curve.v2.set(...end);
        if (mid instanceof Vector3) {
            this.curve.v1.copy(mid);
        }
        else if (Array.isArray(mid)) {
            this.curve.v1.set(...mid);
        }
        else {
            this.curve.v1.copy(this.curve.v0
                .clone()
                .add(this.curve.v2.clone().sub(this.curve.v0))
                .add(new Vector3(0, this.curve.v0.y - this.curve.v2.y, 0)));
        }
        return this.curve.getPoints(segments);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsQuadraticBezierLine, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsQuadraticBezierLine, isStandalone: true, selector: "ngts-quadratic-bezier-line", inputs: { start: { classPropertyName: "start", publicName: "start", isSignal: true, isRequired: false, transformFunction: null }, end: { classPropertyName: "end", publicName: "end", isSignal: true, isRequired: false, transformFunction: null }, mid: { classPropertyName: "mid", publicName: "mid", isSignal: true, isRequired: false, transformFunction: null }, options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "line", first: true, predicate: NgtsLine, descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngts-line [points]="points()" [options]="parameters()">
			<ng-content />
		</ngts-line>
	`, isInline: true, dependencies: [{ kind: "component", type: NgtsLine, selector: "ngts-line", inputs: ["points", "options"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsQuadraticBezierLine, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-quadratic-bezier-line',
                    template: `
		<ngts-line [points]="points()" [options]="parameters()">
			<ng-content />
		</ngts-line>
	`,
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    imports: [NgtsLine],
                }]
        }] });

const eps = 0.00001;
function createShape(width, height, radius0) {
    const shape = new Shape();
    const radius = radius0 - eps;
    shape.absarc(eps, eps, eps, -Math.PI / 2, -Math.PI, true);
    shape.absarc(eps, height - radius * 2, eps, Math.PI, Math.PI / 2, true);
    shape.absarc(width - radius * 2, height - radius * 2, eps, Math.PI / 2, 0, true);
    shape.absarc(width - radius * 2, eps, eps, 0, -Math.PI / 2, true);
    return shape;
}
const defaultOptions$2 = {
    width: 1,
    height: 1,
    depth: 1,
    radius: 0.05,
    smoothness: 4,
    bevelSegments: 4,
    creaseAngle: 0.4,
    steps: 1,
};
class NgtsRoundedBox {
    constructor() {
        this.options = input(defaultOptions$2, { transform: mergeInputs(defaultOptions$2) });
        this.parameters = omit(this.options, [
            'width',
            'height',
            'depth',
            'radius',
            'smoothness',
            'bevelSegments',
            'steps',
            'creaseAngle',
        ]);
        this.width = pick(this.options, 'width');
        this.height = pick(this.options, 'height');
        this.depth = pick(this.options, 'depth');
        this.radius = pick(this.options, 'radius');
        this.smoothness = pick(this.options, 'smoothness');
        this.bevelSegments = pick(this.options, 'bevelSegments');
        this.steps = pick(this.options, 'steps');
        this.creaseAngle = pick(this.options, 'creaseAngle');
        this.meshRef = viewChild.required('mesh');
        this.geometryRef = viewChild('geometry');
        this.shape = computed(() => {
            const [width, height, radius] = [this.width(), this.height(), this.radius()];
            return createShape(width, height, radius);
        });
        this.params = computed(() => {
            const [depth, radius, smoothness, bevelSegments, steps] = [
                this.depth(),
                this.radius(),
                this.smoothness(),
                untracked(this.bevelSegments),
                untracked(this.steps),
            ];
            return {
                depth: depth - radius * 2,
                bevelEnabled: true,
                bevelSegments: bevelSegments * 2,
                steps,
                bevelSize: radius - eps,
                bevelThickness: radius,
                curveSegments: smoothness,
            };
        });
        extend({ ExtrudeGeometry, Mesh });
        const objectEvents = inject(NgtObjectEvents, { host: true });
        effect(() => {
            const mesh = this.meshRef().nativeElement;
            objectEvents.ngtObjectEvents.set(mesh);
        });
        effect(() => {
            const geometry = this.geometryRef()?.nativeElement;
            if (!geometry)
                return;
            geometry.center();
            toCreasedNormals(geometry, untracked(this.creaseAngle));
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsRoundedBox, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsRoundedBox, isStandalone: true, selector: "ngts-rounded-box", inputs: { options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "meshRef", first: true, predicate: ["mesh"], descendants: true, isSignal: true }, { propertyName: "geometryRef", first: true, predicate: ["geometry"], descendants: true, isSignal: true }], hostDirectives: [{ directive: i1.NgtObjectEvents, outputs: ["click", "click", "dblclick", "dblclick", "contextmenu", "contextmenu", "pointerup", "pointerup", "pointerdown", "pointerdown", "pointerover", "pointerover", "pointerout", "pointerout", "pointerenter", "pointerenter", "pointerleave", "pointerleave", "pointermove", "pointermove", "pointermissed", "pointermissed", "pointercancel", "pointercancel", "wheel", "wheel"] }], ngImport: i0, template: `
		<ngt-mesh #mesh [parameters]="parameters()">
			<ngt-extrude-geometry #geometry *args="[shape(), params()]" />
			<ng-content />
		</ngt-mesh>
	`, isInline: true, dependencies: [{ kind: "directive", type: NgtArgs, selector: "ng-template[args]", inputs: ["args"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsRoundedBox, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-rounded-box',
                    template: `
		<ngt-mesh #mesh [parameters]="parameters()">
			<ngt-extrude-geometry #geometry *args="[shape(), params()]" />
			<ng-content />
		</ngt-mesh>
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    imports: [NgtArgs],
                    hostDirectives: [
                        {
                            directive: NgtObjectEvents,
                            outputs: [
                                'click',
                                'dblclick',
                                'contextmenu',
                                'pointerup',
                                'pointerdown',
                                'pointerover',
                                'pointerout',
                                'pointerenter',
                                'pointerleave',
                                'pointermove',
                                'pointermissed',
                                'pointercancel',
                                'wheel',
                            ],
                        },
                    ],
                }]
        }], ctorParameters: () => [] });

const defaultOptions$1 = {
    sdfGlyphSize: 64,
    anchorX: 'center',
    anchorY: 'middle',
    fontSize: 1,
};
class NgtsText {
    constructor() {
        this.text = input.required();
        this.options = input(defaultOptions$1, { transform: mergeInputs(defaultOptions$1) });
        this.parameters = omit(this.options, ['font', 'fontSize', 'sdfGlyphSize', 'anchorX', 'anchorY', 'characters']);
        this.synced = output();
        this.store = injectStore();
        this.invalidate = this.store.select('invalidate');
        this.characters = pick(this.options, 'characters');
        this.font = pick(this.options, 'font');
        this.anchorX = pick(this.options, 'anchorX');
        this.anchorY = pick(this.options, 'anchorY');
        this.sdfGlyphSize = pick(this.options, 'sdfGlyphSize');
        this.fontSize = pick(this.options, 'fontSize');
        this.troikaMesh = new Text();
        inject(DestroyRef).onDestroy(() => {
            this.troikaMesh.dispose();
        });
        effect(() => {
            const [font, characters, invalidate] = [this.font(), this.characters(), this.invalidate()];
            if (font) {
                preloadFont({ font, characters }, () => invalidate());
            }
        });
        effect(() => {
            const [invalidate] = [this.invalidate(), this.text(), this.options()];
            this.troikaMesh.sync(() => {
                invalidate();
                this.synced.emit(this.troikaMesh);
            });
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsText, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.1.0", version: "19.1.4", type: NgtsText, isStandalone: true, selector: "ngts-text", inputs: { text: { classPropertyName: "text", publicName: "text", isSignal: true, isRequired: true, transformFunction: null }, options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, outputs: { synced: "synced" }, ngImport: i0, template: `
		<ngt-primitive
			*args="[troikaMesh]"
			[text]="text()"
			[font]="font()"
			[anchorX]="anchorX()"
			[anchorY]="anchorY()"
			[sdfGlyphSize]="sdfGlyphSize()"
			[fontSize]="fontSize()"
			[parameters]="parameters()"
		>
			<ng-content />
		</ngt-primitive>
	`, isInline: true, dependencies: [{ kind: "directive", type: NgtArgs, selector: "ng-template[args]", inputs: ["args"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsText, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-text',
                    template: `
		<ngt-primitive
			*args="[troikaMesh]"
			[text]="text()"
			[font]="font()"
			[anchorX]="anchorX()"
			[anchorY]="anchorY()"
			[sdfGlyphSize]="sdfGlyphSize()"
			[fontSize]="fontSize()"
			[parameters]="parameters()"
		>
			<ng-content />
		</ngt-primitive>
	`,
                    imports: [NgtArgs],
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }], ctorParameters: () => [] });

const defaultOptions = {
    letterSpacing: 0,
    lineHeight: 1,
    size: 1,
    height: 0.2,
    bevelThickness: 0.1,
    bevelSize: 0.01,
    bevelEnabled: false,
    bevelOffset: 0,
    bevelSegments: 4,
    curveSegments: 8,
};
class NgtsText3D {
    constructor() {
        this.font = input.required();
        this.text = input.required();
        this.options = input(defaultOptions, { transform: mergeInputs(defaultOptions) });
        this.parameters = omit(this.options, [
            'letterSpacing',
            'lineHeight',
            'size',
            'height',
            'bevelThickness',
            'bevelSize',
            'bevelEnabled',
            'bevelOffset',
            'bevelSegments',
            'curveSegments',
            'smooth',
        ]);
        this.meshRef = viewChild.required('mesh');
        this.textGeometryRef = viewChild('textGeometry');
        this.loadedFont = injectFont(this.font);
        this.smooth = pick(this.options, 'smooth');
        this.textOptions = pick(this.options, [
            'letterSpacing',
            'lineHeight',
            'size',
            'height',
            'bevelThickness',
            'bevelSize',
            'bevelEnabled',
            'bevelOffset',
            'bevelSegments',
            'curveSegments',
        ]);
        this.textArgs = computed(() => {
            const [text, font, textOptions] = [this.text(), this.loadedFont(), this.textOptions()];
            if (!font)
                return null;
            return [text, { font, ...textOptions }];
        });
        extend({ Mesh, TextGeometry });
        effect(() => {
            const [mesh, textGeometry, textArgs] = [
                this.meshRef()?.nativeElement,
                this.textGeometryRef()?.nativeElement,
                this.textArgs(),
            ];
            if (!textArgs || !textGeometry || !mesh)
                return;
            const smooth = this.smooth();
            if (smooth) {
                mesh.geometry = mergeVertices(textGeometry, smooth);
                mesh.geometry.computeVertexNormals();
            }
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsText3D, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsText3D, isStandalone: true, selector: "ngts-text-3d", inputs: { font: { classPropertyName: "font", publicName: "font", isSignal: true, isRequired: true, transformFunction: null }, text: { classPropertyName: "text", publicName: "text", isSignal: true, isRequired: true, transformFunction: null }, options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "meshRef", first: true, predicate: ["mesh"], descendants: true, isSignal: true }, { propertyName: "textGeometryRef", first: true, predicate: ["textGeometry"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-mesh #mesh [parameters]="parameters()">
			<ngt-text-geometry #textGeometry *args="textArgs()" />
			<ng-content />
		</ngt-mesh>
	`, isInline: true, dependencies: [{ kind: "directive", type: NgtArgs, selector: "ng-template[args]", inputs: ["args"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsText3D, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-text-3d',
                    template: `
		<ngt-mesh #mesh [parameters]="parameters()">
			<ngt-text-geometry #textGeometry *args="textArgs()" />
			<ng-content />
		</ngt-mesh>
	`,
                    imports: [NgtArgs],
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }], ctorParameters: () => [] });

/**
 * Generated bundle index. Do not edit.
 */

export { NgtsBillboard, NgtsCatmullRomLine, NgtsCubicBezierLine, NgtsEdges, NgtsGradientTexture, NgtsGrid, NgtsHelper, NgtsLine, NgtsPrismGeometry, NgtsQuadraticBezierLine, NgtsRoundedBox, NgtsText, NgtsText3D, injectHelper };
//# sourceMappingURL=angular-three-soba-abstractions.mjs.map
