import * as i0 from '@angular/core';
import { Directive, computed, effect, untracked, input, viewChild, contentChild, TemplateRef, Component, CUSTOM_ELEMENTS_SCHEMA, ChangeDetectionStrategy } from '@angular/core';
import { NgTemplateOutlet } from '@angular/common';
import { injectStore, merge, pick, omit, extend, injectBeforeRender, NgtArgs } from 'angular-three';
import { assertInjector } from 'ngxtension/assert-injector';
import { mergeInputs } from 'ngxtension/inject-inputs';
import { WebGLCubeRenderTarget, HalfFloatType, CubeCamera, Group, OrthographicCamera, PerspectiveCamera } from 'three';
import { injectFBO } from 'angular-three-soba/misc';

class NgtsCameraContent {
    static ngTemplateContextGuard(_, ctx) {
        return true;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsCameraContent, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.1.4", type: NgtsCameraContent, isStandalone: true, selector: "ng-template[cameraContent]", ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsCameraContent, decorators: [{
            type: Directive,
            args: [{ selector: 'ng-template[cameraContent]' }]
        }] });

function injectCubeCamera(options, { injector } = {}) {
    return assertInjector(injectCubeCamera, injector, () => {
        const store = injectStore();
        const gl = store.select('gl');
        const scene = store.select('scene');
        // backfill the options with default values
        const mergedOptions = merge(options, { resolution: 256, near: 0.1, far: 1000 }, 'backfill');
        const resolution = pick(mergedOptions, 'resolution');
        const near = pick(mergedOptions, 'near');
        const far = pick(mergedOptions, 'far');
        const fbo = computed(() => {
            const fbo = new WebGLCubeRenderTarget(resolution());
            fbo.texture.type = HalfFloatType;
            return fbo;
        });
        effect((onCleanup) => {
            const _fbo = fbo();
            onCleanup(() => _fbo.dispose());
        });
        const camera = computed(() => new CubeCamera(near(), far(), fbo()));
        const update = () => {
            const [_scene, _gl, _camera, { envMap, fog }] = [scene(), gl(), camera(), untracked(mergedOptions)];
            let originalFog;
            let originalBackground;
            originalFog = _scene.fog;
            originalBackground = _scene.background;
            _scene.background = envMap || originalBackground;
            _scene.fog = fog || originalFog;
            _camera.update(_gl, _scene);
            _scene.fog = originalFog;
            _scene.background = originalBackground;
        };
        return { fbo, camera, update };
    });
}
const defaultOptions$2 = {
    frames: Infinity,
    resolution: 256,
    near: 0.1,
    far: 1000,
};
class NgtsCubeCamera {
    constructor() {
        this.options = input(defaultOptions$2, { transform: mergeInputs(defaultOptions$2) });
        this.parameters = omit(this.options, ['fog', 'near', 'far', 'envMap', 'resolution', 'frames']);
        this.cubeCamera = injectCubeCamera(pick(this.options, ['near', 'far', 'envMap', 'fog', 'resolution']));
        this.camera = this.cubeCamera.camera;
        this.texture = pick(this.cubeCamera.fbo, 'texture');
        this.groupRef = viewChild.required('group');
        this.cameraContent = contentChild(NgtsCameraContent, { read: TemplateRef });
        extend({ Group });
        let count = 0;
        injectBeforeRender(() => {
            const group = this.groupRef().nativeElement;
            if (!group)
                return;
            const frames = this.options().frames;
            if (frames === Infinity || count < frames) {
                group.visible = false;
                this.cubeCamera.update();
                group.visible = true;
                count++;
            }
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsCubeCamera, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsCubeCamera, isStandalone: true, selector: "ngts-cube-camera", inputs: { options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, queries: [{ propertyName: "cameraContent", first: true, predicate: NgtsCameraContent, descendants: true, read: TemplateRef, isSignal: true }], viewQueries: [{ propertyName: "groupRef", first: true, predicate: ["group"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-group [parameters]="parameters()">
			<ngt-primitive *args="[camera()]" />
			<ngt-group #group>
				<ng-container [ngTemplateOutlet]="cameraContent() ?? null" [ngTemplateOutletContext]="{ $implicit: texture }" />
			</ngt-group>
		</ngt-group>
	`, isInline: true, dependencies: [{ kind: "directive", type: NgtArgs, selector: "ng-template[args]", inputs: ["args"] }, { kind: "directive", type: NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet", "ngTemplateOutletInjector"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsCubeCamera, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-cube-camera',
                    template: `
		<ngt-group [parameters]="parameters()">
			<ngt-primitive *args="[camera()]" />
			<ngt-group #group>
				<ng-container [ngTemplateOutlet]="cameraContent() ?? null" [ngTemplateOutletContext]="{ $implicit: texture }" />
			</ngt-group>
		</ngt-group>
	`,
                    imports: [NgtArgs, NgTemplateOutlet],
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }], ctorParameters: () => [] });

const defaultOptions$1 = {
    frames: Infinity,
    resolution: 256,
    makeDefault: false,
    manual: false,
};
class NgtsOrthographicCamera {
    constructor() {
        this.options = input(defaultOptions$1, { transform: mergeInputs(defaultOptions$1) });
        this.parameters = omit(this.options, [
            'envMap',
            'makeDefault',
            'manual',
            'frames',
            'resolution',
            'left',
            'top',
            'bottom',
            'right',
        ]);
        this.content = contentChild(TemplateRef);
        this.cameraContent = contentChild(NgtsCameraContent, { read: TemplateRef });
        this.cameraRef = viewChild.required('camera');
        this.groupRef = viewChild.required('group');
        this.store = injectStore();
        this.camera = this.store.select('camera');
        this.size = this.store.select('size');
        this.left = computed(() => this.options().left ?? this.size().width / -2);
        this.right = computed(() => this.options().right ?? this.size().width / 2);
        this.top = computed(() => this.options().top ?? this.size().height / 2);
        this.bottom = computed(() => this.options().bottom ?? this.size().height / -2);
        this.manual = pick(this.options, 'manual');
        this.makeDefault = pick(this.options, 'makeDefault');
        this.resolution = pick(this.options, 'resolution');
        this.fbo = injectFBO(() => ({ width: this.resolution() }));
        this.texture = pick(this.fbo, 'texture');
        extend({ OrthographicCamera, Group });
        effect(() => {
            this.cameraRef().nativeElement.updateProjectionMatrix();
        });
        effect(() => {
            const manual = this.manual();
            if (manual)
                return;
            const camera = this.cameraRef().nativeElement;
            camera.updateProjectionMatrix();
        });
        effect((onCleanup) => {
            const makeDefault = this.makeDefault();
            if (!makeDefault)
                return;
            const oldCam = untracked(this.camera);
            this.store.update({ camera: this.cameraRef().nativeElement });
            onCleanup(() => this.store.update(() => ({ camera: oldCam })));
        });
        let count = 0;
        let oldEnvMap = null;
        injectBeforeRender(({ gl, scene }) => {
            const [{ frames, envMap }, group, camera, fbo] = [
                this.options(),
                this.groupRef().nativeElement,
                this.cameraRef().nativeElement,
                this.fbo(),
            ];
            if (this.cameraContent() && group && camera && fbo && (frames === Infinity || count < frames)) {
                group.visible = false;
                gl.setRenderTarget(fbo);
                oldEnvMap = scene.background;
                if (envMap)
                    scene.background = envMap;
                gl.render(scene, camera);
                scene.background = oldEnvMap;
                gl.setRenderTarget(null);
                group.visible = true;
                count++;
            }
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsOrthographicCamera, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsOrthographicCamera, isStandalone: true, selector: "ngts-orthographic-camera", inputs: { options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, queries: [{ propertyName: "content", first: true, predicate: TemplateRef, descendants: true, isSignal: true }, { propertyName: "cameraContent", first: true, predicate: NgtsCameraContent, descendants: true, read: TemplateRef, isSignal: true }], viewQueries: [{ propertyName: "cameraRef", first: true, predicate: ["camera"], descendants: true, isSignal: true }, { propertyName: "groupRef", first: true, predicate: ["group"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-orthographic-camera
			#camera
			[left]="left()"
			[right]="right()"
			[top]="top()"
			[bottom]="bottom()"
			[parameters]="parameters()"
		>
			<ng-container [ngTemplateOutlet]="content() ?? null" />
		</ngt-orthographic-camera>

		<ngt-group #group>
			<ng-container [ngTemplateOutlet]="cameraContent() ?? null" [ngTemplateOutletContext]="{ $implicit: texture }" />
		</ngt-group>
	`, isInline: true, dependencies: [{ kind: "directive", type: NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet", "ngTemplateOutletInjector"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsOrthographicCamera, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-orthographic-camera',
                    template: `
		<ngt-orthographic-camera
			#camera
			[left]="left()"
			[right]="right()"
			[top]="top()"
			[bottom]="bottom()"
			[parameters]="parameters()"
		>
			<ng-container [ngTemplateOutlet]="content() ?? null" />
		</ngt-orthographic-camera>

		<ngt-group #group>
			<ng-container [ngTemplateOutlet]="cameraContent() ?? null" [ngTemplateOutletContext]="{ $implicit: texture }" />
		</ngt-group>
	`,
                    imports: [NgTemplateOutlet],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                }]
        }], ctorParameters: () => [] });

const defaultOptions = {
    frames: Infinity,
    resolution: 256,
    makeDefault: false,
    manual: false,
};
class NgtsPerspectiveCamera {
    constructor() {
        this.options = input(defaultOptions, { transform: mergeInputs(defaultOptions) });
        this.parameters = omit(this.options, ['envMap', 'makeDefault', 'manual', 'frames', 'resolution']);
        this.content = contentChild(TemplateRef);
        this.cameraContent = contentChild(NgtsCameraContent, { read: TemplateRef });
        this.cameraRef = viewChild.required('camera');
        this.groupRef = viewChild.required('group');
        this.store = injectStore();
        this.camera = this.store.select('camera');
        this.size = this.store.select('size');
        this.manual = pick(this.options, 'manual');
        this.makeDefault = pick(this.options, 'makeDefault');
        this.resolution = pick(this.options, 'resolution');
        this.fbo = injectFBO(() => ({ width: this.resolution() }));
        this.texture = pick(this.fbo, 'texture');
        extend({ PerspectiveCamera, Group });
        effect(() => {
            this.cameraRef().nativeElement.updateProjectionMatrix();
        });
        effect(() => {
            const manual = this.manual();
            if (manual)
                return;
            const [camera, size] = [this.cameraRef().nativeElement, this.size()];
            camera.aspect = size.width / size.height;
            camera.updateProjectionMatrix();
        });
        effect((onCleanup) => {
            const makeDefault = this.makeDefault();
            if (!makeDefault)
                return;
            const oldCam = untracked(this.camera);
            this.store.update({ camera: this.cameraRef().nativeElement });
            onCleanup(() => this.store.update(() => ({ camera: oldCam })));
        });
        /**/
        let count = 0;
        let oldEnvMap = null;
        injectBeforeRender(({ gl, scene }) => {
            const [{ frames, envMap }, group, camera, fbo] = [
                this.options(),
                this.groupRef().nativeElement,
                this.cameraRef().nativeElement,
                this.fbo(),
            ];
            if (this.cameraContent() && group && camera && fbo && (frames === Infinity || count < frames)) {
                group.visible = false;
                gl.setRenderTarget(fbo);
                oldEnvMap = scene.background;
                if (envMap)
                    scene.background = envMap;
                gl.render(scene, camera);
                scene.background = oldEnvMap;
                gl.setRenderTarget(null);
                group.visible = true;
                count++;
            }
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsPerspectiveCamera, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsPerspectiveCamera, isStandalone: true, selector: "ngts-perspective-camera", inputs: { options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, queries: [{ propertyName: "content", first: true, predicate: TemplateRef, descendants: true, isSignal: true }, { propertyName: "cameraContent", first: true, predicate: NgtsCameraContent, descendants: true, read: TemplateRef, isSignal: true }], viewQueries: [{ propertyName: "cameraRef", first: true, predicate: ["camera"], descendants: true, isSignal: true }, { propertyName: "groupRef", first: true, predicate: ["group"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-perspective-camera #camera [parameters]="parameters()">
			<ng-container [ngTemplateOutlet]="content() ?? null" />
		</ngt-perspective-camera>

		<ngt-group #group>
			<ng-container [ngTemplateOutlet]="cameraContent() ?? null" [ngTemplateOutletContext]="{ $implicit: texture }" />
		</ngt-group>
	`, isInline: true, dependencies: [{ kind: "directive", type: NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet", "ngTemplateOutletInjector"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsPerspectiveCamera, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-perspective-camera',
                    template: `
		<ngt-perspective-camera #camera [parameters]="parameters()">
			<ng-container [ngTemplateOutlet]="content() ?? null" />
		</ngt-perspective-camera>

		<ngt-group #group>
			<ng-container [ngTemplateOutlet]="cameraContent() ?? null" [ngTemplateOutletContext]="{ $implicit: texture }" />
		</ngt-group>
	`,
                    imports: [NgTemplateOutlet],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                }]
        }], ctorParameters: () => [] });

/**
 * Generated bundle index. Do not edit.
 */

export { NgtsCameraContent, NgtsCubeCamera, NgtsOrthographicCamera, NgtsPerspectiveCamera, injectCubeCamera };
//# sourceMappingURL=angular-three-soba-cameras.mjs.map
