import * as i0 from '@angular/core';
import { input, output, computed, effect, Component, CUSTOM_ELEMENTS_SCHEMA, ChangeDetectionStrategy, model, inject, untracked, ElementRef, Directive } from '@angular/core';
import { omit, injectStore, pick, injectBeforeRender, NgtArgs, NgtHTML, provideHTMLDomElement } from 'angular-three';
import CameraControls from 'camera-controls';
import { mergeInputs } from 'ngxtension/inject-inputs';
import { Box3, MathUtils, Matrix4, Quaternion, Raycaster, Sphere, Spherical, Vector2, Vector3, Vector4 } from 'three';
import { OrbitControls } from 'three-stdlib';
import { DOCUMENT } from '@angular/common';
import { easing } from 'maath';

const defaultOptions$2 = {
    makeDefault: false,
    events: false,
    regress: false,
};
class NgtsCameraControls {
    constructor() {
        this.options = input(defaultOptions$2, { transform: mergeInputs(defaultOptions$2) });
        this.parameters = omit(this.options, ['makeDefault', 'camera', 'regress', 'domElement']);
        this.changed = output();
        this.started = output();
        this.ended = output();
        this.store = injectStore();
        this.invalidate = this.store.select('invalidate');
        this.performanceRegress = this.store.select('performance', 'regress');
        this.defaultCamera = this.store.select('camera');
        this.glDomElement = this.store.select('gl', 'domElement');
        this.eventsElement = this.store.select('events', 'connected');
        this.camera = pick(this.options, 'camera');
        this.regress = pick(this.options, 'regress');
        this.domElement = pick(this.options, 'domElement');
        this.makeDefault = pick(this.options, 'makeDefault');
        this.controls = computed(() => {
            const [camera, defaultCamera] = [this.camera(), this.defaultCamera()];
            return new CameraControls(camera || defaultCamera);
        });
        // to allow for tree shaking, we only import the subset of THREE that is used by camera-controls
        // see https://github.com/yomotsu/camera-controls#important
        CameraControls.install({
            THREE: {
                Box3,
                MathUtils: { clamp: MathUtils.clamp },
                Matrix4,
                Quaternion,
                Raycaster,
                Sphere,
                Spherical,
                Vector2,
                Vector3,
                Vector4,
            },
        });
        effect((onCleanup) => {
            const makeDefault = this.makeDefault();
            if (!makeDefault)
                return;
            const controls = this.controls();
            const oldControls = this.store.snapshot.controls;
            this.store.update({ controls: controls });
            onCleanup(() => void this.store.update({ controls: oldControls }));
        });
        injectBeforeRender(({ delta }) => {
            const controls = this.controls();
            if (controls?.enabled) {
                controls.update(delta);
            }
        }, { priority: -1 });
        effect((onCleanup) => {
            const [domElement, eventsElement, glDomElement, controls] = [
                this.domElement(),
                this.eventsElement(),
                this.glDomElement(),
                this.controls(),
            ];
            controls.connect(domElement || eventsElement || glDomElement);
            onCleanup(() => controls.disconnect());
        });
        effect((onCleanup) => {
            const controls = this.controls();
            if (!controls)
                return;
            const [regress, performanceRegress, invalidate] = [this.regress(), this.performanceRegress(), this.invalidate()];
            const callback = (e) => {
                invalidate();
                if (regress)
                    performanceRegress();
                this.changed.emit(e);
            };
            const startCallback = this.started.emit.bind(this.started);
            const endCallback = this.ended.emit.bind(this.ended);
            controls.addEventListener('update', callback);
            controls.addEventListener('controlstart', startCallback);
            controls.addEventListener('controlend', endCallback);
            controls.addEventListener('control', callback);
            controls.addEventListener('transitionstart', callback);
            controls.addEventListener('wake', callback);
            onCleanup(() => {
                controls.removeEventListener('update', callback);
                controls.removeEventListener('controlstart', startCallback);
                controls.removeEventListener('controlend', endCallback);
                controls.removeEventListener('control', callback);
                controls.removeEventListener('transitionstart', callback);
                controls.removeEventListener('wake', callback);
            });
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsCameraControls, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.1.0", version: "19.1.4", type: NgtsCameraControls, isStandalone: true, selector: "ngts-camera-controls", inputs: { options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, outputs: { changed: "changed", started: "started", ended: "ended" }, ngImport: i0, template: `
		<ngt-primitive *args="[controls()]" [parameters]="parameters()">
			<ng-content />
		</ngt-primitive>
	`, isInline: true, dependencies: [{ kind: "directive", type: NgtArgs, selector: "ng-template[args]", inputs: ["args"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsCameraControls, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-camera-controls',
                    template: `
		<ngt-primitive *args="[controls()]" [parameters]="parameters()">
			<ng-content />
		</ngt-primitive>
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    imports: [NgtArgs],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }], ctorParameters: () => [] });

const defaultOptions$1 = {
    enableDamping: true,
    regress: false,
    makeDefault: false,
    keyEvents: false,
};
class NgtsOrbitControls {
    constructor() {
        this.options = input(defaultOptions$1, { transform: mergeInputs(defaultOptions$1) });
        this.parameters = omit(this.options, ['makeDefault', 'camera', 'regress', 'domElement', 'keyEvents', 'enableDamping']);
        this.changed = output();
        this.started = output();
        this.ended = output();
        this.store = injectStore();
        this.invalidate = this.store.select('invalidate');
        this.performanceRegress = this.store.select('performance', 'regress');
        this.defaultCamera = this.store.select('camera');
        this.glDomElement = this.store.select('gl', 'domElement');
        this.camera = pick(this.options, 'camera');
        this.regress = pick(this.options, 'regress');
        this.keyEvents = pick(this.options, 'keyEvents');
        this.domElement = pick(this.options, 'domElement');
        this.makeDefault = pick(this.options, 'makeDefault');
        this.controls = computed(() => {
            const [camera, defaultCamera] = [this.camera(), this.defaultCamera()];
            const controlsCamera = camera || defaultCamera;
            return new OrbitControls(controlsCamera);
        });
        this.enableDamping = pick(this.options, 'enableDamping');
        injectBeforeRender(() => {
            const controls = this.controls();
            if (controls?.enabled) {
                controls.update();
            }
        }, { priority: -1 });
        effect((onCleanup) => {
            const makeDefault = this.makeDefault();
            if (!makeDefault)
                return;
            const controls = this.controls();
            if (!controls)
                return;
            const oldControls = this.store.get('controls');
            this.store.update({ controls });
            onCleanup(() => void this.store.update({ controls: oldControls }));
        });
        effect((onCleanup) => {
            const [keyEvents, domElement, controls] = [
                this.keyEvents(),
                this.domElement() || this.store.get('events', 'connected') || this.glDomElement(),
                this.controls(),
                this.invalidate(),
                this.regress(),
            ];
            if (!controls)
                return;
            if (keyEvents) {
                controls.connect(keyEvents === true ? domElement : keyEvents);
            }
            else {
                controls.connect(domElement);
            }
            onCleanup(() => void controls.dispose());
        });
        effect((onCleanup) => {
            const controls = this.controls();
            if (!controls)
                return;
            const [invalidate, performanceRegress, regress] = [this.invalidate(), this.performanceRegress(), this.regress()];
            const changeCallback = (e) => {
                invalidate();
                if (regress)
                    performanceRegress();
                this.changed.emit(e);
            };
            const startCallback = this.started.emit.bind(this.started);
            const endCallback = this.ended.emit.bind(this.ended);
            controls.addEventListener('change', changeCallback);
            controls.addEventListener('start', startCallback);
            controls.addEventListener('end', endCallback);
            onCleanup(() => {
                controls.removeEventListener('change', changeCallback);
                controls.removeEventListener('start', startCallback);
                controls.removeEventListener('end', endCallback);
            });
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsOrbitControls, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.1.0", version: "19.1.4", type: NgtsOrbitControls, isStandalone: true, selector: "ngts-orbit-controls", inputs: { options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, outputs: { changed: "changed", started: "started", ended: "ended" }, ngImport: i0, template: `
		<ngt-primitive *args="[controls()]" [parameters]="parameters()" [enableDamping]="enableDamping()">
			<ng-content />
		</ngt-primitive>
	`, isInline: true, dependencies: [{ kind: "directive", type: NgtArgs, selector: "ng-template[args]", inputs: ["args"] }] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsOrbitControls, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-orbit-controls',
                    template: `
		<ngt-primitive *args="[controls()]" [parameters]="parameters()" [enableDamping]="enableDamping()">
			<ng-content />
		</ngt-primitive>
	`,
                    imports: [NgtArgs],
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                }]
        }], ctorParameters: () => [] });

const defaultOptions = {
    eps: 0.00001,
    horizontal: false,
    infinite: false,
    pages: 1,
    distance: 1,
    damping: 0.25,
    maxSpeed: Infinity,
    prepend: false,
    enabled: true,
    style: {},
};
class NgtsScrollControls {
    constructor() {
        this.progress = model(0);
        this.options = input(defaultOptions, { transform: mergeInputs(defaultOptions) });
        this.document = inject(DOCUMENT);
        this.store = injectStore();
        this.gl = this.store.select('gl');
        this.events = this.store.select('events');
        this.invalidate = this.store.select('invalidate');
        this.size = this.store.select('size');
        this.domElement = pick(this.gl, 'domElement');
        this.target = pick(this.domElement, 'parentNode');
        this._el = this.document.createElement('div');
        this._fill = this.document.createElement('div');
        this._fixed = this.document.createElement('div');
        this.style = pick(this.options, 'style');
        this.prepend = pick(this.options, 'prepend');
        this.enabled = pick(this.options, 'enabled');
        this.infinite = pick(this.options, 'infinite');
        this.maxSpeed = pick(this.options, 'maxSpeed');
        this.eps = pick(this.options, 'eps');
        this.horizontal = pick(this.options, 'horizontal');
        this.pages = pick(this.options, 'pages');
        this.distance = pick(this.options, 'distance');
        this.damping = pick(this.options, 'damping');
        this.scroll = 0;
        this.offset = 0;
        this.delta = 0;
        effect((onCleanup) => {
            const target = this.target();
            if (!target)
                return;
            const parent = target;
            const [pages, distance, horizontal, el, fill, fixed, style, prepend, domElement, events] = [
                this.pages(),
                this.distance(),
                this.horizontal(),
                this.el,
                this.fill,
                this.fixed,
                untracked(this.style),
                untracked(this.prepend),
                untracked(this.domElement),
                untracked(this.events),
            ];
            el.style.position = 'absolute';
            el.style.width = '100%';
            el.style.height = '100%';
            el.style[horizontal ? 'overflowX' : 'overflowY'] = 'auto';
            el.style[horizontal ? 'overflowY' : 'overflowX'] = 'hidden';
            el.style.top = '0px';
            el.style.left = '0px';
            for (const key in style) {
                el.style[key] = style[key];
            }
            fixed.style.position = 'sticky';
            fixed.style.top = '0px';
            fixed.style.left = '0px';
            fixed.style.width = '100%';
            fixed.style.height = '100%';
            fixed.style.overflow = 'hidden';
            el.appendChild(fixed);
            fill.style.height = horizontal ? '100%' : `${pages * distance * 100}%`;
            fill.style.width = horizontal ? `${pages * distance * 100}%` : '100%';
            fill.style.pointerEvents = 'none';
            el.appendChild(fill);
            if (prepend)
                parent.prepend(el);
            else
                parent.appendChild(el);
            // Init scroll one pixel in to allow upward/leftward scroll
            el[horizontal ? 'scrollLeft' : 'scrollTop'] = 1;
            const oldTarget = (events.connected || domElement);
            requestAnimationFrame(() => events.connect?.(el));
            const oldCompute = events.compute?.bind(events);
            this.store.snapshot.setEvents({
                compute(event, store) {
                    const state = store.snapshot;
                    // we are using boundingClientRect because we could not rely on target.offsetTop as canvas could be positioned anywhere in dom
                    const { left, top } = parent.getBoundingClientRect();
                    const offsetX = event.clientX - left;
                    const offsetY = event.clientY - top;
                    state.pointer.set((offsetX / state.size.width) * 2 - 1, -(offsetY / state.size.height) * 2 + 1);
                    state.raycaster.setFromCamera(state.pointer, state.camera);
                },
            });
            onCleanup(() => {
                parent.removeChild(el);
                this.store.snapshot.setEvents({ compute: oldCompute });
                events.connect?.(oldTarget);
            });
        });
        effect((onCleanup) => {
            const [el, events] = [this.el, this.events()];
            if (!events.connected || events.connected !== el)
                return;
            const [size, infinite, invalidate, horizontal, enabled] = [
                this.size(),
                this.infinite(),
                this.invalidate(),
                this.horizontal(),
                this.enabled(),
            ];
            const containerLength = size[horizontal ? 'width' : 'height'];
            const scrollLength = el[horizontal ? 'scrollWidth' : 'scrollHeight'];
            const scrollThreshold = scrollLength - containerLength;
            let current = 0;
            let disableScroll = true;
            let firstRun = true;
            const onScroll = () => {
                // Prevent first scroll because it is indirectly caused by the one pixel offset
                if (!enabled || firstRun)
                    return;
                invalidate();
                current = el[horizontal ? 'scrollLeft' : 'scrollTop'];
                this.scroll = current / scrollThreshold;
                if (infinite) {
                    if (!disableScroll) {
                        if (current >= scrollThreshold) {
                            const damp = 1 - this.offset;
                            el[horizontal ? 'scrollLeft' : 'scrollTop'] = 1;
                            this.scroll = this.offset = -damp;
                            disableScroll = true;
                        }
                        else if (current <= 0) {
                            const damp = 1 + this.offset;
                            el[horizontal ? 'scrollLeft' : 'scrollTop'] = scrollLength;
                            this.scroll = this.offset = damp;
                            disableScroll = true;
                        }
                    }
                    if (disableScroll)
                        setTimeout(() => (disableScroll = false), 40);
                }
                this.progress.set(this.scroll);
            };
            el.addEventListener('scroll', onScroll, { passive: true });
            requestAnimationFrame(() => (firstRun = false));
            const onWheel = (e) => (el[horizontal ? 'scrollLeft' : 'scrollTop'] += e.deltaY / 2);
            if (horizontal)
                el.addEventListener('wheel', onWheel, { passive: true });
            onCleanup(() => {
                el.removeEventListener('scroll', onScroll);
                if (horizontal)
                    el.removeEventListener('wheel', onWheel);
            });
        });
        let last = 0;
        injectBeforeRender(({ delta }) => {
            last = this.offset;
            easing.damp(this, 'offset', this.scroll, this.damping(), delta, this.maxSpeed(), undefined, this.eps());
            easing.damp(this, 'delta', Math.abs(last - this.offset), this.damping(), delta, this.maxSpeed(), undefined, this.eps());
            if (this.delta > this.eps())
                this.invalidate()();
        });
    }
    get el() {
        return this._el;
    }
    get fill() {
        return this._fill;
    }
    get fixed() {
        return this._fixed;
    }
    range(from, distance, margin = 0) {
        const start = from - margin;
        const end = start + distance + margin * 2;
        return this.offset < start ? 0 : this.offset > end ? 1 : (this.offset - start) / (end - start);
    }
    curve(from, distance, margin = 0) {
        return Math.sin(this.range(from, distance, margin) * Math.PI);
    }
    visible(from, distance, margin = 0) {
        const start = from - margin;
        const end = start + distance + margin * 2;
        return this.offset >= start && this.offset <= end;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsScrollControls, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.1.0", version: "19.1.4", type: NgtsScrollControls, isStandalone: true, selector: "ngts-scroll-controls", inputs: { progress: { classPropertyName: "progress", publicName: "progress", isSignal: true, isRequired: false, transformFunction: null }, options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, outputs: { progress: "progressChange" }, ngImport: i0, template: `
		<ng-content />
	`, isInline: true, changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsScrollControls, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-scroll-controls',
                    template: `
		<ng-content />
	`,
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }], ctorParameters: () => [] });
class NgtsScrollCanvas {
    constructor() {
        this.host = inject(ElementRef);
        this.scrollControls = inject(NgtsScrollControls);
        this.store = injectStore();
        this.viewport = this.store.select('viewport');
        injectBeforeRender(() => {
            const group = this.host.nativeElement;
            group.position.x = this.scrollControls.horizontal()
                ? -this.viewport().width * (this.scrollControls.pages() - 1) * this.scrollControls.offset
                : 0;
            group.position.y = this.scrollControls.horizontal()
                ? 0
                : this.viewport().height * (this.scrollControls.pages() - 1) * this.scrollControls.offset;
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsScrollCanvas, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.1.4", type: NgtsScrollCanvas, isStandalone: true, selector: "ngt-group[ngtsScrollCanvas]", ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsScrollCanvas, decorators: [{
            type: Directive,
            args: [{ selector: 'ngt-group[ngtsScrollCanvas]' }]
        }], ctorParameters: () => [] });
class NgtsScrollHtml extends NgtHTML {
    constructor() {
        super();
        this.scrollControls = inject(NgtsScrollControls);
        this.size = this.store.select('size');
        injectBeforeRender(() => {
            if (this.scrollControls.delta > this.scrollControls.eps()) {
                this.host.nativeElement.style.transform = `translate3d(${this.scrollControls.horizontal()
                    ? -this.size().width * (this.scrollControls.pages() - 1) * this.scrollControls.offset
                    : 0}px,${this.scrollControls.horizontal() ? 0 : this.size().height * (this.scrollControls.pages() - 1) * -this.scrollControls.offset}px,0)`;
            }
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsScrollHtml, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.1.4", type: NgtsScrollHtml, isStandalone: true, selector: "div[ngtsScrollHTML]", host: { styleAttribute: "position: absolute; top: 0; left: 0; will-change: transform;" }, providers: [provideHTMLDomElement([NgtsScrollControls], (scrollControls) => scrollControls.fixed)], usesInheritance: true, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsScrollHtml, decorators: [{
            type: Directive,
            args: [{
                    selector: 'div[ngtsScrollHTML]',
                    host: { style: 'position: absolute; top: 0; left: 0; will-change: transform;' },
                    providers: [provideHTMLDomElement([NgtsScrollControls], (scrollControls) => scrollControls.fixed)],
                }]
        }], ctorParameters: () => [] });

/**
 * Generated bundle index. Do not edit.
 */

export { NgtsCameraControls, NgtsOrbitControls, NgtsScrollCanvas, NgtsScrollControls, NgtsScrollHtml };
//# sourceMappingURL=angular-three-soba-controls.mjs.map
