import * as i0 from '@angular/core';
import { input, computed, effect, Component, CUSTOM_ELEMENTS_SCHEMA, ChangeDetectionStrategy, inject, DestroyRef, Injector, Directive, viewChild, contentChild, TemplateRef, signal, untracked } from '@angular/core';
import { omit, is, pick, NgtArgs, injectBeforeRender, injectStore, getLocalState, extend, applyProps } from 'angular-three';
import CustomShaderMaterial from 'three-custom-shader-material/vanilla';
import { MeshDistortMaterial, MeshPortalMaterial, meshPortalMaterialApplySDF, BlurPass, MeshReflectorMaterial, MeshDiscardMaterial, MeshTransmissionMaterial, MeshWobbleMaterial } from 'angular-three-soba/vanilla-exports';
import { mergeInputs } from 'ngxtension/inject-inputs';
import { NgTemplateOutlet } from '@angular/common';
import { injectFBO, getVersion, injectIntersect } from 'angular-three-soba/misc';
import { NgtsRenderTexture, NgtsRenderTextureContent } from 'angular-three-soba/staging';
import { ShaderMaterial, Mesh, Plane, Vector3, Matrix4, Vector4, PerspectiveCamera, LinearFilter, HalfFloatType, WebGLRenderTarget, DepthTexture, DepthFormat, UnsignedShortType, FrontSide, NoToneMapping, BackSide } from 'three';
import { FullScreenQuad } from 'three-stdlib';
import { MeshRefractionMaterial, PointMaterial } from 'angular-three-soba/shaders';
import { MeshBVHUniformStruct, MeshBVH, SAH } from 'three-mesh-bvh';

class NgtsCustomShaderMaterial {
    constructor() {
        this.baseMaterial = input.required();
        this.attach = input('material');
        this.options = input({});
        this.parameters = omit(this.options, ['fragmentShader', 'vertexShader', 'uniforms', 'cacheKey']);
        this.base = computed(() => {
            const baseMaterial = this.baseMaterial();
            return is.ref(baseMaterial) ? baseMaterial.nativeElement : baseMaterial;
        });
        this.fragmentShader = pick(this.options, 'fragmentShader');
        this.vertexShader = pick(this.options, 'vertexShader');
        this.uniforms = pick(this.options, 'uniforms');
        this.cacheKey = pick(this.options, 'cacheKey');
        this.material = computed(() => {
            const [base, fragmentShader, vertexShader, uniforms, cacheKey] = [
                this.base(),
                this.fragmentShader(),
                this.vertexShader(),
                this.uniforms(),
                this.cacheKey(),
            ];
            // NOTE: this is specific to angular-three
            if (is.instance(base)) {
                delete base['__ngt__'];
                delete base['__ngt_renderer__'];
            }
            return new CustomShaderMaterial({
                baseMaterial: base,
                fragmentShader,
                vertexShader,
                uniforms,
                cacheKey,
            });
        });
        effect((onCleanup) => {
            const material = this.material();
            onCleanup(() => material.dispose());
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsCustomShaderMaterial, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.1.0", version: "19.1.4", type: NgtsCustomShaderMaterial, isStandalone: true, selector: "ngts-custom-shader-material", inputs: { baseMaterial: { classPropertyName: "baseMaterial", publicName: "baseMaterial", isSignal: true, isRequired: true, transformFunction: null }, attach: { classPropertyName: "attach", publicName: "attach", isSignal: true, isRequired: false, transformFunction: null }, options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, ngImport: i0, template: `
		<ngt-primitive *args="[material()]" [attach]="attach()" [parameters]="parameters()">
			<ng-content />
		</ngt-primitive>
	`, isInline: true, dependencies: [{ kind: "directive", type: NgtArgs, selector: "ng-template[args]", inputs: ["args"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsCustomShaderMaterial, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-custom-shader-material',
                    template: `
		<ngt-primitive *args="[material()]" [attach]="attach()" [parameters]="parameters()">
			<ng-content />
		</ngt-primitive>
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    imports: [NgtArgs],
                }]
        }], ctorParameters: () => [] });

const defaultOptions$5 = {
    speed: 1,
};
class NgtsMeshDistortMaterial {
    constructor() {
        this.attach = input('material');
        this.options = input(defaultOptions$5, { transform: mergeInputs(defaultOptions$5) });
        this.parameters = omit(this.options, ['speed']);
        this.material = new MeshDistortMaterial();
        this.speed = pick(this.options, 'speed');
        inject(DestroyRef).onDestroy(() => this.material.dispose());
        injectBeforeRender(({ clock }) => {
            this.material.time = clock.getElapsedTime() * this.speed();
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsMeshDistortMaterial, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.1.0", version: "19.1.4", type: NgtsMeshDistortMaterial, isStandalone: true, selector: "ngts-mesh-distort-material", inputs: { attach: { classPropertyName: "attach", publicName: "attach", isSignal: true, isRequired: false, transformFunction: null }, options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, ngImport: i0, template: `
		<ngt-primitive *args="[material]" [attach]="attach()" [parameters]="parameters()">
			<ng-content />
		</ngt-primitive>
	`, isInline: true, dependencies: [{ kind: "directive", type: NgtArgs, selector: "ng-template[args]", inputs: ["args"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsMeshDistortMaterial, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-mesh-distort-material',
                    template: `
		<ngt-primitive *args="[material]" [attach]="attach()" [parameters]="parameters()">
			<ng-content />
		</ngt-primitive>
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    imports: [NgtArgs],
                }]
        }], ctorParameters: () => [] });

/**
 * This directive is used inside of the render texture, hence has access to the render texture store (a portal store)
 */
class ManagePortalScene {
    constructor() {
        this.events = input();
        this.rootScene = input.required();
        this.material = input.required();
        this.priority = input.required();
        this.worldUnits = input.required();
        const injector = inject(Injector);
        const renderTextureStore = injectStore();
        const portalScene = renderTextureStore.select('scene');
        const portalSetEvents = renderTextureStore.select('setEvents');
        const buffer1 = injectFBO();
        const buffer2 = injectFBO();
        const fullScreenQuad = computed(() => {
            // This fullscreen-quad is used to blend the two textures
            const blend = { value: 0 };
            const quad = new FullScreenQuad(new ShaderMaterial({
                uniforms: {
                    a: { value: buffer1().texture },
                    b: { value: buffer2().texture },
                    blend,
                },
                vertexShader: /*glsl*/ `
		      varying vec2 vUv;
		      void main() {
		        vUv = uv;
		        gl_Position = projectionMatrix * modelViewMatrix * vec4( position, 1.0 );
		      }`,
                fragmentShader: /*glsl*/ `
		      uniform sampler2D a;
		      uniform sampler2D b;
		      uniform float blend;
		      varying vec2 vUv;
		      #include <packing>
		      void main() {
		        vec4 ta = texture2D(a, vUv);
		        vec4 tb = texture2D(b, vUv);
		        gl_FragColor = mix(tb, ta, blend);
		        #include <tonemapping_fragment>
		        #include <${getVersion() >= 154 ? 'colorspace_fragment' : 'encodings_fragment'}>
		      }`,
            }));
            return [quad, blend];
        });
        effect(() => {
            portalScene().matrixAutoUpdate = false;
        });
        effect(() => {
            const [events, setEvents] = [this.events(), portalSetEvents()];
            if (!events)
                return;
            setEvents({ enabled: events });
        });
        effect((onCleanup) => {
            const priority = this.priority();
            // we start the before render in effect because we need the priority input to be resolved
            const sub = injectBeforeRender(({ gl, camera }) => {
                const material = this.material();
                const localState = getLocalState(material);
                if (!localState)
                    return;
                const parent = localState.parent();
                if (!parent)
                    return;
                const materialBlend = 'blend' in material && typeof material.blend === 'number' ? material.blend : 0;
                const [worldUnits, rootScene, scene, [quad, blend]] = [
                    this.worldUnits(),
                    this.rootScene(),
                    portalScene(),
                    fullScreenQuad(),
                ];
                // Move portal contents along with the parent if worldUnits is true
                if (!worldUnits) {
                    // If the portal renders exclusively the original scene needs to be updated
                    if (priority && materialBlend === 1)
                        parent.updateWorldMatrix(true, false);
                    scene.matrixWorld.copy(parent.matrixWorld);
                }
                else {
                    scene.matrixWorld.identity();
                }
                // This bit is only necessary if the portal is blended, now it has a render-priority
                // and will take over the render loop
                if (priority) {
                    if (materialBlend > 0 && materialBlend < 1) {
                        // If blend is ongoing (> 0 and < 1) then we need to render both the root scene
                        // and the portal scene, both will then be mixed in the quad from above
                        blend.value = materialBlend;
                        gl.setRenderTarget(buffer1());
                        gl.render(scene, camera);
                        gl.setRenderTarget(buffer2());
                        gl.render(rootScene, camera);
                        gl.setRenderTarget(null);
                        quad.render(gl);
                    }
                    else if (materialBlend === 1) {
                        // However if blend is 1 we only need to render the portal scene
                        gl.render(scene, camera);
                    }
                }
            }, { injector, priority });
            onCleanup(() => sub());
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: ManagePortalScene, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "17.1.0", version: "19.1.4", type: ManagePortalScene, isStandalone: true, selector: "ngts-manage-portal-scene", inputs: { events: { classPropertyName: "events", publicName: "events", isSignal: true, isRequired: false, transformFunction: null }, rootScene: { classPropertyName: "rootScene", publicName: "rootScene", isSignal: true, isRequired: true, transformFunction: null }, material: { classPropertyName: "material", publicName: "material", isSignal: true, isRequired: true, transformFunction: null }, priority: { classPropertyName: "priority", publicName: "priority", isSignal: true, isRequired: true, transformFunction: null }, worldUnits: { classPropertyName: "worldUnits", publicName: "worldUnits", isSignal: true, isRequired: true, transformFunction: null } }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: ManagePortalScene, decorators: [{
            type: Directive,
            args: [{ selector: 'ngts-manage-portal-scene' }]
        }], ctorParameters: () => [] });
const defaultOptions$4 = {
    blend: 0,
    blur: 0,
    resolution: 512,
    worldUnits: false,
    eventPriority: 0,
    renderPriority: 0,
    events: false,
};
class NgtsMeshPortalMaterial {
    constructor() {
        this.attach = input('material');
        this.options = input(defaultOptions$4, { transform: mergeInputs(defaultOptions$4) });
        this.parameters = omit(this.options, ['blur', 'resolution', 'worldUnits', 'eventPriority', 'renderPriority', 'events']);
        this.blur = pick(this.options, 'blur');
        this.eventPriority = pick(this.options, 'eventPriority');
        this.renderPriority = pick(this.options, 'renderPriority');
        this.events = pick(this.options, 'events');
        this.worldUnits = pick(this.options, 'worldUnits');
        this.materialRef = viewChild.required('material');
        this.content = contentChild.required(TemplateRef);
        this.store = injectStore();
        this.size = this.store.select('size');
        this.viewport = this.store.select('viewport');
        this.gl = this.store.select('gl');
        this.setEvents = this.store.select('setEvents');
        this.rootScene = this.store.select('scene');
        this.materialResolution = computed(() => [
            this.size().width * this.viewport().dpr,
            this.size().height * this.viewport().dpr,
        ]);
        this.resolution = pick(this.options, 'resolution');
        this.parent = signal(null);
        this.visible = injectIntersect(this.parent, { source: signal(true) });
        this.renderTextureFrames = computed(() => (this.visible() ? Infinity : 0));
        this.renderTextureCompute = computed(() => {
            const [parent, material] = [this.parent(), this.materialRef().nativeElement];
            const computeFn = (event, state) => {
                if (!parent)
                    return false;
                state.snapshot.pointer.set((event.offsetX / state.snapshot.size.width) * 2 - 1, -(event.offsetY / state.snapshot.size.height) * 2 + 1);
                state.snapshot.raycaster.setFromCamera(state.snapshot.pointer, state.snapshot.camera);
                if ('blend' in material && material.blend === 0) {
                    // We run a quick check against the parent, if it isn't hit there's no need to raycast at all
                    const [intersection] = state.snapshot.raycaster.intersectObject(parent);
                    if (!intersection) {
                        // Cancel out the raycast camera if the parent mesh isn't hit
                        Object.assign(state.snapshot.raycaster, { camera: undefined });
                        return false;
                    }
                }
                return;
            };
            return computeFn;
        });
        this.priority = signal(0);
        extend({ MeshPortalMaterial });
        effect(() => {
            const material = this.materialRef().nativeElement;
            const localState = getLocalState(material);
            if (!localState)
                return;
            const materialParent = localState.parent();
            if (!materialParent || !(materialParent instanceof Mesh))
                return;
            // Since the ref above is not tied to a mesh directly (we're inside a material),
            // it has to be tied to the parent mesh here
            this.parent.set(materialParent);
        });
        effect(() => {
            const events = this.events();
            if (!events)
                return;
            const setEvents = this.setEvents();
            setEvents({ enabled: !events });
        });
        effect(() => {
            const [material, parent] = [this.materialRef().nativeElement, this.parent()];
            if (!parent)
                return;
            const [resolution, blur, gl] = [this.resolution(), this.blur(), this.gl()];
            // apply the SDF mask once
            if (blur && material.sdf == null) {
                meshPortalMaterialApplySDF(parent, resolution, gl);
            }
        });
        injectBeforeRender(() => {
            const material = this.materialRef().nativeElement;
            const priority = 'blend' in material && typeof material.blend === 'number' && material.blend > 0
                ? Math.max(1, this.renderPriority())
                : 0;
            // If blend is > 0 then the portal is being entered, the render-priority must change
            if (this.priority() !== priority) {
                this.priority.set(priority);
            }
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsMeshPortalMaterial, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsMeshPortalMaterial, isStandalone: true, selector: "ngts-mesh-portal-material", inputs: { attach: { classPropertyName: "attach", publicName: "attach", isSignal: true, isRequired: false, transformFunction: null }, options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, queries: [{ propertyName: "content", first: true, predicate: TemplateRef, descendants: true, isSignal: true }], viewQueries: [{ propertyName: "materialRef", first: true, predicate: ["material"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-mesh-portal-material
			#material
			[attach]="attach()"
			[blur]="blur()"
			[blend]="0"
			[resolution]="materialResolution()"
			[parameters]="parameters()"
		>
			<ngts-render-texture
				[options]="{
					frames: renderTextureFrames(),
					eventPriority: eventPriority(),
					renderPriority: renderPriority(),
					compute: renderTextureCompute(),
				}"
			>
				<ng-template renderTextureContent let-injector="injector">
					<ng-container *ngTemplateOutlet="content(); injector: injector" />
					<ngts-manage-portal-scene
						[events]="events()"
						[rootScene]="rootScene()"
						[priority]="priority()"
						[material]="material"
						[worldUnits]="worldUnits()"
					/>
				</ng-template>
			</ngts-render-texture>
		</ngt-mesh-portal-material>
	`, isInline: true, dependencies: [{ kind: "component", type: NgtsRenderTexture, selector: "ngts-render-texture", inputs: ["attach", "options"] }, { kind: "directive", type: NgtsRenderTextureContent, selector: "ng-template[renderTextureContent]" }, { kind: "directive", type: ManagePortalScene, selector: "ngts-manage-portal-scene", inputs: ["events", "rootScene", "material", "priority", "worldUnits"] }, { kind: "directive", type: NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet", "ngTemplateOutletInjector"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsMeshPortalMaterial, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-mesh-portal-material',
                    template: `
		<ngt-mesh-portal-material
			#material
			[attach]="attach()"
			[blur]="blur()"
			[blend]="0"
			[resolution]="materialResolution()"
			[parameters]="parameters()"
		>
			<ngts-render-texture
				[options]="{
					frames: renderTextureFrames(),
					eventPriority: eventPriority(),
					renderPriority: renderPriority(),
					compute: renderTextureCompute(),
				}"
			>
				<ng-template renderTextureContent let-injector="injector">
					<ng-container *ngTemplateOutlet="content(); injector: injector" />
					<ngts-manage-portal-scene
						[events]="events()"
						[rootScene]="rootScene()"
						[priority]="priority()"
						[material]="material"
						[worldUnits]="worldUnits()"
					/>
				</ng-template>
			</ngts-render-texture>
		</ngt-mesh-portal-material>
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    imports: [NgtsRenderTexture, NgtsRenderTextureContent, ManagePortalScene, NgTemplateOutlet],
                }]
        }], ctorParameters: () => [] });

const defaultOptions$3 = {
    mixBlur: 0,
    mixStrength: 1,
    resolution: 256,
    blur: [0, 0],
    minDepthThreshold: 0.9,
    maxDepthThreshold: 1,
    depthScale: 0,
    depthToBlurRatioBias: 0.25,
    mirror: 0,
    distortion: 1,
    mixContrast: 1,
    reflectorOffset: 0,
};
class NgtsMeshReflectorMaterial {
    constructor() {
        this.attach = input('material');
        this.options = input(defaultOptions$3, { transform: mergeInputs(defaultOptions$3) });
        this.parameters = omit(this.options, [
            'distortionMap',
            'resolution',
            'mixBlur',
            'mixStrength',
            'blur',
            'minDepthThreshold',
            'maxDepthThreshold',
            'depthScale',
            'depthToBlurRatioBias',
            'mirror',
            'distortion',
            'mixContrast',
            'reflectorOffset',
        ]);
        this.store = injectStore();
        this.gl = this.store.select('gl');
        this.materialRef = viewChild('material');
        this.reflectOptions = pick(this.options, [
            'distortionMap',
            'resolution',
            'mixBlur',
            'mixStrength',
            'blur',
            'minDepthThreshold',
            'maxDepthThreshold',
            'depthScale',
            'depthToBlurRatioBias',
            'mirror',
            'distortion',
            'mixContrast',
            'reflectorOffset',
        ]);
        this.blur = pick(this.reflectOptions, 'blur');
        this.normalizedBlur = computed(() => {
            const blur = this.blur();
            return Array.isArray(blur) ? blur : [blur, blur];
        });
        this.hasBlur = computed(() => this.normalizedBlur()[0] + this.normalizedBlur()[1] > 0);
        this.reflectorPlane = new Plane();
        this.normal = new Vector3();
        this.reflectorWorldPosition = new Vector3();
        this.cameraWorldPosition = new Vector3();
        this.rotationMatrix = new Matrix4();
        this.lookAtPosition = new Vector3(0, 0, -1);
        this.clipPlane = new Vector4();
        this.view = new Vector3();
        this.target = new Vector3();
        this.q = new Vector4();
        this.textureMatrix = new Matrix4();
        this.virtualCamera = new PerspectiveCamera();
        this.reflectState = computed(() => {
            const [gl, { resolution, minDepthThreshold, maxDepthThreshold, depthScale, depthToBlurRatioBias, mirror, mixBlur, mixStrength, distortion, distortionMap, mixContrast, }, blur, hasBlur,] = [this.gl(), this.reflectOptions(), this.normalizedBlur(), this.hasBlur()];
            const renderTargetParameters = {
                minFilter: LinearFilter,
                magFilter: LinearFilter,
                type: HalfFloatType,
            };
            const fbo1 = new WebGLRenderTarget(resolution, resolution, renderTargetParameters);
            fbo1.depthBuffer = true;
            fbo1.depthTexture = new DepthTexture(resolution, resolution);
            fbo1.depthTexture.format = DepthFormat;
            fbo1.depthTexture.type = UnsignedShortType;
            const fbo2 = new WebGLRenderTarget(resolution, resolution, renderTargetParameters);
            const blurPass = new BlurPass({
                gl,
                resolution,
                width: blur[0],
                height: blur[1],
                minDepthThreshold,
                maxDepthThreshold,
                depthScale,
                depthToBlurRatioBias,
            });
            const defines = {};
            if (hasBlur) {
                defines['USE_BLUR'] = '';
            }
            if (depthScale > 0) {
                defines['USE_DEPTH'] = '';
            }
            if (distortionMap) {
                defines['USE_DISTORTION'] = '';
            }
            const reflectorParameters = {
                mirror,
                textureMatrix: this.textureMatrix,
                mixBlur,
                tDiffuse: fbo1.texture,
                tDepth: fbo1.depthTexture,
                tDiffuseBlur: fbo2.texture,
                hasBlur,
                mixStrength,
                minDepthThreshold,
                maxDepthThreshold,
                depthScale,
                depthToBlurRatioBias,
                distortion,
                distortionMap,
                mixContrast,
                defines,
            };
            return { fbo1, fbo2, blurPass, reflectorParameters };
        });
        this.definesKey = computed(() => {
            const defines = this.reflectState().reflectorParameters.defines;
            return Object.entries(defines).reduce((acc, [key, value]) => (value != null ? `${acc} ${key}` : acc), '');
        });
        this.material = computed(() => {
            const prevMaterial = untracked(this.materialRef)?.nativeElement;
            if (prevMaterial) {
                prevMaterial.dispose();
                delete prevMaterial['__ngt__'];
                delete prevMaterial['__ngt_renderer__'];
            }
            // tracking defines key so that the material is recreated when the defines change
            this.definesKey();
            return new MeshReflectorMaterial();
        });
        effect(() => {
            const material = this.materialRef()?.nativeElement;
            if (!material)
                return;
            const { reflectorParameters } = this.reflectState();
            applyProps(material, { ...reflectorParameters, ...this.parameters() });
        });
        inject(DestroyRef).onDestroy(() => {
            const material = this.materialRef()?.nativeElement;
            if (material) {
                material.dispose();
                delete material['__ngt__'];
                delete material['__ngt_renderer__'];
            }
        });
        injectBeforeRender(({ gl, scene }) => {
            const material = this.materialRef()?.nativeElement;
            if (!material)
                return;
            const localState = getLocalState(material);
            if (!localState)
                return;
            const parent = Reflect.get(material, 'parent') ?? untracked(localState.parent);
            if (!parent)
                return;
            const { fbo1, fbo2, blurPass } = this.reflectState();
            const hasBlur = this.hasBlur();
            parent.visible = false;
            const currentXrEnabled = gl.xr.enabled;
            const currentShadowAutoUpdate = gl.shadowMap.autoUpdate;
            this.beforeRender();
            gl.xr.enabled = false;
            gl.shadowMap.autoUpdate = false;
            gl.setRenderTarget(fbo1);
            gl.state.buffers.depth.setMask(true);
            if (!gl.autoClear)
                gl.clear();
            gl.render(scene, this.virtualCamera);
            if (hasBlur)
                blurPass.render(gl, fbo1, fbo2);
            gl.xr.enabled = currentXrEnabled;
            gl.shadowMap.autoUpdate = currentShadowAutoUpdate;
            parent.visible = true;
            gl.setRenderTarget(null);
        });
    }
    beforeRender() {
        const material = this.materialRef()?.nativeElement;
        if (!material)
            return;
        const localState = getLocalState(material);
        if (!localState)
            return;
        const parent = Reflect.get(material, 'parent') ?? untracked(localState.parent);
        if (!parent)
            return;
        const { camera } = this.store.snapshot;
        const { reflectorOffset } = untracked(this.reflectOptions);
        this.reflectorWorldPosition.setFromMatrixPosition(parent.matrixWorld);
        this.cameraWorldPosition.setFromMatrixPosition(camera.matrixWorld);
        this.rotationMatrix.extractRotation(parent.matrixWorld);
        this.normal.set(0, 0, 1);
        this.normal.applyMatrix4(this.rotationMatrix);
        this.reflectorWorldPosition.addScaledVector(this.normal, reflectorOffset);
        this.view.subVectors(this.reflectorWorldPosition, this.cameraWorldPosition);
        // Avoid rendering when reflector is facing away
        if (this.view.dot(this.normal) > 0)
            return;
        this.view.reflect(this.normal).negate();
        this.view.add(this.reflectorWorldPosition);
        this.rotationMatrix.extractRotation(camera.matrixWorld);
        this.lookAtPosition.set(0, 0, -1);
        this.lookAtPosition.applyMatrix4(this.rotationMatrix);
        this.lookAtPosition.add(this.cameraWorldPosition);
        this.target.subVectors(this.reflectorWorldPosition, this.lookAtPosition);
        this.target.reflect(this.normal).negate();
        this.target.add(this.reflectorWorldPosition);
        this.virtualCamera.position.copy(this.view);
        this.virtualCamera.up.set(0, 1, 0);
        this.virtualCamera.up.applyMatrix4(this.rotationMatrix);
        this.virtualCamera.up.reflect(this.normal);
        this.virtualCamera.lookAt(this.target);
        this.virtualCamera.far = camera.far; // Used in WebGLBackground
        this.virtualCamera.updateMatrixWorld();
        this.virtualCamera.projectionMatrix.copy(camera.projectionMatrix);
        // Update the texture matrix
        this.textureMatrix.set(0.5, 0.0, 0.0, 0.5, 0.0, 0.5, 0.0, 0.5, 0.0, 0.0, 0.5, 0.5, 0.0, 0.0, 0.0, 1.0);
        this.textureMatrix.multiply(this.virtualCamera.projectionMatrix);
        this.textureMatrix.multiply(this.virtualCamera.matrixWorldInverse);
        this.textureMatrix.multiply(parent.matrixWorld);
        // Now update projection matrix with new clip plane, implementing code from: http://www.terathon.com/code/oblique.html
        // Paper explaining this technique: http://www.terathon.com/lengyel/Lengyel-Oblique.pdf
        this.reflectorPlane.setFromNormalAndCoplanarPoint(this.normal, this.reflectorWorldPosition);
        this.reflectorPlane.applyMatrix4(this.virtualCamera.matrixWorldInverse);
        this.clipPlane.set(this.reflectorPlane.normal.x, this.reflectorPlane.normal.y, this.reflectorPlane.normal.z, this.reflectorPlane.constant);
        const projectionMatrix = this.virtualCamera.projectionMatrix;
        this.q.x = (Math.sign(this.clipPlane.x) + projectionMatrix.elements[8]) / projectionMatrix.elements[0];
        this.q.y = (Math.sign(this.clipPlane.y) + projectionMatrix.elements[9]) / projectionMatrix.elements[5];
        this.q.z = -1.0;
        this.q.w = (1.0 + projectionMatrix.elements[10]) / projectionMatrix.elements[14];
        // Calculate the scaled plane vector
        this.clipPlane.multiplyScalar(2.0 / this.clipPlane.dot(this.q));
        // Replacing the third row of the projection matrix
        projectionMatrix.elements[2] = this.clipPlane.x;
        projectionMatrix.elements[6] = this.clipPlane.y;
        projectionMatrix.elements[10] = this.clipPlane.z + 1.0;
        projectionMatrix.elements[14] = this.clipPlane.w;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsMeshReflectorMaterial, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsMeshReflectorMaterial, isStandalone: true, selector: "ngts-mesh-reflector-material", inputs: { attach: { classPropertyName: "attach", publicName: "attach", isSignal: true, isRequired: false, transformFunction: null }, options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "materialRef", first: true, predicate: ["material"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-primitive #material *args="[material()]" [attach]="attach()">
			<ng-content />
		</ngt-primitive>
	`, isInline: true, dependencies: [{ kind: "directive", type: NgtArgs, selector: "ng-template[args]", inputs: ["args"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsMeshReflectorMaterial, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-mesh-reflector-material',
                    template: `
		<ngt-primitive #material *args="[material()]" [attach]="attach()">
			<ng-content />
		</ngt-primitive>
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    imports: [NgtArgs],
                }]
        }], ctorParameters: () => [] });

const defaultOptions$2 = {
    bounces: 2,
    ior: 2.4,
    fresnel: 0,
    aberrationStrength: 0,
    color: 'white',
    fastChroma: true,
};
function isCubeTexture(def) {
    return def && def.isCubeTexture;
}
class NgtsMeshRefractionMaterial {
    constructor() {
        this.envMap = input.required();
        this.attach = input('material');
        this.options = input(defaultOptions$2, { transform: mergeInputs(defaultOptions$2) });
        this.parameters = omit(this.options, ['fastChroma', 'aberrationStrength']);
        this.fastChroma = pick(this.options, 'fastChroma');
        this.aberrationStrength = pick(this.options, 'aberrationStrength');
        this.materialRef = viewChild('material');
        this.store = injectStore();
        this.size = this.store.select('size');
        this.resolution = computed(() => [this.size().width, this.size().height]);
        this.defines = computed(() => {
            const _defines = {};
            const [envMap, aberrationStrength, fastChroma] = [
                untracked(this.envMap),
                this.aberrationStrength(),
                this.fastChroma(),
            ];
            const isCubeMap = isCubeTexture(envMap);
            const w = (isCubeMap ? envMap.image[0]?.width : envMap.image.width) ?? 1024;
            const cubeSize = w / 4;
            const _lodMax = Math.floor(Math.log2(cubeSize));
            const _cubeSize = Math.pow(2, _lodMax);
            const width = 3 * Math.max(_cubeSize, 16 * 7);
            const height = 4 * _cubeSize;
            if (isCubeMap)
                _defines['ENVMAP_TYPE_CUBEM'] = '';
            _defines['CUBEUV_TEXEL_WIDTH'] = `${1.0 / width}`;
            _defines['CUBEUV_TEXEL_HEIGHT'] = `${1.0 / height}`;
            _defines['CUBEUV_MAX_MIP'] = `${_lodMax}.0`;
            // Add defines from chromatic aberration
            if (aberrationStrength > 0)
                _defines['CHROMATIC_ABERRATIONS'] = '';
            if (fastChroma)
                _defines['FAST_CHROMA'] = '';
            return _defines;
        });
        this.defineKeys = computed(() => {
            return Object.keys(this.defines()).join(' ');
        });
        this.material = computed(() => {
            const prevMaterial = untracked(this.materialRef)?.nativeElement;
            if (prevMaterial) {
                prevMaterial.dispose();
                delete prevMaterial['__ngt__'];
                delete prevMaterial['__ngt_renderer__'];
            }
            // track by this.defineKeys
            this.defineKeys();
            return new MeshRefractionMaterial();
        });
        effect(() => {
            const material = this.materialRef()?.nativeElement;
            if (!material)
                return;
            const localState = getLocalState(material);
            if (!localState)
                return;
            const parent = untracked(localState.parent);
            const geometry = parent.geometry;
            if (geometry) {
                material.bvh = new MeshBVHUniformStruct();
                material.bvh.updateFrom(new MeshBVH(geometry.clone().toNonIndexed(), { strategy: SAH }));
            }
        });
        injectBeforeRender(({ camera }) => {
            const material = this.materialRef()?.nativeElement;
            if (material) {
                material.viewMatrixInverse = camera.matrixWorld;
                material.projectionMatrixInverse = camera.projectionMatrixInverse;
            }
        });
        inject(DestroyRef).onDestroy(() => {
            const material = this.materialRef()?.nativeElement;
            if (material) {
                material.dispose();
                delete material['__ngt__'];
                delete material['__ngt_renderer__'];
            }
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsMeshRefractionMaterial, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsMeshRefractionMaterial, isStandalone: true, selector: "ngts-mesh-refraction-material", inputs: { envMap: { classPropertyName: "envMap", publicName: "envMap", isSignal: true, isRequired: true, transformFunction: null }, attach: { classPropertyName: "attach", publicName: "attach", isSignal: true, isRequired: false, transformFunction: null }, options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "materialRef", first: true, predicate: ["material"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-primitive
			*args="[material()]"
			#material
			[attach]="attach()"
			[parameters]="parameters()"
			[envMap]="envMap()"
			[aberrationStrength]="aberrationStrength()"
			[resolution]="resolution()"
			[defines]="defines()"
		>
			<ng-content />
		</ngt-primitive>
	`, isInline: true, dependencies: [{ kind: "directive", type: NgtArgs, selector: "ng-template[args]", inputs: ["args"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsMeshRefractionMaterial, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-mesh-refraction-material',
                    template: `
		<ngt-primitive
			*args="[material()]"
			#material
			[attach]="attach()"
			[parameters]="parameters()"
			[envMap]="envMap()"
			[aberrationStrength]="aberrationStrength()"
			[resolution]="resolution()"
			[defines]="defines()"
		>
			<ng-content />
		</ngt-primitive>
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    imports: [NgtArgs],
                }]
        }], ctorParameters: () => [] });

const defaultOptions$1 = {
    transmissionSampler: false,
    backside: false,
    side: FrontSide,
    transmission: 1,
    thickness: 0,
    backsideThickness: 0,
    backsideEnvMapIntensity: 1,
    samples: 10,
};
class NgtsMeshTransmissionMaterial {
    constructor() {
        this.attach = input('material');
        this.options = input(defaultOptions$1, { transform: mergeInputs(defaultOptions$1) });
        this.parameters = omit(this.options, [
            'buffer',
            'transmissionSampler',
            'backside',
            'side',
            'transmission',
            'thickness',
            'backsideThickness',
            'backsideEnvMapIntensity',
            'samples',
            'resolution',
            'backsideResolution',
            'background',
            'anisotropy',
            'anisotropicBlur',
        ]);
        this.resolution = pick(this.options, 'resolution');
        this.backsideResolution = pick(this.options, 'backsideResolution');
        this.samples = pick(this.options, 'samples');
        this.buffer = pick(this.options, 'buffer');
        this.anisotropy = pick(this.options, 'anisotropy');
        this.anisotropicBlur = pick(this.options, 'anisotropicBlur');
        this.background = pick(this.options, 'background');
        this.backside = pick(this.options, 'backside');
        this.backsideThickness = pick(this.options, 'backsideThickness');
        this.backsideEnvMapIntensity = pick(this.options, 'backsideEnvMapIntensity');
        this.transmissionSampler = pick(this.options, 'transmissionSampler');
        this.transmission = pick(this.options, 'transmission');
        this.thickness = pick(this.options, 'thickness');
        this.side = pick(this.options, 'side');
        this.materialRef = viewChild('material');
        this.backResolution = computed(() => this.backsideResolution() || this.resolution());
        this.fboBack = injectFBO(() => ({ width: this.backResolution() }));
        this.fboMain = injectFBO(() => ({ width: this.resolution() }));
        this.bufferTexture = computed(() => this.buffer() || this.fboMain().texture);
        this.anisotropicBlurOption = computed(() => this.anisotropicBlur() || this.anisotropy());
        this.discardMaterial = new MeshDiscardMaterial();
        this.material = computed(() => {
            const previousMaterial = untracked(this.materialRef)?.nativeElement;
            if (previousMaterial) {
                previousMaterial.dispose();
                delete previousMaterial['__ngt__'];
                delete previousMaterial['__ngt_renderer__'];
            }
            const [samples, transmissionSampler] = [this.samples(), this.transmissionSampler()];
            return new MeshTransmissionMaterial({ samples, transmissionSampler });
        });
        inject(DestroyRef).onDestroy(() => {
            const material = this.materialRef()?.nativeElement;
            if (material) {
                material.dispose();
                delete material['__ngt__'];
                delete material['__ngt_renderer__'];
            }
        });
        let oldBg;
        let oldEnvMapIntensity;
        let oldTone;
        let parent;
        injectBeforeRender(({ clock, gl, scene, camera }) => {
            const material = this.materialRef()?.nativeElement;
            if (!material)
                return;
            const [fboMain, fboBack, transmissionSampler, background, backside, backsideThickness, backsideEnvMapIntensity, thickness, side,] = [
                this.fboMain(),
                this.fboBack(),
                this.transmissionSampler(),
                this.background(),
                this.backside(),
                this.backsideThickness(),
                this.backsideEnvMapIntensity(),
                this.thickness(),
                this.side(),
            ];
            material.time = clock.getElapsedTime();
            // Render only if the buffer matches the built-in and no transmission sampler is set
            // @ts-expect-error - we know material.buffer is not just undefined | null
            if (material.buffer === fboMain.texture && !transmissionSampler) {
                parent = getLocalState(material)?.parent();
                if (parent) {
                    // Save defaults
                    oldTone = gl.toneMapping;
                    oldBg = scene.background;
                    oldEnvMapIntensity = material.envMapIntensity;
                    // Switch off tonemapping lest it double tone maps
                    // Save the current background and set the HDR as the new BG
                    // Use discardmaterial, the parent will be invisible, but it's shadows will still be cast
                    gl.toneMapping = NoToneMapping;
                    if (background)
                        scene.background = background;
                    parent.material = this.discardMaterial;
                    if (backside) {
                        // Render into the backside buffer
                        gl.setRenderTarget(fboBack);
                        gl.render(scene, camera);
                        // And now prepare the material for the main render using the backside buffer
                        parent.material = material;
                        Object.assign(parent.material, {
                            buffer: fboBack.texture,
                            thickness: backsideThickness,
                            side: BackSide,
                            envMapIntensity: backsideEnvMapIntensity,
                        });
                    }
                    // Render into the main buffer
                    gl.setRenderTarget(fboMain);
                    gl.render(scene, camera);
                    parent.material = material;
                    Object.assign(parent.material, {
                        buffer: fboMain.texture,
                        thickness,
                        side,
                        envMapIntensity: oldEnvMapIntensity,
                    });
                    // Set old state back
                    scene.background = oldBg;
                    gl.setRenderTarget(null);
                    gl.toneMapping = oldTone;
                }
            }
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsMeshTransmissionMaterial, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsMeshTransmissionMaterial, isStandalone: true, selector: "ngts-mesh-transmission-material", inputs: { attach: { classPropertyName: "attach", publicName: "attach", isSignal: true, isRequired: false, transformFunction: null }, options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "materialRef", first: true, predicate: ["material"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-primitive
			*args="[material()]"
			#material
			[attach]="attach()"
			[parameters]="parameters()"
			[_transmission]="transmission()"
			[transmission]="transmissionSampler() ? transmission() : 0"
			[buffer]="bufferTexture()"
			[anisotropicBlur]="anisotropicBlurOption()"
			[side]="side()"
			[thickness]="thickness()"
		>
			<ng-content />
		</ngt-primitive>
	`, isInline: true, dependencies: [{ kind: "directive", type: NgtArgs, selector: "ng-template[args]", inputs: ["args"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsMeshTransmissionMaterial, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-mesh-transmission-material',
                    template: `
		<ngt-primitive
			*args="[material()]"
			#material
			[attach]="attach()"
			[parameters]="parameters()"
			[_transmission]="transmission()"
			[transmission]="transmissionSampler() ? transmission() : 0"
			[buffer]="bufferTexture()"
			[anisotropicBlur]="anisotropicBlurOption()"
			[side]="side()"
			[thickness]="thickness()"
		>
			<ng-content />
		</ngt-primitive>
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    imports: [NgtArgs],
                }]
        }], ctorParameters: () => [] });

const defaultOptions = {
    speed: 1,
};
class NgtsMeshWobbleMaterial {
    constructor() {
        this.attach = input('material');
        this.options = input(defaultOptions, { transform: mergeInputs(defaultOptions) });
        this.parameters = omit(this.options, ['speed']);
        this.material = new MeshWobbleMaterial();
        inject(DestroyRef).onDestroy(() => this.material.dispose());
        injectBeforeRender(({ clock }) => {
            const material = this.material;
            material.time = clock.elapsedTime * this.options().speed;
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsMeshWobbleMaterial, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.1.0", version: "19.1.4", type: NgtsMeshWobbleMaterial, isStandalone: true, selector: "ngts-mesh-wobble-material", inputs: { attach: { classPropertyName: "attach", publicName: "attach", isSignal: true, isRequired: false, transformFunction: null }, options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, ngImport: i0, template: `
		<ngt-primitive *args="[material]" [parameters]="parameters()" [attach]="attach()">
			<ng-content />
		</ngt-primitive>
	`, isInline: true, dependencies: [{ kind: "directive", type: NgtArgs, selector: "ng-template[args]", inputs: ["args"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsMeshWobbleMaterial, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-mesh-wobble-material',
                    template: `
		<ngt-primitive *args="[material]" [parameters]="parameters()" [attach]="attach()">
			<ng-content />
		</ngt-primitive>
	`,
                    imports: [NgtArgs],
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }], ctorParameters: () => [] });

class NgtsPointMaterial {
    constructor() {
        this.attach = input('material');
        this.options = input({});
        this.material = new PointMaterial(this.options());
        inject(DestroyRef).onDestroy(() => this.material.dispose());
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsPointMaterial, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.1.0", version: "19.1.4", type: NgtsPointMaterial, isStandalone: true, selector: "ngts-point-material", inputs: { attach: { classPropertyName: "attach", publicName: "attach", isSignal: true, isRequired: false, transformFunction: null }, options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, ngImport: i0, template: `
		<ngt-primitive *args="[material]" [attach]="attach()" [parameters]="options()">
			<ng-content />
		</ngt-primitive>
	`, isInline: true, dependencies: [{ kind: "directive", type: NgtArgs, selector: "ng-template[args]", inputs: ["args"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsPointMaterial, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-point-material',
                    template: `
		<ngt-primitive *args="[material]" [attach]="attach()" [parameters]="options()">
			<ng-content />
		</ngt-primitive>
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    imports: [NgtArgs],
                }]
        }], ctorParameters: () => [] });

/**
 * Generated bundle index. Do not edit.
 */

export { ManagePortalScene, NgtsCustomShaderMaterial, NgtsMeshDistortMaterial, NgtsMeshPortalMaterial, NgtsMeshReflectorMaterial, NgtsMeshRefractionMaterial, NgtsMeshTransmissionMaterial, NgtsMeshWobbleMaterial, NgtsPointMaterial };
//# sourceMappingURL=angular-three-soba-materials.mjs.map
