import * as i0 from '@angular/core';
import { computed, isSignal, signal, effect, untracked, Directive, input, viewChild, Component, CUSTOM_ELEMENTS_SCHEMA, ChangeDetectionStrategy, inject, DestroyRef, TemplateRef, ViewContainerRef, output, Renderer2, model, ElementRef } from '@angular/core';
import { injectBeforeRender, resolveRef, injectStore, getLocalState, NgtArgs, omit, pick, extend, applyProps, is, NgtHTML, addEffect, addAfterEffect, checkUpdate } from 'angular-three';
import { assertInjector } from 'ngxtension/assert-injector';
import { AnimationMixer, BufferAttribute, REVISION, Mesh, BoxGeometry, MeshNormalMaterial, AxesHelper, Vector3, Euler, Object3D, WebGLRenderTarget, LinearFilter, HalfFloatType, DepthTexture, FloatType, DepthFormat, UnsignedShortType, DoubleSide, Group, PlaneGeometry, ShaderMaterial, Vector2, OrthographicCamera, PerspectiveCamera, WebGLCubeRenderTarget, CubeCamera, InstancedBufferAttribute, Color } from 'three';
import { mergeInputs } from 'ngxtension/inject-inputs';
import { DecalGeometry, MeshSurfaceSampler } from 'three-stdlib';
import { NgTemplateOutlet } from '@angular/common';

/**
 * Use afterNextRender
 */
function injectAnimations(animations, object, { injector } = {}) {
    return assertInjector(injectAnimations, injector, () => {
        const mixer = new AnimationMixer(null);
        injectBeforeRender(({ delta }) => {
            if (!mixer.getRoot())
                return;
            mixer.update(delta);
        });
        let cached = {};
        const actions = {};
        const clips = [];
        const names = [];
        const actualObject = computed(() => {
            if (isSignal(object) || typeof object === 'function') {
                return resolveRef(object());
            }
            return resolveRef(object);
        });
        const ready = signal(false);
        effect((onCleanup) => {
            const obj = actualObject();
            if (!obj)
                return;
            Object.assign(mixer, { _root: obj });
            const maybeAnimationClips = animations();
            if (!maybeAnimationClips)
                return;
            const animationClips = Array.isArray(maybeAnimationClips) ? maybeAnimationClips : maybeAnimationClips.animations;
            for (let i = 0; i < animationClips.length; i++) {
                const clip = animationClips[i];
                names.push(clip.name);
                clips.push(clip);
                if (!actions[clip.name]) {
                    Object.defineProperty(actions, clip.name, {
                        enumerable: true,
                        get: () => {
                            if (!cached[clip.name]) {
                                cached[clip.name] = mixer.clipAction(clip, obj);
                            }
                            return cached[clip.name];
                        },
                    });
                }
            }
            if (!untracked(ready)) {
                ready.set(true);
            }
            onCleanup(() => {
                // clear cached
                cached = {};
                // stop all actions
                mixer.stopAllAction();
                // uncache actions
                Object.values(actions).forEach((action) => {
                    mixer.uncacheAction(action, obj);
                });
            });
        });
        const result = { ready, clips, mixer, actions, names };
        Object.defineProperty(result, 'isReady', { get: ready });
        return result;
    });
}

class NgtsBakeShadows {
    constructor() {
        const store = injectStore();
        const gl = store.select('gl');
        effect((onCleanup) => {
            const _gl = gl();
            _gl.shadowMap.autoUpdate = false;
            _gl.shadowMap.needsUpdate = true;
            onCleanup(() => {
                _gl.shadowMap.autoUpdate = _gl.shadowMap.needsUpdate = true;
            });
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsBakeShadows, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.1.4", type: NgtsBakeShadows, isStandalone: true, selector: "ngts-bake-shadows", ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsBakeShadows, decorators: [{
            type: Directive,
            args: [{ selector: 'ngts-bake-shadows' }]
        }], ctorParameters: () => [] });

/**
 * Used exclusively as a child of a BufferGeometry.
 * Computes the BufferAttribute by calling the `compute` function
 * and attaches the attribute to the geometry.
 */
class NgtsComputedAttribute {
    constructor() {
        this.compute = input.required();
        this.name = input.required();
        this.options = input({});
        this.bufferAttribute = new BufferAttribute(new Float32Array(0), 1);
        this.attributeRef = viewChild('attribute');
        effect(() => {
            const bufferAttribute = this.attributeRef()?.nativeElement;
            if (!bufferAttribute)
                return;
            const localState = getLocalState(bufferAttribute);
            if (!localState)
                return;
            const geometry = bufferAttribute.parent ?? localState.parent();
            const attribute = this.compute()(geometry);
            bufferAttribute.copy(attribute);
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsComputedAttribute, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsComputedAttribute, isStandalone: true, selector: "ngts-computed-attribute", inputs: { compute: { classPropertyName: "compute", publicName: "compute", isSignal: true, isRequired: true, transformFunction: null }, name: { classPropertyName: "name", publicName: "name", isSignal: true, isRequired: true, transformFunction: null }, options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "attributeRef", first: true, predicate: ["attribute"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-primitive #attribute *args="[bufferAttribute]" [attach]="['attributes', name()]" [parameters]="options()">
			<ng-content />
		</ngt-primitive>
	`, isInline: true, dependencies: [{ kind: "directive", type: NgtArgs, selector: "ng-template[args]", inputs: ["args"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsComputedAttribute, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-computed-attribute',
                    template: `
		<ngt-primitive #attribute *args="[bufferAttribute]" [attach]="['attributes', name()]" [parameters]="options()">
			<ng-content />
		</ngt-primitive>
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    imports: [NgtArgs],
                }]
        }], ctorParameters: () => [] });

function getVersion() {
    return parseInt(REVISION.replace(/\D+/g, ''));
}

const defaultOptions$1 = {
    polygonOffsetFactor: -10,
    debug: false,
    depthTest: false,
};
class NgtsDecal {
    constructor() {
        this.mesh = input();
        this.options = input(defaultOptions$1, { transform: mergeInputs(defaultOptions$1) });
        this.parameters = omit(this.options, [
            'debug',
            'map',
            'depthTest',
            'polygonOffsetFactor',
            'position',
            'scale',
            'rotation',
        ]);
        this.meshRef = viewChild.required('mesh');
        this.helperRef = viewChild('helper');
        this.map = pick(this.options, 'map');
        this.depthTest = pick(this.options, 'depthTest');
        this.polygonOffsetFactor = pick(this.options, 'polygonOffsetFactor');
        this.debug = pick(this.options, 'debug');
        this.position = pick(this.options, 'position');
        this.rotation = pick(this.options, 'rotation');
        this.scale = pick(this.options, 'scale');
        extend({ Mesh, BoxGeometry, MeshNormalMaterial, AxesHelper });
        effect((onCleanup) => {
            const thisMesh = this.meshRef().nativeElement;
            const localState = getLocalState(thisMesh);
            if (!localState)
                return;
            const parent = resolveRef(this.mesh()) || localState.parent();
            if (parent && !parent.isMesh) {
                throw new Error('<ngts-decal> must have a Mesh as parent or specify its "mesh" input');
            }
            if (!parent)
                return;
            const parentLocalState = getLocalState(parent);
            if (!parentLocalState)
                return;
            // track parent's children
            const parentNonObjects = parentLocalState.nonObjects();
            if (!parentNonObjects || !parentNonObjects.length) {
                return;
            }
            // if parent geometry doesn't have its attributes populated properly, we skip
            if (!parent.geometry?.attributes?.['position']) {
                return;
            }
            const [position, rotation, scale] = [this.position(), this.rotation(), this.scale()];
            const state = { position: new Vector3(), rotation: new Euler(), scale: new Vector3(1, 1, 1) };
            applyProps(state, { position, scale });
            // zero out the parents matrix world for this operation
            const matrixWorld = parent.matrixWorld.clone();
            parent.matrixWorld.identity();
            if (!rotation || typeof rotation === 'number') {
                const o = new Object3D();
                o.position.copy(state.position);
                o.lookAt(parent.position);
                if (typeof rotation === 'number')
                    o.rotateZ(rotation);
                applyProps(state, { rotation: o.rotation });
            }
            else {
                applyProps(state, { rotation });
            }
            thisMesh.geometry = new DecalGeometry(parent, state.position, state.rotation, state.scale);
            const helper = this.helperRef()?.nativeElement;
            if (helper) {
                applyProps(helper, state);
                // Prevent the helpers from blocking rays
                helper.traverse((child) => (child.raycast = () => null));
            }
            // Reset parents matrix-world
            parent.matrixWorld = matrixWorld;
            onCleanup(() => {
                thisMesh.geometry.dispose();
            });
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsDecal, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "19.1.4", type: NgtsDecal, isStandalone: true, selector: "ngts-decal", inputs: { mesh: { classPropertyName: "mesh", publicName: "mesh", isSignal: true, isRequired: false, transformFunction: null }, options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "meshRef", first: true, predicate: ["mesh"], descendants: true, isSignal: true }, { propertyName: "helperRef", first: true, predicate: ["helper"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-mesh #mesh [parameters]="parameters()">
			<ngt-value [rawValue]="true" attach="material.transparent" />
			<ngt-value [rawValue]="true" attach="material.polygonOffset" />
			<ngt-value [rawValue]="map()" attach="material.map" />

			<ng-content>
				<!-- we only want to pass through these material properties if they don't use a custom material -->
				<ngt-value [rawValue]="polygonOffsetFactor()" attach="material.polygonOffsetFactor" />
				<ngt-value [rawValue]="depthTest()" attach="material.depthTest" />
			</ng-content>

			@if (debug()) {
				<ngt-mesh #helper>
					<ngt-box-geometry />
					<ngt-mesh-normal-material [wireframe]="true" />
					<ngt-axes-helper />
				</ngt-mesh>
			}
		</ngt-mesh>
	`, isInline: true, changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsDecal, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-decal',
                    template: `
		<ngt-mesh #mesh [parameters]="parameters()">
			<ngt-value [rawValue]="true" attach="material.transparent" />
			<ngt-value [rawValue]="true" attach="material.polygonOffset" />
			<ngt-value [rawValue]="map()" attach="material.map" />

			<ng-content>
				<!-- we only want to pass through these material properties if they don't use a custom material -->
				<ngt-value [rawValue]="polygonOffsetFactor()" attach="material.polygonOffsetFactor" />
				<ngt-value [rawValue]="depthTest()" attach="material.depthTest" />
			</ng-content>

			@if (debug()) {
				<ngt-mesh #helper>
					<ngt-box-geometry />
					<ngt-mesh-normal-material [wireframe]="true" />
					<ngt-axes-helper />
				</ngt-mesh>
			}
		</ngt-mesh>
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }], ctorParameters: () => [] });

/**
 * NOTE: Sets `BufferAttribute.updateRange` since r159.
 */
const setUpdateRange = (attribute, updateRange) => {
    if ('updateRanges' in attribute) {
        // attribute.updateRanges[0] = updateRange;
        attribute.addUpdateRange(updateRange.offset, updateRange.count);
    }
    else {
        Object.assign(attribute, { updateRange });
    }
};
const LinearEncoding = 3000;
const sRGBEncoding = 3001;

function injectFBO(params = () => ({}), { injector } = {}) {
    return assertInjector(injectFBO, injector, () => {
        const store = injectStore();
        const size = store.select('size');
        const viewport = store.select('viewport');
        const width = computed(() => {
            const { width } = params();
            return typeof width === 'number' ? width : size().width * viewport().dpr;
        });
        const height = computed(() => {
            const { height } = params();
            return typeof height === 'number' ? height : size().height * viewport().dpr;
        });
        const settings = computed(() => {
            const { width, settings } = params();
            const _settings = (typeof width === 'number' ? settings : width) || {};
            if (_settings.samples === undefined) {
                _settings.samples = 0;
            }
            return _settings;
        });
        const target = computed(() => {
            const [{ samples = 0, depth, ...targetSettings }, _width, _height] = [
                untracked(settings),
                untracked(width),
                untracked(height),
            ];
            const target = new WebGLRenderTarget(_width, _height, {
                minFilter: LinearFilter,
                magFilter: LinearFilter,
                type: HalfFloatType,
                ...targetSettings,
            });
            if (depth) {
                target.depthTexture = new DepthTexture(_width, _height, FloatType);
            }
            target.samples = samples;
            return target;
        });
        effect(() => {
            const [{ samples = 0 }, _width, _height, _target] = [settings(), width(), height(), target()];
            _target.setSize(_width, _height);
            if (samples)
                _target.samples = samples;
        });
        inject(DestroyRef).onDestroy(() => target().dispose());
        return target;
    });
}
class NgtsFBO {
    constructor() {
        this.fbo = input({});
        this.template = inject(TemplateRef);
        this.viewContainerRef = inject(ViewContainerRef);
        let ref;
        const fboTarget = injectFBO(() => {
            const { width, height, ...settings } = this.fbo();
            return { width, height, settings };
        });
        effect(() => {
            ref = this.viewContainerRef.createEmbeddedView(this.template, { $implicit: fboTarget });
            ref.detectChanges();
        });
        inject(DestroyRef).onDestroy(() => {
            ref?.destroy();
        });
    }
    static ngTemplateContextGuard(_, ctx) {
        return true;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsFBO, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "17.1.0", version: "19.1.4", type: NgtsFBO, isStandalone: true, selector: "ng-template[fbo]", inputs: { fbo: { classPropertyName: "fbo", publicName: "fbo", isSignal: true, isRequired: false, transformFunction: null } }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsFBO, decorators: [{
            type: Directive,
            args: [{ selector: 'ng-template[fbo]' }]
        }], ctorParameters: () => [] });

function injectDepthBuffer(params = () => ({}), { injector } = {}) {
    return assertInjector(injectDepthBuffer, injector, () => {
        const size = computed(() => params().size || 256);
        const frames = computed(() => params().frames || Infinity);
        const store = injectStore();
        const width = store.select('size', 'width');
        const height = store.select('size', 'height');
        const dpr = store.select('viewport', 'dpr');
        const w = computed(() => size() || width() * dpr());
        const h = computed(() => size() || height() * dpr());
        const depthConfig = computed(() => {
            const depthTexture = new DepthTexture(w(), h());
            depthTexture.format = DepthFormat;
            depthTexture.type = UnsignedShortType;
            return { depthTexture };
        });
        const depthFBO = injectFBO(() => ({ width: w(), height: h(), settings: depthConfig() }));
        let count = 0;
        injectBeforeRender(({ gl, scene, camera }) => {
            if (frames() === Infinity || count < frames()) {
                gl.setRenderTarget(depthFBO());
                gl.render(scene, camera);
                gl.setRenderTarget(null);
                count++;
            }
        });
        return pick(depthFBO, 'depthTexture');
    });
}

const defaultHtmlOptions = {
    occlude: false,
    transform: false,
    castShadow: false,
    receiveShadow: false,
};
class NgtsHTML {
    constructor() {
        this.options = input(defaultHtmlOptions, { transform: mergeInputs(defaultHtmlOptions) });
        this.parameters = omit(this.options, ['occlude', 'castShadow', 'receiveShadow', 'transform']);
        this.groupRef = viewChild.required('group');
        this.occlusionMeshRef = viewChild('occlusionMesh');
        this.occlusionGeometryRef = viewChild('occlusionGeometry');
        this.occlude = pick(this.options, 'occlude');
        this.transform = pick(this.options, 'transform');
        this.castShadow = pick(this.options, 'castShadow');
        this.receiveShadow = pick(this.options, 'receiveShadow');
        this.scale = pick(this.options, 'scale');
        this.isRaycastOcclusion = computed(() => {
            const occlude = this.occlude();
            return (occlude && occlude !== 'blending') || (Array.isArray(occlude) && occlude.length && is.ref(occlude[0]));
        });
        this.shaders = computed(() => {
            const transform = this.transform();
            const vertexShader = !transform
                ? /* language=glsl glsl */ `
          /*
            This shader is from the THREE's SpriteMaterial.
            We need to turn the backing plane into a Sprite
            (make it always face the camera) if "transfrom"
            is false.
          */
          #include <common>

          void main() {
            vec2 center = vec2(0., 1.);
            float rotation = 0.0;

            // This is somewhat arbitrary, but it seems to work well
            // Need to figure out how to derive this dynamically if it even matters
            float size = 0.03;

            vec4 mvPosition = modelViewMatrix * vec4( 0.0, 0.0, 0.0, 1.0 );
            vec2 scale;
            scale.x = length( vec3( modelMatrix[ 0 ].x, modelMatrix[ 0 ].y, modelMatrix[ 0 ].z ) );
            scale.y = length( vec3( modelMatrix[ 1 ].x, modelMatrix[ 1 ].y, modelMatrix[ 1 ].z ) );

            bool isPerspective = isPerspectiveMatrix( projectionMatrix );
            if ( isPerspective ) scale *= - mvPosition.z;

            vec2 alignedPosition = ( position.xy - ( center - vec2( 0.5 ) ) ) * scale * size;
            vec2 rotatedPosition;
            rotatedPosition.x = cos( rotation ) * alignedPosition.x - sin( rotation ) * alignedPosition.y;
            rotatedPosition.y = sin( rotation ) * alignedPosition.x + cos( rotation ) * alignedPosition.y;
            mvPosition.xy += rotatedPosition;

            gl_Position = projectionMatrix * mvPosition;
          }
      `
                : undefined;
            const fragmentShader = /* language=glsl glsl */ `
      void main() {
        gl_FragColor = vec4(0.0, 0.0, 0.0, 0.0);
      }
    `;
            return { vertexShader, fragmentShader };
        });
        this.vertexShader = pick(this.shaders, 'vertexShader');
        this.fragmentShader = pick(this.shaders, 'fragmentShader');
        this.DoubleSide = DoubleSide;
        extend({ Group, Mesh, PlaneGeometry, ShaderMaterial });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsHTML, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "19.1.4", type: NgtsHTML, isStandalone: true, selector: "ngts-html", inputs: { options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "groupRef", first: true, predicate: ["group"], descendants: true, isSignal: true }, { propertyName: "occlusionMeshRef", first: true, predicate: ["occlusionMesh"], descendants: true, isSignal: true }, { propertyName: "occlusionGeometryRef", first: true, predicate: ["occlusionGeometry"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-group #group [parameters]="parameters()">
			@if (occlude() && !isRaycastOcclusion()) {
				<ngt-mesh #occlusionMesh [castShadow]="castShadow()" [receiveShadow]="receiveShadow()">
					<ng-content select="[data-occlusion-geometry]">
						<ngt-plane-geometry #occlusionGeometry />
					</ng-content>
					<ng-content select="[data-occlusion-material]">
						<ngt-shader-material
							[side]="DoubleSide"
							[vertexShader]="vertexShader()"
							[fragmentShader]="fragmentShader()"
						/>
					</ng-content>
				</ngt-mesh>
			}
		</ngt-group>

		<ng-content />
	`, isInline: true, changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsHTML, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-html',
                    template: `
		<ngt-group #group [parameters]="parameters()">
			@if (occlude() && !isRaycastOcclusion()) {
				<ngt-mesh #occlusionMesh [castShadow]="castShadow()" [receiveShadow]="receiveShadow()">
					<ng-content select="[data-occlusion-geometry]">
						<ngt-plane-geometry #occlusionGeometry />
					</ng-content>
					<ng-content select="[data-occlusion-material]">
						<ngt-shader-material
							[side]="DoubleSide"
							[vertexShader]="vertexShader()"
							[fragmentShader]="fragmentShader()"
						/>
					</ng-content>
				</ngt-mesh>
			}
		</ngt-group>

		<ng-content />
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }], ctorParameters: () => [] });

const v1 = new Vector3();
const v2 = new Vector3();
const v3 = new Vector3();
const v4 = new Vector2();
function defaultCalculatePosition(el, camera, size) {
    const objectPos = v1.setFromMatrixPosition(el.matrixWorld);
    objectPos.project(camera);
    const widthHalf = size.width / 2;
    const heightHalf = size.height / 2;
    return [objectPos.x * widthHalf + widthHalf, -(objectPos.y * heightHalf) + heightHalf];
}
function isObjectBehindCamera(el, camera) {
    const objectPos = v1.setFromMatrixPosition(el.matrixWorld);
    const cameraPos = v2.setFromMatrixPosition(camera.matrixWorld);
    const deltaCamObj = objectPos.sub(cameraPos);
    const camDir = camera.getWorldDirection(v3);
    return deltaCamObj.angleTo(camDir) > Math.PI / 2;
}
function isObjectVisible(el, camera, raycaster, occlude) {
    const elPos = v1.setFromMatrixPosition(el.matrixWorld);
    const screenPos = elPos.clone();
    screenPos.project(camera);
    v4.set(screenPos.x, screenPos.y);
    raycaster.setFromCamera(v4, camera);
    const intersects = raycaster.intersectObjects(occlude, true);
    if (intersects.length) {
        const intersectionDistance = intersects[0].distance;
        const pointDistance = elPos.distanceTo(raycaster.ray.origin);
        return pointDistance < intersectionDistance;
    }
    return true;
}
function objectScale(el, camera) {
    if (camera instanceof OrthographicCamera)
        return camera.zoom;
    if (camera instanceof PerspectiveCamera) {
        const objectPos = v1.setFromMatrixPosition(el.matrixWorld);
        const cameraPos = v2.setFromMatrixPosition(camera.matrixWorld);
        const vFOV = (camera.fov * Math.PI) / 180;
        const dist = objectPos.distanceTo(cameraPos);
        const scaleFOV = 2 * Math.tan(vFOV / 2) * dist;
        return 1 / scaleFOV;
    }
    return 1;
}
function objectZIndex(el, camera, zIndexRange) {
    if (camera instanceof PerspectiveCamera || camera instanceof OrthographicCamera) {
        const objectPos = v1.setFromMatrixPosition(el.matrixWorld);
        const cameraPos = v2.setFromMatrixPosition(camera.matrixWorld);
        const dist = objectPos.distanceTo(cameraPos);
        const A = (zIndexRange[1] - zIndexRange[0]) / (camera.far - camera.near);
        const B = zIndexRange[1] - A * camera.far;
        return Math.round(A * dist + B);
    }
    return undefined;
}
function epsilon(value) {
    return Math.abs(value) < 1e-10 ? 0 : value;
}
function getCSSMatrix(matrix, multipliers, prepend = '') {
    let matrix3d = 'matrix3d(';
    for (let i = 0; i !== 16; i++) {
        matrix3d += epsilon(multipliers[i] * matrix.elements[i]) + (i !== 15 ? ',' : ')');
    }
    return prepend + matrix3d;
}
const getCameraCSSMatrix = ((multipliers) => {
    return (matrix) => getCSSMatrix(matrix, multipliers);
})([1, -1, 1, 1, 1, -1, 1, 1, 1, -1, 1, 1, 1, -1, 1, 1]);
const getObjectCSSMatrix = ((scaleMultipliers) => {
    return (matrix, factor) => getCSSMatrix(matrix, scaleMultipliers(factor), 'translate(-50%,-50%)');
})((f) => [1 / f, 1 / f, 1 / f, 1, -1 / f, -1 / f, -1 / f, -1, 1 / f, 1 / f, 1 / f, 1, 1, 1, 1, 1]);

const defaultHtmlContentOptions = {
    eps: 0.001,
    zIndexRange: [16777271, 0],
    pointerEvents: 'auto',
    calculatePosition: defaultCalculatePosition,
    containerClass: '',
    containerStyle: {},
    center: false,
    prepend: false,
    fullscreen: false,
    sprite: false,
};
class NgtsHTMLContent extends NgtHTML {
    constructor() {
        super();
        this.options = input(defaultHtmlContentOptions, {
            transform: mergeInputs(defaultHtmlContentOptions),
            alias: 'ngtsHTMLContent',
        });
        this.occluded = output();
        this.html = inject(NgtsHTML);
        this.transformOuterRef = viewChild('transformOuter');
        this.transformInnerRef = viewChild('transformInner');
        this.containerRef = viewChild('container');
        this.gl = this.store.select('gl');
        this.events = this.store.select('events');
        this.camera = this.store.select('camera');
        this.scene = this.store.select('scene');
        this.size = this.store.select('size');
        this.parent = pick(this.options, 'parent');
        this.zIndexRange = pick(this.options, 'zIndexRange');
        this.calculatePosition = pick(this.options, 'calculatePosition');
        this.prepend = pick(this.options, 'prepend');
        this.center = pick(this.options, 'center');
        this.fullscreen = pick(this.options, 'fullscreen');
        this.pointerEvents = pick(this.options, 'pointerEvents');
        this.containerClass = pick(this.options, 'containerClass');
        this.containerStyle = pick(this.options, 'containerStyle');
        this.target = computed(() => {
            const parent = resolveRef(this.parent());
            if (parent)
                return parent;
            return (this.events().connected || this.gl().domElement.parentNode);
        });
        const renderer = inject(Renderer2);
        let isMeshSizeSet = false;
        effect(() => {
            const [occlude, canvasEl, zIndexRange] = [
                this.html.occlude(),
                untracked(this.gl).domElement,
                untracked(this.zIndexRange),
            ];
            if (occlude && occlude === 'blending') {
                renderer.setStyle(canvasEl, 'z-index', `${Math.floor(zIndexRange[0] / 2)}`);
                renderer.setStyle(canvasEl, 'position', 'absolute');
                renderer.setStyle(canvasEl, 'pointer-events', 'none');
            }
            else {
                renderer.removeStyle(canvasEl, 'z-index');
                renderer.removeStyle(canvasEl, 'position');
                renderer.removeStyle(canvasEl, 'pointer-events');
            }
        });
        effect((onCleanup) => {
            const [transform, target, hostEl, prepend, scene, calculatePosition, group, size, camera] = [
                this.html.transform(),
                this.target(),
                this.host.nativeElement,
                untracked(this.prepend),
                untracked(this.scene),
                untracked(this.calculatePosition),
                untracked(this.html.groupRef).nativeElement,
                untracked(this.size),
                untracked(this.camera),
            ];
            scene.updateMatrixWorld();
            renderer.setStyle(hostEl, 'position', 'absolute');
            renderer.setStyle(hostEl, 'top', '0');
            renderer.setStyle(hostEl, 'left', '0');
            if (transform) {
                renderer.setStyle(hostEl, 'pointer-events', 'none');
                renderer.setStyle(hostEl, 'overflow', 'hidden');
                renderer.removeStyle(hostEl, 'transform');
                renderer.removeStyle(hostEl, 'transform-origin');
            }
            else {
                const vec = calculatePosition(group, camera, size);
                renderer.setStyle(hostEl, 'transform', `translate3d(${vec[0]}px,${vec[1]}px,0)`);
                renderer.setStyle(hostEl, 'transform-origin', '0 0');
                renderer.removeStyle(hostEl, 'pointer-events');
                renderer.removeStyle(hostEl, 'overflow');
            }
            if (prepend)
                target.prepend(hostEl);
            else
                target.appendChild(hostEl);
            onCleanup(() => {
                if (target)
                    target.removeChild(hostEl);
            });
        });
        effect(() => {
            const _ = [this.options(), this.html.options()];
            isMeshSizeSet = false;
        });
        let visible = true;
        let oldZoom = 0;
        let oldPosition = [0, 0];
        injectBeforeRender(({ camera: rootCamera }) => {
            const [hostEl, transformOuterEl, transformInnerEl, group, occlusionMesh, occlusionGeometry, isRaycastOcclusion, { camera, size, viewport, raycaster, scene }, { calculatePosition, eps, zIndexRange, sprite, distanceFactor }, { transform, occlude, scale },] = [
                this.host.nativeElement,
                this.transformOuterRef()?.nativeElement,
                this.transformInnerRef()?.nativeElement,
                this.html.groupRef().nativeElement,
                this.html.occlusionMeshRef()?.nativeElement,
                this.html.occlusionGeometryRef()?.nativeElement,
                this.html.isRaycastOcclusion(),
                this.store.snapshot,
                this.options(),
                this.html.options(),
            ];
            if (group) {
                camera.updateMatrixWorld();
                group.updateWorldMatrix(true, false);
                const vec = transform ? oldPosition : calculatePosition(group, camera, size);
                if (transform ||
                    Math.abs(oldZoom - camera.zoom) > eps ||
                    Math.abs(oldPosition[0] - vec[0]) > eps ||
                    Math.abs(oldPosition[1] - vec[1]) > eps) {
                    const isBehindCamera = isObjectBehindCamera(group, camera);
                    let raytraceTarget = false;
                    if (isRaycastOcclusion) {
                        if (Array.isArray(occlude)) {
                            raytraceTarget = occlude.map((item) => resolveRef(item));
                        }
                        else if (occlude !== 'blending') {
                            raytraceTarget = [scene];
                        }
                    }
                    const previouslyVisible = visible;
                    if (raytraceTarget) {
                        const isVisible = isObjectVisible(group, camera, raycaster, raytraceTarget);
                        visible = isVisible && !isBehindCamera;
                    }
                    else {
                        visible = !isBehindCamera;
                    }
                    if (previouslyVisible !== visible) {
                        if (this.occluded['listeners'])
                            this.occluded.emit(!visible);
                        else
                            renderer.setStyle(hostEl, 'display', visible ? 'block' : 'none');
                    }
                    const halfRange = Math.floor(zIndexRange[0] / 2);
                    const zRange = occlude
                        ? isRaycastOcclusion //
                            ? [zIndexRange[0], halfRange]
                            : [halfRange - 1, 0]
                        : zIndexRange;
                    renderer.setStyle(hostEl, 'z-index', `${objectZIndex(group, camera, zRange)}`);
                    if (transform) {
                        const [widthHalf, heightHalf] = [size.width / 2, size.height / 2];
                        const fov = camera.projectionMatrix.elements[5] * heightHalf;
                        const { isOrthographicCamera, top, left, bottom, right } = camera;
                        const cameraMatrix = getCameraCSSMatrix(camera.matrixWorldInverse);
                        const cameraTransform = isOrthographicCamera
                            ? `scale(${fov})translate(${epsilon(-(right + left) / 2)}px,${epsilon((top + bottom) / 2)}px)`
                            : `translateZ(${fov}px)`;
                        let matrix = group.matrixWorld;
                        if (sprite) {
                            matrix = camera.matrixWorldInverse.clone().transpose().copyPosition(matrix).scale(group.scale);
                            matrix.elements[3] = matrix.elements[7] = matrix.elements[11] = 0;
                            matrix.elements[15] = 1;
                        }
                        renderer.setStyle(hostEl, 'width', size.width + 'px');
                        renderer.setStyle(hostEl, 'height', size.height + 'px');
                        renderer.setStyle(hostEl, 'perspective', isOrthographicCamera ? '' : `${fov}px`);
                        if (transformOuterEl && transformInnerEl) {
                            renderer.setStyle(transformOuterEl, 'transform', `${cameraTransform}${cameraMatrix}translate(${widthHalf}px,${heightHalf}px)`);
                            renderer.setStyle(transformInnerEl, 'transform', getObjectCSSMatrix(matrix, 1 / ((distanceFactor || 10) / 400)));
                        }
                    }
                    else {
                        const scale = distanceFactor === undefined ? 1 : objectScale(group, camera) * distanceFactor;
                        renderer.setStyle(hostEl, 'transform', `translate3d(${vec[0]}px,${vec[1]}px,0) scale(${scale})`);
                    }
                    oldPosition = vec;
                    oldZoom = camera.zoom;
                }
            }
            if (!isRaycastOcclusion && occlusionMesh && !isMeshSizeSet) {
                if (transform) {
                    if (transformOuterEl) {
                        const el = transformOuterEl.children[0];
                        if (el?.clientWidth && el?.clientHeight) {
                            const { isOrthographicCamera } = camera;
                            if (isOrthographicCamera || occlusionGeometry) {
                                if (scale) {
                                    if (!Array.isArray(scale)) {
                                        occlusionMesh.scale.setScalar(1 / scale);
                                    }
                                    else if (scale instanceof Vector3) {
                                        occlusionMesh.scale.copy(scale.clone().divideScalar(1));
                                    }
                                    else {
                                        occlusionMesh.scale.set(1 / scale[0], 1 / scale[1], 1 / scale[2]);
                                    }
                                }
                            }
                            else {
                                const ratio = (distanceFactor || 10) / 400;
                                const w = el.clientWidth * ratio;
                                const h = el.clientHeight * ratio;
                                occlusionMesh.scale.set(w, h, 1);
                            }
                            isMeshSizeSet = true;
                        }
                    }
                }
                else {
                    const ele = hostEl.children[0];
                    if (ele?.clientWidth && ele?.clientHeight) {
                        const ratio = 1 / viewport.factor;
                        const w = ele.clientWidth * ratio;
                        const h = ele.clientHeight * ratio;
                        occlusionMesh.scale.set(w, h, 1);
                        isMeshSizeSet = true;
                    }
                    occlusionMesh.lookAt(rootCamera.position);
                }
            }
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsHTMLContent, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "19.1.4", type: NgtsHTMLContent, isStandalone: true, selector: "[ngtsHTMLContent]", inputs: { options: { classPropertyName: "options", publicName: "ngtsHTMLContent", isSignal: true, isRequired: false, transformFunction: null } }, outputs: { occluded: "occluded" }, host: { attributes: { "data-ngts-html-content": "" } }, viewQueries: [{ propertyName: "transformOuterRef", first: true, predicate: ["transformOuter"], descendants: true, isSignal: true }, { propertyName: "transformInnerRef", first: true, predicate: ["transformInner"], descendants: true, isSignal: true }, { propertyName: "containerRef", first: true, predicate: ["container"], descendants: true, isSignal: true }], usesInheritance: true, ngImport: i0, template: `
		@if (html.transform()) {
			<div
				#transformOuter
				style="position: absolute; top: 0; left: 0; transform-style: preserve-3d; pointer-events: none;"
				[style.width.px]="size().width"
				[style.height.px]="size().height"
			>
				<div #transformInner style="position: absolute" [style.pointer-events]="pointerEvents()">
					<div #container [class]="containerClass()" [style]="containerStyle()">
						<ng-container [ngTemplateOutlet]="content" />
					</div>
				</div>
			</div>
		} @else {
			<div
				#container
				style="position:absolute"
				[style.transform]="center() ? 'translate3d(-50%,-50%,0)' : 'none'"
				[style.top]="fullscreen() ? -size().height / 2 + 'px' : 'unset'"
				[style.left]="fullscreen() ? -size().width / 2 + 'px' : 'unset'"
				[style.width]="fullscreen() ? size().width : 'unset'"
				[style.height]="fullscreen() ? size().height : 'unset'"
				[class]="containerClass()"
				[style]="containerStyle()"
			>
				<ng-container [ngTemplateOutlet]="content" />
			</div>
		}

		<ng-template #content>
			<ng-content />
		</ng-template>
	`, isInline: true, dependencies: [{ kind: "directive", type: NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet", "ngTemplateOutletInjector"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsHTMLContent, decorators: [{
            type: Component,
            args: [{
                    selector: '[ngtsHTMLContent]',
                    template: `
		@if (html.transform()) {
			<div
				#transformOuter
				style="position: absolute; top: 0; left: 0; transform-style: preserve-3d; pointer-events: none;"
				[style.width.px]="size().width"
				[style.height.px]="size().height"
			>
				<div #transformInner style="position: absolute" [style.pointer-events]="pointerEvents()">
					<div #container [class]="containerClass()" [style]="containerStyle()">
						<ng-container [ngTemplateOutlet]="content" />
					</div>
				</div>
			</div>
		} @else {
			<div
				#container
				style="position:absolute"
				[style.transform]="center() ? 'translate3d(-50%,-50%,0)' : 'none'"
				[style.top]="fullscreen() ? -size().height / 2 + 'px' : 'unset'"
				[style.left]="fullscreen() ? -size().width / 2 + 'px' : 'unset'"
				[style.width]="fullscreen() ? size().width : 'unset'"
				[style.height]="fullscreen() ? size().height : 'unset'"
				[class]="containerClass()"
				[style]="containerStyle()"
			>
				<ng-container [ngTemplateOutlet]="content" />
			</div>
		}

		<ng-template #content>
			<ng-content />
		</ng-template>
	`,
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    imports: [NgTemplateOutlet],
                    host: { 'data-ngts-html-content': '' },
                }]
        }], ctorParameters: () => [] });

function injectIntersect(object, { injector, source = signal(false) } = {}) {
    return assertInjector(injectIntersect, injector, () => {
        let check = false;
        let temp = false;
        effect((onCleanup) => {
            const obj = resolveRef(object());
            if (!obj)
                return;
            // Stamp out frustum check pre-emptively
            const unsub1 = addEffect(() => {
                check = false;
                return true;
            });
            // If the object is inside the frustum three will call onRender
            const oldOnRender = obj.onBeforeRender.bind(obj);
            obj.onBeforeRender = () => (check = true);
            // Compare the check value against the temp value, if it differs set state
            const unsub2 = addAfterEffect(() => {
                if (check !== temp)
                    source.set((temp = check));
                return true;
            });
            onCleanup(() => {
                obj.onBeforeRender = oldOnRender;
                unsub1();
                unsub2();
            });
        });
        return source.asReadonly();
    });
}
class NgtsIntersect {
    constructor() {
        this.intersect = model(false);
        const host = inject(ElementRef);
        injectIntersect(() => host, { source: this.intersect });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsIntersect, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "17.1.0", version: "19.1.4", type: NgtsIntersect, isStandalone: true, selector: "[intersect]", inputs: { intersect: { classPropertyName: "intersect", publicName: "intersect", isSignal: true, isRequired: false, transformFunction: null } }, outputs: { intersect: "intersectChange" }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsIntersect, decorators: [{
            type: Directive,
            args: [{ selector: '[intersect]' }]
        }], ctorParameters: () => [] });

class NgtsPreload {
    constructor() {
        this.all = model();
        this.scene = model();
        this.camera = model();
        this.store = injectStore();
        this.gl = this.store.select('gl');
        this.defaultScene = this.store.select('scene');
        this.defaultCamera = this.store.select('camera');
        this.trueScene = computed(() => {
            const scene = this.scene();
            if (scene)
                return resolveRef(scene);
            return this.defaultScene();
        });
        this.trueCamera = computed(() => {
            const camera = this.camera();
            if (camera)
                return resolveRef(camera);
            return this.defaultCamera();
        });
        effect(() => {
            const invisible = [];
            const [all, scene, camera, gl] = [this.all(), this.trueScene(), this.trueCamera(), this.gl()];
            if (!scene || !camera)
                return;
            if (all) {
                // Find all invisible objects, store and then flip them
                scene.traverse((object) => {
                    if (!object.visible) {
                        invisible.push(object);
                        object.visible = true;
                    }
                });
            }
            // Now compile the scene
            gl.compile(scene, camera);
            // And for good measure, hit it with a cube camera
            const cubeRenderTarget = new WebGLCubeRenderTarget(128);
            const cubeCamera = new CubeCamera(0.01, 100000, cubeRenderTarget);
            cubeCamera.update(gl, scene);
            cubeRenderTarget.dispose();
            // Flips these objects back
            invisible.forEach((object) => (object.visible = false));
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsPreload, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "17.1.0", version: "19.1.4", type: NgtsPreload, isStandalone: true, selector: "ngts-preload", inputs: { all: { classPropertyName: "all", publicName: "all", isSignal: true, isRequired: false, transformFunction: null }, scene: { classPropertyName: "scene", publicName: "scene", isSignal: true, isRequired: false, transformFunction: null }, camera: { classPropertyName: "camera", publicName: "camera", isSignal: true, isRequired: false, transformFunction: null } }, outputs: { all: "allChange", scene: "sceneChange", camera: "cameraChange" }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsPreload, decorators: [{
            type: Directive,
            args: [{ selector: 'ngts-preload' }]
        }], ctorParameters: () => [] });

function injectSurfaceSampler(mesh, options = () => ({}), { injector } = {}) {
    return assertInjector(injectSurfaceSampler, injector, () => {
        const initialBufferAttribute = (() => {
            const arr = Array.from({ length: options().count ?? 16 }, () => [
                1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1,
            ]).flat();
            return new InstancedBufferAttribute(Float32Array.from(arr), 16);
        })();
        return computed(() => {
            const currentMesh = resolveRef(mesh());
            if (!currentMesh)
                return initialBufferAttribute;
            const localState = getLocalState(currentMesh);
            if (!localState)
                return initialBufferAttribute;
            const nonObjects = localState.nonObjects();
            if (!nonObjects ||
                !nonObjects.length ||
                nonObjects.every((nonObject) => !nonObject.isBufferGeometry)) {
                return initialBufferAttribute;
            }
            const sampler = new MeshSurfaceSampler(currentMesh);
            const { weight, count = 16, transform, instanceMesh } = options();
            if (weight) {
                sampler.setWeightAttribute(weight);
            }
            sampler.build();
            const position = new Vector3();
            const normal = new Vector3();
            const color = new Color();
            const dummy = new Object3D();
            const instance = resolveRef(instanceMesh);
            currentMesh.updateMatrixWorld(true);
            for (let i = 0; i < count; i++) {
                sampler.sample(position, normal, color);
                if (typeof transform === 'function') {
                    transform({ dummy, sampledMesh: currentMesh, position, normal, color }, i);
                }
                else {
                    dummy.position.copy(position);
                }
                dummy.updateMatrix();
                if (instance) {
                    instance.setMatrixAt(i, dummy.matrix);
                }
                dummy.matrix.toArray(initialBufferAttribute.array, i * 16);
            }
            if (instance) {
                checkUpdate(instance.instanceMatrix);
            }
            checkUpdate(initialBufferAttribute);
            return new InstancedBufferAttribute(initialBufferAttribute.array, initialBufferAttribute.itemSize).copy(initialBufferAttribute);
        });
    });
}
const defaultOptions = {
    count: 16,
};
class NgtsSampler {
    constructor() {
        this.mesh = input(null);
        this.instances = input(null);
        this.options = input(defaultOptions, { transform: mergeInputs(defaultOptions) });
        this.parameters = omit(this.options, ['weight', 'transform', 'count']);
        // NOTE: this could have been a viewChild.required, but we need to _try_ to consume
        //  this Signal earlier than when a viewChild.required would resolve.
        this.groupRef = viewChild('group');
        extend({ Group });
        const sampleState = computed(() => {
            const group = this.groupRef()?.nativeElement;
            const localState = getLocalState(group);
            if (!localState)
                return { mesh: null, instanced: null };
            const [mesh, instances] = [resolveRef(this.mesh()), resolveRef(this.instances())];
            const objects = localState.objects();
            return {
                mesh: mesh ?? objects.find((c) => c.type === 'Mesh'),
                instanced: instances ?? objects.find((c) => !!Object.getOwnPropertyDescriptor(c, 'instanceMatrix')),
            };
        });
        const meshToSample = pick(sampleState, 'mesh');
        const instancedToSample = pick(sampleState, 'instanced');
        // NOTE: because injectSurfaceSampler returns a computed, we need to consume
        //  this computed in a Reactive Context (an effect) to ensure the inner logic of
        //  injectSurfaceSampler is run properly.
        const sampler = injectSurfaceSampler(meshToSample, () => ({
            count: this.options().count,
            transform: this.options().transform,
            weight: this.options().weight,
            instanceMesh: instancedToSample(),
        }));
        effect(sampler);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsSampler, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsSampler, isStandalone: true, selector: "ngts-sampler", inputs: { mesh: { classPropertyName: "mesh", publicName: "mesh", isSignal: true, isRequired: false, transformFunction: null }, instances: { classPropertyName: "instances", publicName: "instances", isSignal: true, isRequired: false, transformFunction: null }, options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "groupRef", first: true, predicate: ["group"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-group #group [parameters]="parameters()">
			<ng-content />
		</ngt-group>
	`, isInline: true, changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsSampler, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-sampler',
                    template: `
		<ngt-group #group [parameters]="parameters()">
			<ng-content />
		</ngt-group>
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }], ctorParameters: () => [] });

const tV0 = new Vector3();
const tV1 = new Vector3();
const tV2 = new Vector3();
function getPoint2(point3, camera, size) {
    const widthHalf = size.width / 2;
    const heightHalf = size.height / 2;
    camera.updateMatrixWorld(false);
    const vector = point3.project(camera);
    vector.x = vector.x * widthHalf + widthHalf;
    vector.y = -(vector.y * heightHalf) + heightHalf;
    return vector;
}
function getPoint3(point2, camera, size, zValue = 1) {
    const vector = tV0.set((point2.x / size.width) * 2 - 1, -(point2.y / size.height) * 2 + 1, zValue);
    vector.unproject(camera);
    return vector;
}
function calculateScaleFactor(point3, radiusPx, camera, size) {
    const point2 = getPoint2(tV2.copy(point3), camera, size);
    let scale = 0;
    for (let i = 0; i < 2; ++i) {
        const point2off = tV1.copy(point2).setComponent(i, point2.getComponent(i) + radiusPx);
        const point3off = getPoint3(point2off, camera, size, point2off.z);
        scale = Math.max(scale, point3.distanceTo(point3off));
    }
    return scale;
}

/**
 * Generated bundle index. Do not edit.
 */

export { LinearEncoding, NgtsBakeShadows, NgtsComputedAttribute, NgtsDecal, NgtsFBO, NgtsHTML, NgtsHTMLContent, NgtsIntersect, NgtsPreload, NgtsSampler, calculateScaleFactor, getVersion, injectAnimations, injectDepthBuffer, injectFBO, injectIntersect, injectSurfaceSampler, sRGBEncoding, setUpdateRange };
//# sourceMappingURL=angular-three-soba-misc.mjs.map
