import * as i0 from '@angular/core';
import { input, effect, untracked, inject, DestroyRef, Directive, viewChild, Component, CUSTOM_ELEMENTS_SCHEMA, ChangeDetectionStrategy, computed } from '@angular/core';
import { injectStore, omit, pick, extend, getLocalState, injectBeforeRender, resolveRef, checkUpdate, vector3, NgtArgs } from 'angular-three';
import { mergeInputs } from 'ngxtension/inject-inputs';
import { LOD, Matrix4, Mesh, Group, Color, Material, Vector3, Quaternion, DynamicDrawUsage, InstancedMesh, InstancedBufferAttribute, Ray, Sphere, Points, BufferAttribute, BufferGeometry, Vector2 } from 'three';
import { setUpdateRange } from 'angular-three-soba/misc';
import { Line2, LineMaterial, LineSegmentsGeometry } from 'three-stdlib';

class NgtsAdaptiveDpr {
    constructor() {
        this.pixelated = input(false);
        const store = injectStore();
        const gl = store.select('gl');
        const active = store.select('internal', 'active');
        const current = store.select('performance', 'current');
        const initialDpr = store.select('viewport', 'initialDpr');
        const setDpr = store.select('setDpr');
        effect(() => {
            const [_current, pixelated, domElement, _setDpr, _initialDpr] = [
                current(),
                untracked(this.pixelated),
                untracked(gl).domElement,
                untracked(setDpr),
                untracked(initialDpr),
            ];
            _setDpr(_current * _initialDpr);
            if (pixelated && domElement)
                domElement.style.imageRendering = _current === 1 ? 'auto' : 'pixelated';
        });
        inject(DestroyRef).onDestroy(() => {
            const [domElement, _active, _setDpr, _initialDpr, pixelated] = [
                untracked(gl).domElement,
                untracked(active),
                untracked(setDpr),
                untracked(initialDpr),
                untracked(this.pixelated),
            ];
            if (_active)
                _setDpr(_initialDpr);
            if (pixelated && domElement)
                domElement.style.imageRendering = 'auto';
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsAdaptiveDpr, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "17.1.0", version: "19.1.4", type: NgtsAdaptiveDpr, isStandalone: true, selector: "ngts-adaptive-dpr", inputs: { pixelated: { classPropertyName: "pixelated", publicName: "pixelated", isSignal: true, isRequired: false, transformFunction: null } }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsAdaptiveDpr, decorators: [{
            type: Directive,
            args: [{ selector: 'ngts-adaptive-dpr' }]
        }], ctorParameters: () => [] });

class NgtsAdaptiveEvents {
    constructor() {
        const store = injectStore();
        const current = store.select('performance', 'current');
        const currentEnabled = store.snapshot.events.enabled;
        effect((onCleanup) => {
            const _current = current();
            onCleanup(() => store.snapshot.setEvents({ enabled: _current === 1 }));
        });
        inject(DestroyRef).onDestroy(() => {
            store.snapshot.setEvents({ enabled: currentEnabled });
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsAdaptiveEvents, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.1.4", type: NgtsAdaptiveEvents, isStandalone: true, selector: "ngts-adaptive-events", ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsAdaptiveEvents, decorators: [{
            type: Directive,
            args: [{ selector: 'ngts-adaptive-events' }]
        }], ctorParameters: () => [] });

const defaultOptions$1 = {
    hysteresis: 0,
};
class NgtsDetailed {
    constructor() {
        this.distances = input.required();
        this.options = input(defaultOptions$1, { transform: mergeInputs(defaultOptions$1) });
        this.parameters = omit(this.options, ['hysteresis']);
        this.lodRef = viewChild.required('lod');
        this.hysteresis = pick(this.options, 'hysteresis');
        extend({ LOD });
        effect(() => {
            const lod = this.lodRef().nativeElement;
            const localState = getLocalState(lod);
            if (!localState)
                return;
            const [, distances, hysteresis] = [localState.objects(), this.distances(), this.hysteresis()];
            lod.levels.length = 0;
            lod.children.forEach((object, index) => {
                lod.levels.push({ object, distance: distances[index], hysteresis });
            });
        });
        injectBeforeRender(({ camera }) => {
            this.lodRef().nativeElement.update(camera);
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsDetailed, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsDetailed, isStandalone: true, selector: "ngts-detailed", inputs: { distances: { classPropertyName: "distances", publicName: "distances", isSignal: true, isRequired: true, transformFunction: null }, options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "lodRef", first: true, predicate: ["lod"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-lOD #lod [parameters]="parameters()">
			<ng-content />
		</ngt-lOD>
	`, isInline: true, changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsDetailed, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-detailed',
                    template: `
		<ngt-lOD #lod [parameters]="parameters()">
			<ng-content />
		</ngt-lOD>
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }], ctorParameters: () => [] });

const _instanceLocalMatrix = new Matrix4();
const _instanceWorldMatrix = new Matrix4();
const _instanceIntersects = [];
const _mesh = new Mesh();
class PositionMesh extends Group {
    constructor() {
        super();
        this.color = new Color('white');
        this.instance = undefined;
    }
    // This will allow the virtual instance have bounds
    get geometry() {
        return resolveRef(this.instance)?.geometry;
    }
    // And this will allow the virtual instance to receive events
    raycast(raycaster, intersects) {
        const parent = resolveRef(this.instance);
        if (!parent)
            return;
        if (!parent.geometry || !parent.material)
            return;
        _mesh.geometry = parent.geometry;
        const matrixWorld = parent.matrixWorld;
        const instanceId = parent.userData['instances'].indexOf(this);
        // If the instance wasn't found or exceeds the parents draw range, bail out
        if (instanceId === -1 || instanceId > parent.count)
            return;
        // calculate the world matrix for each instance
        parent.getMatrixAt(instanceId, _instanceLocalMatrix);
        _instanceWorldMatrix.multiplyMatrices(matrixWorld, _instanceLocalMatrix);
        // the mesh represents this single instance
        _mesh.matrixWorld = _instanceWorldMatrix;
        // raycast side according to instance material
        if (parent.material instanceof Material)
            _mesh.material.side = parent.material.side;
        else
            _mesh.material.side = parent.material[0].side;
        _mesh.raycast(raycaster, _instanceIntersects);
        // process the result of raycast
        for (let i = 0, l = _instanceIntersects.length; i < l; i++) {
            const intersect = _instanceIntersects[i];
            intersect.instanceId = instanceId;
            intersect.object = this;
            intersects.push(intersect);
        }
        _instanceIntersects.length = 0;
    }
}

const parentMatrix$1 = new Matrix4();
const instanceMatrix = new Matrix4();
const tempMatrix = new Matrix4();
const translation = new Vector3();
const rotation = new Quaternion();
const scale = new Vector3();
class NgtsInstance {
    constructor() {
        this.options = input({});
        this.instances = inject(NgtsInstances);
        this.positionMeshRef = viewChild.required('positionMesh');
        extend({ PositionMesh });
        effect((onCleanup) => {
            const cleanup = this.instances.subscribe(this.positionMeshRef().nativeElement);
            onCleanup(() => cleanup());
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsInstance, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsInstance, isStandalone: true, selector: "ngts-instance", inputs: { options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "positionMeshRef", first: true, predicate: ["positionMesh"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-position-mesh #positionMesh [instance]="instances.instancedMeshRef()" [parameters]="options()">
			<ng-content />
		</ngt-position-mesh>
	`, isInline: true, changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsInstance, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-instance',
                    template: `
		<ngt-position-mesh #positionMesh [instance]="instances.instancedMeshRef()" [parameters]="options()">
			<ng-content />
		</ngt-position-mesh>
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }], ctorParameters: () => [] });
const defaultOptions = {
    limit: 1000,
    frames: Infinity,
};
class NgtsInstances {
    constructor() {
        this.DynamicDrawUsage = DynamicDrawUsage;
        this.options = input(defaultOptions, { transform: mergeInputs(defaultOptions) });
        this.parameters = omit(this.options, ['limit', 'frames', 'range']);
        this.instancedMeshRef = viewChild.required('instancedMesh');
        this.limit = pick(this.options, 'limit');
        this.buffers = computed(() => {
            const limit = this.limit();
            const matrices = new Float32Array(limit * 16);
            for (let i = 0; i < limit; i++) {
                tempMatrix.identity().toArray(matrices, i * 16);
            }
            const colors = new Float32Array([...Array.from({ length: limit * 3 }, () => 1)]);
            return { matrices, colors };
        });
        this.instances = [];
        extend({ InstancedMesh, InstancedBufferAttribute });
        effect(() => {
            const instancedMesh = this.instancedMeshRef()?.nativeElement;
            if (!instancedMesh)
                return;
            checkUpdate(instancedMesh.instanceMatrix);
        });
        let iterations = 0;
        let count = 0;
        injectBeforeRender(() => {
            const instancedMesh = this.instancedMeshRef()?.nativeElement;
            if (!instancedMesh)
                return;
            const { frames, limit, range } = this.options();
            const { matrices, colors } = this.buffers();
            if (frames === Infinity || iterations < frames) {
                instancedMesh.updateMatrix();
                instancedMesh.updateMatrixWorld();
                parentMatrix$1.copy(instancedMesh.matrixWorld).invert();
                count = Math.min(limit, range !== undefined ? range : limit, this.instances.length);
                instancedMesh.count = count;
                setUpdateRange(instancedMesh.instanceMatrix, { offset: 0, count: count * 16 });
                if (instancedMesh.instanceColor) {
                    setUpdateRange(instancedMesh.instanceColor, { offset: 0, count: count * 3 });
                }
                for (let i = 0; i < this.instances.length; i++) {
                    const instance = resolveRef(this.instances[i]);
                    if (instance) {
                        // Multiply the inverse of the InstancedMesh world matrix or else
                        // Instances will be double-transformed if <Instances> isn't at identity
                        instance.matrixWorld.decompose(translation, rotation, scale);
                        instanceMatrix.compose(translation, rotation, scale).premultiply(parentMatrix$1);
                        instanceMatrix.toArray(matrices, i * 16);
                        checkUpdate(instancedMesh.instanceMatrix);
                        instance.color.toArray(colors, i * 3);
                        checkUpdate(instancedMesh.instanceColor);
                    }
                }
                iterations++;
            }
        });
    }
    subscribe(ref) {
        this.instances.push(ref);
        return () => {
            this.instances = this.instances.filter((i) => i !== ref);
        };
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsInstances, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsInstances, isStandalone: true, selector: "ngts-instances", inputs: { options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "instancedMeshRef", first: true, predicate: ["instancedMesh"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-instanced-mesh
			#instancedMesh
			[userData]="{ instances }"
			[matrixAutoUpdate]="false"
			[raycast]="null"
			[parameters]="parameters()"
		>
			<ngt-instanced-buffer-attribute
				attach="instanceMatrix"
				[usage]="DynamicDrawUsage"
				[count]="buffers().matrices.length / 16"
				[array]="buffers().matrices"
				[itemSize]="16"
			/>
			<ngt-instanced-buffer-attribute
				attach="instanceColor"
				[usage]="DynamicDrawUsage"
				[count]="buffers().colors.length / 3"
				[array]="buffers().colors"
				[itemSize]="3"
			/>
			<ng-content />
		</ngt-instanced-mesh>
	`, isInline: true, changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsInstances, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-instances',
                    template: `
		<ngt-instanced-mesh
			#instancedMesh
			[userData]="{ instances }"
			[matrixAutoUpdate]="false"
			[raycast]="null"
			[parameters]="parameters()"
		>
			<ngt-instanced-buffer-attribute
				attach="instanceMatrix"
				[usage]="DynamicDrawUsage"
				[count]="buffers().matrices.length / 16"
				[array]="buffers().matrices"
				[itemSize]="16"
			/>
			<ngt-instanced-buffer-attribute
				attach="instanceColor"
				[usage]="DynamicDrawUsage"
				[count]="buffers().colors.length / 3"
				[array]="buffers().colors"
				[itemSize]="3"
			/>
			<ng-content />
		</ngt-instanced-mesh>
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }], ctorParameters: () => [] });

const _inverseMatrix = new Matrix4();
const _ray = new Ray();
const _sphere = new Sphere();
const _position = new Vector3();
class PositionPoint extends Group {
    constructor() {
        super();
        this.size = 0;
        this.color = new Color('white');
        this.instance = undefined;
    }
    // This will allow the virtual instance have bounds
    get geometry() {
        return resolveRef(this.instance)?.geometry;
    }
    raycast(raycaster, intersects) {
        const parent = resolveRef(this.instance);
        if (!parent || !parent.geometry)
            return;
        const instanceId = parent.userData['instances'].indexOf(this);
        // If the instance wasn't found or exceeds the parents draw range, bail out
        if (instanceId === -1 || instanceId > parent.geometry.drawRange.count)
            return;
        const threshold = raycaster.params.Points?.threshold ?? 1;
        _sphere.set(this.getWorldPosition(_position), threshold);
        if (raycaster.ray.intersectsSphere(_sphere) === false)
            return;
        _inverseMatrix.copy(parent.matrixWorld).invert();
        _ray.copy(raycaster.ray).applyMatrix4(_inverseMatrix);
        const localThreshold = threshold / ((this.scale.x + this.scale.y + this.scale.z) / 3);
        const localThresholdSq = localThreshold * localThreshold;
        const rayPointDistanceSq = _ray.distanceSqToPoint(this.position);
        if (rayPointDistanceSq < localThresholdSq) {
            const intersectPoint = new Vector3();
            _ray.closestPointToPoint(this.position, intersectPoint);
            intersectPoint.applyMatrix4(this.matrixWorld);
            const distance = raycaster.ray.origin.distanceTo(intersectPoint);
            if (distance < raycaster.near || distance > raycaster.far)
                return;
            intersects.push({
                distance: distance,
                distanceToRay: Math.sqrt(rayPointDistanceSq),
                point: intersectPoint,
                index: instanceId,
                face: null,
                object: this,
            });
        }
    }
}

class NgtsPoint {
    constructor() {
        this.options = input({});
        this.positionPointRef = viewChild.required('positionPoint');
        this.points = inject(NgtsPointsInstances);
        extend({ PositionPoint });
        effect((onCleanup) => {
            const cleanUp = this.points.subscribe(this.positionPointRef().nativeElement);
            onCleanup(() => cleanUp());
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsPoint, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsPoint, isStandalone: true, selector: "ngts-point", inputs: { options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "positionPointRef", first: true, predicate: ["positionPoint"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-position-point #positionPoint [parameters]="options()" [instance]="points.pointsRef()">
			<ng-content />
		</ngt-position-point>
	`, isInline: true, changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsPoint, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-point',
                    template: `
		<ngt-position-point #positionPoint [parameters]="options()" [instance]="points.pointsRef()">
			<ng-content />
		</ngt-position-point>
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }], ctorParameters: () => [] });
class NgtsPointsBuffer {
    constructor() {
        this.positions = input.required();
        this.colors = input();
        this.sizes = input();
        this.stride = input(3);
        this.options = input({});
        this.pointsRef = viewChild.required('points');
        this.DynamicDrawUsage = DynamicDrawUsage;
        extend({ Points, BufferAttribute, BufferGeometry });
        injectBeforeRender(() => {
            const points = this.pointsRef()?.nativeElement;
            if (!points)
                return;
            const attributes = points.geometry.attributes;
            checkUpdate(attributes['position']);
            if (this.colors())
                checkUpdate(attributes['color']);
            if (this.sizes())
                checkUpdate(attributes['size']);
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsPointsBuffer, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "19.1.4", type: NgtsPointsBuffer, isStandalone: true, selector: "ngts-points-buffer", inputs: { positions: { classPropertyName: "positions", publicName: "positions", isSignal: true, isRequired: true, transformFunction: null }, colors: { classPropertyName: "colors", publicName: "colors", isSignal: true, isRequired: false, transformFunction: null }, sizes: { classPropertyName: "sizes", publicName: "sizes", isSignal: true, isRequired: false, transformFunction: null }, stride: { classPropertyName: "stride", publicName: "stride", isSignal: true, isRequired: false, transformFunction: null }, options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "pointsRef", first: true, predicate: ["points"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-points #points [parameters]="options()">
			<ngt-buffer-geometry>
				<ngt-buffer-attribute
					attach="attributes.position"
					[count]="positions().length / stride()"
					[array]="positions()"
					[itemSize]="stride()"
					[usage]="DynamicDrawUsage"
				/>
				@if (colors(); as colors) {
					<ngt-buffer-attribute
						attach="attributes.color"
						[count]="colors.length / stride()"
						[array]="colors"
						[itemSize]="3"
						[usage]="DynamicDrawUsage"
					/>
				}
				@if (sizes(); as sizes) {
					<ngt-buffer-attribute
						attach="attributes.size"
						[count]="sizes.length / stride()"
						[array]="sizes"
						[itemSize]="1"
						[usage]="DynamicDrawUsage"
					/>
				}
			</ngt-buffer-geometry>
			<ng-content />
		</ngt-points>
	`, isInline: true, changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsPointsBuffer, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-points-buffer',
                    template: `
		<ngt-points #points [parameters]="options()">
			<ngt-buffer-geometry>
				<ngt-buffer-attribute
					attach="attributes.position"
					[count]="positions().length / stride()"
					[array]="positions()"
					[itemSize]="stride()"
					[usage]="DynamicDrawUsage"
				/>
				@if (colors(); as colors) {
					<ngt-buffer-attribute
						attach="attributes.color"
						[count]="colors.length / stride()"
						[array]="colors"
						[itemSize]="3"
						[usage]="DynamicDrawUsage"
					/>
				}
				@if (sizes(); as sizes) {
					<ngt-buffer-attribute
						attach="attributes.size"
						[count]="sizes.length / stride()"
						[array]="sizes"
						[itemSize]="1"
						[usage]="DynamicDrawUsage"
					/>
				}
			</ngt-buffer-geometry>
			<ng-content />
		</ngt-points>
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }], ctorParameters: () => [] });
const parentMatrix = new Matrix4();
const position = new Vector3();
const defaultInstancesOptions = { limit: 1000 };
class NgtsPointsInstances {
    constructor() {
        this.options = input(defaultInstancesOptions, { transform: mergeInputs(defaultInstancesOptions) });
        this.parameters = omit(this.options, ['limit', 'range']);
        this.pointsRef = viewChild.required('points');
        this.limit = pick(this.options, 'limit');
        this.buffers = computed(() => {
            const limit = this.limit();
            return {
                positions: new Float32Array(limit * 3),
                colors: Float32Array.from({ length: limit * 3 }, () => 1),
                sizes: Float32Array.from({ length: limit }, () => 1),
            };
        });
        this.positionPoints = [];
        this.DynamicDrawUsage = DynamicDrawUsage;
        extend({ Points, BufferAttribute, BufferGeometry });
        effect(() => {
            const points = this.pointsRef()?.nativeElement;
            if (!points)
                return;
            checkUpdate(points.geometry.attributes['position']);
        });
        injectBeforeRender(() => {
            const points = this.pointsRef()?.nativeElement;
            if (!points)
                return;
            const { limit, range } = this.options();
            const { positions, sizes, colors } = this.buffers();
            points.updateMatrix();
            points.updateMatrixWorld();
            parentMatrix.copy(points.matrixWorld).invert();
            points.geometry.drawRange.count = Math.min(limit, range !== undefined ? range : limit, this.positionPoints.length);
            for (let i = 0; i < this.positionPoints.length; i++) {
                const positionPoint = resolveRef(this.positionPoints[i]);
                if (positionPoint) {
                    positionPoint.getWorldPosition(position).applyMatrix4(parentMatrix);
                    position.toArray(positions, i * 3);
                    checkUpdate(points.geometry.attributes['position']);
                    positionPoint.matrixWorldNeedsUpdate = true;
                    positionPoint.color.toArray(colors, i * 3);
                    checkUpdate(points.geometry.attributes['color']);
                    sizes.set([positionPoint.size], i);
                    checkUpdate(points.geometry.attributes['size']);
                }
            }
        });
    }
    subscribe(ref) {
        this.positionPoints.push(ref);
        return () => {
            this.positionPoints = this.positionPoints.filter((p) => p !== ref);
        };
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsPointsInstances, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsPointsInstances, isStandalone: true, selector: "ngts-points-instances", inputs: { options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "pointsRef", first: true, predicate: ["points"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-points
			#points
			[userData]="{ instances: positionPoints }"
			[matrixAutoUpdate]="false"
			[raycast]="null"
			[parameters]="parameters()"
		>
			<ngt-buffer-geometry>
				<ngt-buffer-attribute
					attach="attributes.position"
					[count]="buffers().positions.length / 3"
					[array]="buffers().positions"
					[itemSize]="3"
					[usage]="DynamicDrawUsage"
				/>
				<ngt-buffer-attribute
					attach="attributes.color"
					[count]="buffers().colors.length / 3"
					[array]="buffers().colors"
					[itemSize]="3"
					[usage]="DynamicDrawUsage"
				/>
				<ngt-buffer-attribute
					attach="attributes.size"
					[count]="buffers().sizes.length"
					[array]="buffers().sizes"
					[itemSize]="1"
					[usage]="DynamicDrawUsage"
				/>
			</ngt-buffer-geometry>
			<ng-content />
		</ngt-points>
	`, isInline: true, changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsPointsInstances, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-points-instances',
                    template: `
		<ngt-points
			#points
			[userData]="{ instances: positionPoints }"
			[matrixAutoUpdate]="false"
			[raycast]="null"
			[parameters]="parameters()"
		>
			<ngt-buffer-geometry>
				<ngt-buffer-attribute
					attach="attributes.position"
					[count]="buffers().positions.length / 3"
					[array]="buffers().positions"
					[itemSize]="3"
					[usage]="DynamicDrawUsage"
				/>
				<ngt-buffer-attribute
					attach="attributes.color"
					[count]="buffers().colors.length / 3"
					[array]="buffers().colors"
					[itemSize]="3"
					[usage]="DynamicDrawUsage"
				/>
				<ngt-buffer-attribute
					attach="attributes.size"
					[count]="buffers().sizes.length"
					[array]="buffers().sizes"
					[itemSize]="1"
					[usage]="DynamicDrawUsage"
				/>
			</ngt-buffer-geometry>
			<ng-content />
		</ngt-points>
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }], ctorParameters: () => [] });

class SegmentObject {
    constructor() {
        this.color = new Color('white');
        this.start = new Vector3(0, 0, 0);
        this.end = new Vector3(0, 0, 0);
    }
}

class NgtsSegment {
    constructor() {
        this.start = input.required();
        this.end = input.required();
        this.color = input();
        this.normalizedStart = vector3(this.start);
        this.normalizedEnd = vector3(this.end);
        this.segmentRef = viewChild.required('segment');
        this.segments = inject(NgtsSegments);
        extend({ SegmentObject });
        effect((onCleanup) => {
            const cleanUp = this.segments.subscribe(this.segmentRef());
            onCleanup(() => cleanUp());
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsSegment, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsSegment, isStandalone: true, selector: "ngts-segment", inputs: { start: { classPropertyName: "start", publicName: "start", isSignal: true, isRequired: true, transformFunction: null }, end: { classPropertyName: "end", publicName: "end", isSignal: true, isRequired: true, transformFunction: null }, color: { classPropertyName: "color", publicName: "color", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "segmentRef", first: true, predicate: ["segment"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-segment-object #segment [color]="color()" [start]="normalizedStart()" [end]="normalizedEnd()" />
	`, isInline: true, changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsSegment, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-segment',
                    template: `
		<ngt-segment-object #segment [color]="color()" [start]="normalizedStart()" [end]="normalizedEnd()" />
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }], ctorParameters: () => [] });
const defaultSegmentsOptions = {
    limit: 1000,
    lineWidth: 1.0,
};
class NgtsSegments {
    constructor() {
        this.options = input(defaultSegmentsOptions, { transform: mergeInputs(defaultSegmentsOptions) });
        this.parameters = omit(this.options, ['limit', 'lineWidth']);
        this.lineWidth = pick(this.options, 'lineWidth');
        this.limit = pick(this.options, 'limit');
        this.lineRef = viewChild('line');
        this.segments = [];
        this.line = new Line2();
        this.material = new LineMaterial();
        this.geometry = new LineSegmentsGeometry();
        this.resolution = new Vector2(512, 512);
        this.materialParameters = computed(() => ({
            vertexColors: true,
            resolution: this.resolution,
            linewidth: this.lineWidth(),
            ...this.parameters(),
        }));
        this.positions = computed(() => {
            const limit = this.limit();
            return Array.from({ length: limit * 6 }, () => 0);
        });
        this.colors = computed(() => {
            const limit = this.limit();
            return Array.from({ length: limit * 6 }, () => 0);
        });
        injectBeforeRender(() => {
            const [limit, positions, colors] = [this.limit(), this.positions(), this.colors()];
            for (let i = 0; i < limit; i++) {
                const segment = resolveRef(this.segments[i]);
                if (segment) {
                    positions[i * 6 + 0] = segment.start.x;
                    positions[i * 6 + 1] = segment.start.y;
                    positions[i * 6 + 2] = segment.start.z;
                    positions[i * 6 + 3] = segment.end.x;
                    positions[i * 6 + 4] = segment.end.y;
                    positions[i * 6 + 5] = segment.end.z;
                    colors[i * 6 + 0] = segment.color.r;
                    colors[i * 6 + 1] = segment.color.g;
                    colors[i * 6 + 2] = segment.color.b;
                    colors[i * 6 + 3] = segment.color.r;
                    colors[i * 6 + 4] = segment.color.g;
                    colors[i * 6 + 5] = segment.color.b;
                }
            }
            this.geometry.setColors(colors);
            this.geometry.setPositions(positions);
            this.line.computeLineDistances();
        });
    }
    subscribe(ref) {
        this.segments.push(ref);
        return () => {
            this.segments = this.segments.filter((i) => i !== ref);
        };
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsSegments, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsSegments, isStandalone: true, selector: "ngts-segments", inputs: { options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "lineRef", first: true, predicate: ["line"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-primitive #line *args="[line]">
			<ngt-primitive *args="[geometry]" attach="geometry" />
			<ngt-primitive *args="[material]" attach="material" [parameters]="materialParameters()" />
			<ng-content />
		</ngt-primitive>
	`, isInline: true, dependencies: [{ kind: "directive", type: NgtArgs, selector: "ng-template[args]", inputs: ["args"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsSegments, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-segments',
                    template: `
		<ngt-primitive #line *args="[line]">
			<ngt-primitive *args="[geometry]" attach="geometry" />
			<ngt-primitive *args="[material]" attach="material" [parameters]="materialParameters()" />
			<ng-content />
		</ngt-primitive>
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    imports: [NgtArgs],
                }]
        }], ctorParameters: () => [] });

/**
 * Generated bundle index. Do not edit.
 */

export { NgtsAdaptiveDpr, NgtsAdaptiveEvents, NgtsDetailed, NgtsInstance, NgtsInstances, NgtsPoint, NgtsPointsBuffer, NgtsPointsInstances, NgtsSegment, NgtsSegments };
//# sourceMappingURL=angular-three-soba-performances.mjs.map
