import * as i0 from '@angular/core';
import { input, viewChild, computed, effect, inject, DestroyRef, untracked, Component, CUSTOM_ELEMENTS_SCHEMA, ChangeDetectionStrategy, signal, Directive, output, Injector, contentChild, TemplateRef, numberAttribute, ViewContainerRef } from '@angular/core';
import { omit, injectStore, pick, extend, getLocalState, injectBeforeRender, checkUpdate, NgtArgs, vector3, is, resolveRef, injectLoader, applyProps, prepare, NgtPortal, NgtPortalContent } from 'angular-three';
import { ProgressiveLightMap, SoftShadowMaterial, CausticsProjectionMaterial, createCausticsUpdate, SpotLightMaterial } from 'angular-three-soba/vanilla-exports';
import { mergeInputs } from 'ngxtension/inject-inputs';
import { Group, Mesh, PlaneGeometry, Box3, Vector3, Quaternion, Matrix4, LinearFilter, UnsignedByteType, LinearMipmapLinearFilter, FloatType, CustomBlending, OneFactor, SrcAlphaFactor, CameraHelper, Scene, LineBasicMaterial, OrthographicCamera, Sphere, Color, MeshDepthMaterial, ShaderMaterial, MeshBasicMaterial, WebGLRenderTarget, CubeTexture, CubeReflectionMapping, EquirectangularReflectionMapping, CubeTextureLoader, WebGLCubeRenderTarget, HalfFloatType, CubeCamera, MathUtils, DoubleSide, RingGeometry, AlwaysStencilFunc, ReplaceStencilOp, NotEqualStencilFunc, EqualStencilFunc, KeepStencilOp, RepeatWrapping, Vector2, DirectionalLight, Object3D, CylinderGeometry, RGBAFormat, SpotLightHelper, SpotLight, AmbientLight, PointLight } from 'three';
import { SimplexNoise, HorizontalBlurShader, VerticalBlurShader, RGBELoader, EXRLoader, GroundProjectedEnv, Sky, FullScreenQuad } from 'three-stdlib';
import { injectHelper, NgtsEdges, NgtsHelper } from 'angular-three-soba/abstractions';
import { injectFBO, sRGBEncoding, LinearEncoding, getVersion } from 'angular-three-soba/misc';
import { NgTemplateOutlet } from '@angular/common';
import { HDRJPGLoader, GainMapLoader } from '@monogrid/gainmap-js';
import { assertInjector } from 'ngxtension/assert-injector';
import { injectTexture } from 'angular-three-soba/loaders';

const defaultOptions$e = {
    temporal: false,
    frames: 40,
    limit: Infinity,
    blend: 20,
    scale: 10,
    opacity: 1,
    alphaTest: 0.75,
    color: 'black',
    colorBlend: 2,
    resolution: 1024,
    toneMapped: true,
};
class NgtsAccumulativeShadows {
    constructor() {
        this.nullTraversal = () => null;
        this.Math = Math;
        this.options = input(defaultOptions$e, { transform: mergeInputs(defaultOptions$e) });
        this.parameters = omit(this.options, [
            'temporal',
            'frames',
            'limit',
            'blend',
            'scale',
            'opacity',
            'alphaTest',
            'color',
            'colorBlend',
            'resolution',
            'toneMapped',
        ]);
        this.lightsRef = viewChild.required('lights');
        this.planeRef = viewChild.required('plane');
        this.store = injectStore();
        this.gl = this.store.select('gl');
        this.camera = this.store.select('camera');
        this.scene = this.store.select('scene');
        this.invalidate = this.store.select('invalidate');
        this.opacity = pick(this.options, 'opacity');
        this.alphaTest = pick(this.options, 'alphaTest');
        this.limit = pick(this.options, 'limit');
        this.resolution = pick(this.options, 'resolution');
        this.pLM = computed(() => {
            if (this.previousPLM) {
                this.previousPLM.clear();
            }
            return (this.previousPLM = new ProgressiveLightMap(this.gl(), this.scene(), this.resolution()));
        });
        this.scale = pick(this.options, 'scale');
        this.toneMapped = pick(this.options, 'toneMapped');
        this.color = pick(this.options, 'color');
        this.colorBlend = pick(this.options, 'colorBlend');
        this.map = computed(() => this.pLM().progressiveLightMap2.texture);
        this.lightsMap = new Map();
        this.temporal = computed(() => !!this.options().temporal);
        this.frames = computed(() => Math.max(2, this.options().frames));
        this.blend = computed(() => Math.max(2, this.options().frames === Infinity ? this.options().blend : this.options().frames));
        this.count = 0;
        extend({ Group, SoftShadowMaterial, Mesh, PlaneGeometry });
        effect(() => {
            this.pLM().configure(this.planeRef().nativeElement);
        });
        effect(() => {
            const sceneLS = getLocalState(this.scene());
            if (!sceneLS)
                return;
            // track deps
            this.planeRef();
            this.options();
            sceneLS.objects();
            // Reset internals, buffers, ...
            this.reset();
            // Update lightmap
            if (!this.temporal() && this.frames() !== Infinity)
                this.update(this.blend());
        });
        injectBeforeRender(() => {
            const [frames, temporal, invalidate, limit] = [this.frames(), !!this.temporal(), this.invalidate(), this.limit()];
            if ((temporal || frames === Infinity) && this.count < frames && this.count < limit) {
                invalidate();
                this.update();
                this.count++;
            }
        });
        inject(DestroyRef).onDestroy(() => {
            this.previousPLM?.clear();
        });
    }
    getMesh() {
        return this.planeRef().nativeElement;
    }
    reset() {
        // Clear buffers, reset opacities, set frame count to 0
        untracked(this.pLM).clear();
        const material = untracked(this.planeRef).nativeElement.material;
        material.opacity = 0;
        material.alphaTest = 0;
        this.count = 0;
    }
    update(frames = 1) {
        // Adapt the opacity-blend ratio to the number of frames
        const material = this.planeRef().nativeElement.material;
        if (!this.temporal()) {
            material.opacity = this.opacity();
            material.alphaTest = this.alphaTest();
        }
        else {
            material.opacity = Math.min(this.opacity(), material.opacity + this.opacity() / this.blend());
            material.alphaTest = Math.min(this.alphaTest(), material.alphaTest + this.alphaTest() / this.blend());
        }
        // Switch accumulative lights on
        this.lightsRef().nativeElement.visible = true;
        // Collect scene lights and meshes
        this.pLM().prepare();
        // Update the lightmap and the accumulative lights
        for (let i = 0; i < frames; i++) {
            this.lightsMap.forEach((lightUpdate) => lightUpdate());
            this.pLM().update(this.camera(), this.blend());
        }
        // Switch lights off
        this.lightsRef().nativeElement.visible = false;
        // Restore lights and meshes
        this.pLM().finish();
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsAccumulativeShadows, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsAccumulativeShadows, isStandalone: true, selector: "ngts-accumulative-shadows", inputs: { options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "lightsRef", first: true, predicate: ["lights"], descendants: true, isSignal: true }, { propertyName: "planeRef", first: true, predicate: ["plane"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-group [parameters]="parameters()">
			<ngt-group #lights [traverse]="nullTraversal">
				<ng-content />
			</ngt-group>

			<ngt-mesh #plane [scale]="scale()" [rotation]="[-Math.PI / 2, 0, 0]" [receiveShadow]="true">
				<ngt-plane-geometry />
				<ngt-soft-shadow-material
					[depthWrite]="false"
					[toneMapped]="toneMapped()"
					[color]="color()"
					[blend]="colorBlend()"
					[map]="map()"
					[transparent]="true"
				/>
			</ngt-mesh>
		</ngt-group>
	`, isInline: true, changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsAccumulativeShadows, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-accumulative-shadows',
                    template: `
		<ngt-group [parameters]="parameters()">
			<ngt-group #lights [traverse]="nullTraversal">
				<ng-content />
			</ngt-group>

			<ngt-mesh #plane [scale]="scale()" [rotation]="[-Math.PI / 2, 0, 0]" [receiveShadow]="true">
				<ngt-plane-geometry />
				<ngt-soft-shadow-material
					[depthWrite]="false"
					[toneMapped]="toneMapped()"
					[color]="color()"
					[blend]="colorBlend()"
					[map]="map()"
					[transparent]="true"
				/>
			</ngt-mesh>
		</ngt-group>
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }], ctorParameters: () => [] });

function easeInExpo(x) {
    return x === 0 ? 0 : Math.pow(2, 10 * x - 10);
}
const defaultOptions$d = {
    floor: 0.25,
    segments: 20,
};
class NgtsBackdrop {
    constructor() {
        this.Math = Math;
        this.options = input(defaultOptions$d, { transform: mergeInputs(defaultOptions$d) });
        this.parameters = omit(this.options, ['floor', 'segments', 'receiveShadow']);
        this.groupRef = viewChild.required('group');
        this.planeRef = viewChild('plane');
        this.receiveShadow = pick(this.options, 'receiveShadow');
        this.segments = pick(this.options, 'segments');
        this.floor = pick(this.options, 'floor');
        extend({ Group, Mesh, PlaneGeometry });
        effect(() => {
            const plane = this.planeRef()?.nativeElement;
            if (!plane)
                return;
            const [segments, floor] = [this.segments(), this.floor()];
            let i = 0;
            const offset = segments / segments / 2;
            const position = plane.attributes['position'];
            for (let x = 0; x < segments + 1; x++) {
                for (let y = 0; y < segments + 1; y++) {
                    position.setXYZ(i++, x / segments - offset + (x === 0 ? -floor : 0), y / segments - offset, easeInExpo(x / segments));
                }
            }
            checkUpdate(position);
            plane.computeVertexNormals();
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsBackdrop, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsBackdrop, isStandalone: true, selector: "ngts-backdrop", inputs: { options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "groupRef", first: true, predicate: ["group"], descendants: true, isSignal: true }, { propertyName: "planeRef", first: true, predicate: ["plane"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-group #group [parameters]="parameters()">
			<ngt-mesh [receiveShadow]="receiveShadow()" [rotation]="[-Math.PI / 2, 0, Math.PI / 2]">
				<ngt-plane-geometry #plane *args="[1, 1, segments(), segments()]" />
				<ng-content />
			</ngt-mesh>
		</ngt-group>
	`, isInline: true, dependencies: [{ kind: "directive", type: NgtArgs, selector: "ng-template[args]", inputs: ["args"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsBackdrop, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-backdrop',
                    template: `
		<ngt-group #group [parameters]="parameters()">
			<ngt-mesh [receiveShadow]="receiveShadow()" [rotation]="[-Math.PI / 2, 0, Math.PI / 2]">
				<ngt-plane-geometry #plane *args="[1, 1, segments(), segments()]" />
				<ng-content />
			</ngt-mesh>
		</ngt-group>
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    imports: [NgtArgs],
                }]
        }], ctorParameters: () => [] });

const boundingBox = new Box3();
const boundingBoxSize = new Vector3();
class NgtsBBAnchor {
    constructor() {
        this.options = input.required();
        this.parameters = omit(this.options, ['anchor']);
        this.bbAnchorRef = viewChild.required('bbAnchor');
        this.parent = signal(null);
        this.anchor = vector3(this.options, 'anchor');
        extend({ Group });
        // Reattach group created by this component to the parent's parent,
        // so it becomes a sibling of its initial parent.
        // We do that so the children have no impact on a bounding box of a parent.
        effect(() => {
            const bbAnchorLS = getLocalState(this.bbAnchorRef().nativeElement);
            const bbAnchorParent = bbAnchorLS?.parent();
            if (bbAnchorParent?.parent) {
                this.parent.set(bbAnchorParent);
                bbAnchorParent.parent.add(this.bbAnchorRef().nativeElement);
            }
        });
        injectBeforeRender(() => {
            const parent = this.parent();
            if (parent) {
                boundingBox.setFromObject(parent);
                boundingBox.getSize(boundingBoxSize);
                const anchor = this.anchor();
                const bbAnchor = this.bbAnchorRef().nativeElement;
                bbAnchor.position.set(parent.position.x + (boundingBoxSize.x * anchor.x) / 2, parent.position.y + (boundingBoxSize.y * anchor.y) / 2, parent.position.z + (boundingBoxSize.z * anchor.z) / 2);
            }
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsBBAnchor, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsBBAnchor, isStandalone: true, selector: "ngts-bb-anchor", inputs: { options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: true, transformFunction: null } }, viewQueries: [{ propertyName: "bbAnchorRef", first: true, predicate: ["bbAnchor"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-group #bbAnchor [parameters]="parameters()">
			<ng-content />
		</ngt-group>
	`, isInline: true, changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsBBAnchor, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-bb-anchor',
                    template: `
		<ngt-group #bbAnchor [parameters]="parameters()">
			<ng-content />
		</ngt-group>
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }], ctorParameters: () => [] });

var AnimationState;
(function (AnimationState) {
    AnimationState[AnimationState["NONE"] = 0] = "NONE";
    AnimationState[AnimationState["START"] = 1] = "START";
    AnimationState[AnimationState["ACTIVE"] = 2] = "ACTIVE";
})(AnimationState || (AnimationState = {}));
function isBox3(def) {
    return !!def && def.isBox3;
}
function interpolateFuncDefault(t) {
    // Imitates the previously used THREE.MathUtils.damp
    return 1 - Math.exp(-5 * t) + 0.007 * t;
}
const defaultOptions$c = {
    maxDuration: 1.0,
    margin: 1.2,
    interpolateFunc: interpolateFuncDefault,
    clip: false,
    fit: false,
    observe: false,
};
class NgtsBounds {
    constructor() {
        this.options = input(defaultOptions$c, { transform: mergeInputs(defaultOptions$c) });
        this.groupRef = viewChild.required('group');
        this.store = injectStore();
        this.camera = this.store.select('camera');
        this.invalidate = this.store.select('invalidate');
        this.size = this.store.select('size');
        this.controls = this.store.select('controls');
        this.clipOption = pick(this.options, 'clip');
        this.fitOption = pick(this.options, 'fit');
        this.observe = pick(this.options, 'observe');
        this.origin = {
            camPos: new Vector3(),
            camRot: new Quaternion(),
            camZoom: 1,
        };
        this.goal = {
            camPos: undefined,
            camRot: undefined,
            camZoom: undefined,
            camUp: undefined,
            target: undefined,
        };
        this.animationState = AnimationState.NONE;
        // represent animation state from 0 to 1
        this.t = 0;
        this.box = new Box3();
        extend({ Group });
        effect((onCleanup) => {
            const [controls, camera] = [this.controls(), untracked(this.camera)];
            if (!controls)
                return;
            const callback = () => {
                if (this.goal.target && this.animationState !== AnimationState.NONE) {
                    const front = new Vector3().setFromMatrixColumn(camera.matrix, 2);
                    const d0 = this.origin.camPos.distanceTo(controls.target);
                    const d1 = (this.goal.camPos || this.origin.camPos).distanceTo(this.goal.target);
                    const d = (1 - this.t) * d0 + this.t * d1;
                    controls.target.copy(camera.position).addScaledVector(front, -d);
                    controls.update();
                }
                this.animationState = AnimationState.NONE;
            };
            controls.addEventListener('start', callback);
            onCleanup(() => controls.removeEventListener('start', callback));
        });
        let count = 0;
        effect(() => {
            const [clip, fit, observe] = [
                this.clipOption(),
                this.fitOption(),
                this.observe(),
                this.size(),
                this.camera(),
                // TODO: (chau) for some reason, if this effect tracks controls changes, the initial bounds doesn't target properly
                this.controls(),
            ];
            if (observe || count++ === 0) {
                this.refresh();
                if (fit)
                    this.reset().fit();
                if (clip)
                    this.clip();
            }
        });
        injectBeforeRender(({ delta }) => {
            // This [additional animation step START] is needed to guarantee that delta used in animation isn't absurdly high (2-3 seconds) which is actually possible if rendering happens on demand...
            if (this.animationState === AnimationState.START) {
                this.animationState = AnimationState.ACTIVE;
                this.invalidate();
            }
            else if (this.animationState === AnimationState.ACTIVE) {
                const [{ maxDuration, interpolateFunc }, camera, controls, invalidate] = [
                    this.options(),
                    this.camera(),
                    this.controls(),
                    this.invalidate(),
                ];
                this.t += delta / maxDuration;
                if (this.t >= 1) {
                    this.goal.camPos && camera.position.copy(this.goal.camPos);
                    this.goal.camRot && camera.quaternion.copy(this.goal.camRot);
                    this.goal.camUp && camera.up.copy(this.goal.camUp);
                    this.goal.camZoom && is.orthographicCamera(camera) && (camera.zoom = this.goal.camZoom);
                    camera.updateMatrixWorld();
                    camera.updateProjectionMatrix();
                    if (controls && this.goal.target) {
                        controls.target.copy(this.goal.target);
                        controls.update();
                    }
                    this.animationState = AnimationState.NONE;
                }
                else {
                    const k = interpolateFunc(this.t);
                    this.goal.camPos && camera.position.lerpVectors(this.origin.camPos, this.goal.camPos, k);
                    this.goal.camRot && camera.quaternion.slerpQuaternions(this.origin.camRot, this.goal.camRot, k);
                    this.goal.camUp && camera.up.set(0, 1, 0).applyQuaternion(camera.quaternion);
                    this.goal.camZoom &&
                        is.orthographicCamera(camera) &&
                        (camera.zoom = (1 - k) * this.origin.camZoom + k * this.goal.camZoom);
                    camera.updateMatrixWorld();
                    camera.updateProjectionMatrix();
                }
                invalidate();
            }
        });
    }
    getSize() {
        const [camera, { margin }] = [untracked(this.camera), untracked(this.options)];
        const boxSize = this.box.getSize(new Vector3());
        const center = this.box.getCenter(new Vector3());
        const maxSize = Math.max(boxSize.x, boxSize.y, boxSize.z);
        const fitHeightDistance = is.orthographicCamera(camera)
            ? maxSize * 4
            : maxSize / (2 * Math.atan((Math.PI * camera.fov) / 360));
        const fitWidthDistance = is.orthographicCamera(camera) ? maxSize * 4 : fitHeightDistance / camera.aspect;
        const distance = margin * Math.max(fitHeightDistance, fitWidthDistance);
        return { box: this.box, size: boxSize, center, distance };
    }
    refresh(object) {
        const [group, camera] = [untracked(this.groupRef).nativeElement, untracked(this.camera)];
        if (isBox3(object))
            this.box.copy(object);
        else {
            const target = object || group;
            if (!target)
                return this;
            target.updateWorldMatrix(true, true);
            this.box.setFromObject(target);
        }
        if (this.box.isEmpty()) {
            const max = camera.position.length() || 10;
            this.box.setFromCenterAndSize(new Vector3(), new Vector3(max, max, max));
        }
        this.origin.camPos.copy(camera.position);
        this.origin.camRot.copy(camera.quaternion);
        is.orthographicCamera(camera) && (this.origin.camZoom = camera.zoom);
        this.goal.camPos = undefined;
        this.goal.camRot = undefined;
        this.goal.camZoom = undefined;
        this.goal.camUp = undefined;
        this.goal.target = undefined;
        return this;
    }
    reset() {
        const [camera] = [untracked(this.camera)];
        const { center, distance } = this.getSize();
        const direction = camera.position.clone().sub(center).normalize();
        this.goal.camPos = center.clone().addScaledVector(direction, distance);
        this.goal.target = center.clone();
        const mCamRot = new Matrix4().lookAt(this.goal.camPos, this.goal.target, camera.up);
        this.goal.camRot = new Quaternion().setFromRotationMatrix(mCamRot);
        this.animationState = AnimationState.START;
        this.t = 0;
        return this;
    }
    moveTo(position) {
        this.goal.camPos = Array.isArray(position) ? new Vector3(...position) : position.clone();
        this.animationState = AnimationState.START;
        this.t = 0;
        return this;
    }
    lookAt({ target, up }) {
        const [camera] = [untracked(this.camera)];
        this.goal.target = Array.isArray(target) ? new Vector3(...target) : target.clone();
        if (up) {
            this.goal.camUp = Array.isArray(up) ? new Vector3(...up) : up.clone();
        }
        else {
            this.goal.camUp = camera.up.clone();
        }
        const mCamRot = new Matrix4().lookAt(this.goal.camPos || camera.position, this.goal.target, this.goal.camUp);
        this.goal.camRot = new Quaternion().setFromRotationMatrix(mCamRot);
        this.animationState = AnimationState.START;
        this.t = 0;
        return this;
    }
    fit() {
        const [camera, controls, { margin }] = [untracked(this.camera), untracked(this.controls), untracked(this.options)];
        if (!is.orthographicCamera(camera)) {
            // For non-orthographic cameras, fit should behave exactly like reset
            return this.reset();
        }
        // For orthographic cameras, fit should only modify the zoom value
        let maxHeight = 0;
        let maxWidth = 0;
        const vertices = [
            new Vector3(this.box.min.x, this.box.min.y, this.box.min.z),
            new Vector3(this.box.min.x, this.box.max.y, this.box.min.z),
            new Vector3(this.box.min.x, this.box.min.y, this.box.max.z),
            new Vector3(this.box.min.x, this.box.max.y, this.box.max.z),
            new Vector3(this.box.max.x, this.box.max.y, this.box.max.z),
            new Vector3(this.box.max.x, this.box.max.y, this.box.min.z),
            new Vector3(this.box.max.x, this.box.min.y, this.box.max.z),
            new Vector3(this.box.max.x, this.box.min.y, this.box.min.z),
        ];
        // Transform the center and each corner to camera space
        const pos = this.goal.camPos || camera.position;
        const target = this.goal.target || controls.target;
        const up = this.goal.camUp || camera.up;
        const mCamWInv = target
            ? new Matrix4().lookAt(pos, target, up).setPosition(pos).invert()
            : camera.matrixWorldInverse;
        for (const v of vertices) {
            v.applyMatrix4(mCamWInv);
            maxHeight = Math.max(maxHeight, Math.abs(v.y));
            maxWidth = Math.max(maxWidth, Math.abs(v.x));
        }
        maxHeight *= 2;
        maxWidth *= 2;
        const zoomForHeight = (camera.top - camera.bottom) / maxHeight;
        const zoomForWidth = (camera.right - camera.left) / maxWidth;
        this.goal.camZoom = Math.min(zoomForHeight, zoomForWidth) / margin;
        this.animationState = AnimationState.START;
        this.t = 0;
        return this;
    }
    clip() {
        const [camera, controls, invalidate] = [
            untracked(this.camera),
            untracked(this.controls),
            untracked(this.invalidate),
        ];
        const { distance } = this.getSize();
        camera.near = distance / 100;
        camera.far = distance * 100;
        camera.updateProjectionMatrix();
        if (controls) {
            controls.maxDistance = distance * 10;
            controls.update();
        }
        invalidate();
        return this;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsBounds, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsBounds, isStandalone: true, selector: "ngts-bounds", inputs: { options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "groupRef", first: true, predicate: ["group"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-group #group>
			<ng-content />
		</ngt-group>
	`, isInline: true, changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsBounds, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-bounds',
                    template: `
		<ngt-group #group>
			<ng-content />
		</ngt-group>
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }], ctorParameters: () => [] });

const defaultOptions$b = {
    intensity: 1,
    decayRate: 0.65,
    maxYaw: 0.1,
    maxPitch: 0.1,
    maxRoll: 0.1,
    yawFrequency: 0.1,
    pitchFrequency: 0.1,
    rollFrequency: 0.1,
};
class NgtsCameraShake {
    get intensity() {
        return this.intensityOption;
    }
    set intensity(val) {
        this.intensityOption = Math.min(1, Math.max(0, val));
    }
    constructor() {
        this.options = input(defaultOptions$b, { transform: mergeInputs(defaultOptions$b) });
        this.store = injectStore();
        this.camera = this.store.select('camera');
        this.defaultControls = this.store.select('controls');
        this.initialRotation = untracked(this.camera).rotation.clone();
        this.intensityOption = untracked(this.options).intensity;
        this.yawNoise = new SimplexNoise();
        this.pitchNoise = new SimplexNoise();
        this.rollNoise = new SimplexNoise();
        effect((onCleanup) => {
            const defaultControls = this.defaultControls();
            if (!defaultControls)
                return;
            const camera = this.camera();
            const callback = () => void (this.initialRotation = camera.rotation.clone());
            defaultControls.addEventListener('change', callback);
            callback();
            onCleanup(() => void defaultControls.removeEventListener('change', callback));
        });
        injectBeforeRender(({ delta, clock }) => {
            const [{ maxYaw, yawFrequency, maxPitch, pitchFrequency, maxRoll, rollFrequency, decay, decayRate }, camera] = [
                this.options(),
                this.camera(),
            ];
            const shake = Math.pow(this.intensity, 2);
            const yaw = maxYaw * shake * this.yawNoise.noise(clock.elapsedTime * yawFrequency, 1);
            const pitch = maxPitch * shake * this.pitchNoise.noise(clock.elapsedTime * pitchFrequency, 1);
            const roll = maxRoll * shake * this.rollNoise.noise(clock.elapsedTime * rollFrequency, 1);
            camera.rotation.set(this.initialRotation.x + pitch, this.initialRotation.y + yaw, this.initialRotation.z + roll);
            if (decay && this.intensity > 0) {
                this.intensity -= decayRate * delta;
            }
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsCameraShake, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "17.1.0", version: "19.1.4", type: NgtsCameraShake, isStandalone: true, selector: "ngts-camera-shake", inputs: { options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsCameraShake, decorators: [{
            type: Directive,
            args: [{ selector: 'ngts-camera-shake' }]
        }], ctorParameters: () => [] });

const NORMAL_OPTIONS = {
    depth: true,
    minFilter: LinearFilter,
    magFilter: LinearFilter,
    type: UnsignedByteType,
};
const CAUSTIC_OPTIONS = {
    minFilter: LinearMipmapLinearFilter,
    magFilter: LinearFilter,
    type: FloatType,
    generateMipmaps: true,
};
const defaultOptions$a = {
    frames: 1,
    debug: false,
    causticsOnly: false,
    backside: false,
    ior: 1.1,
    backsideIOR: 1.1,
    worldRadius: 0.3125,
    intensity: 0.05,
    color: 'white',
    resolution: 2024,
    lightSource: [5, 5, 5],
};
class NgtsCaustics {
    constructor() {
        this.Math = Math;
        this.CustomBlending = CustomBlending;
        this.OneFactor = OneFactor;
        this.SrcAlphaFactor = SrcAlphaFactor;
        this.options = input(defaultOptions$a, { transform: mergeInputs(defaultOptions$a) });
        this.parameters = omit(this.options, [
            'frames',
            'debug',
            'causticsOnly',
            'backside',
            'ior',
            'backsideIOR',
            'worldRadius',
            'intensity',
            'color',
            'resolution',
            'lightSource',
        ]);
        this.debug = pick(this.options, 'debug');
        this.color = pick(this.options, 'color');
        this.resolution = pick(this.options, 'resolution');
        this.groupRef = viewChild.required('group');
        this.sceneRef = viewChild.required('scene');
        this.cameraRef = viewChild.required('camera');
        this.planeRef = viewChild.required('plane');
        this.normalTargetParams = computed(() => ({
            width: this.resolution(),
            height: this.resolution(),
            settings: NORMAL_OPTIONS,
        }));
        // Buffers for front and back faces
        this.normalTarget = injectFBO(this.normalTargetParams);
        this.normalTargetB = injectFBO(this.normalTargetParams);
        this.causticsTargetParams = computed(() => ({
            width: this.resolution(),
            height: this.resolution(),
            settings: CAUSTIC_OPTIONS,
        }));
        this.causticsTarget = injectFBO(this.causticsTargetParams);
        this.causticsTargetB = injectFBO(this.causticsTargetParams);
        this.cameraHelper = injectHelper(() => (this.debug() ? this.cameraRef().nativeElement : null), () => CameraHelper);
        extend({ CausticsProjectionMaterial, Group, Scene, Mesh, PlaneGeometry, LineBasicMaterial, OrthographicCamera });
        effect(() => {
            // track all changes
            const [group, scene, plane] = [
                this.groupRef().nativeElement,
                this.sceneRef().nativeElement,
                this.planeRef().nativeElement,
                this.options(),
            ];
            const groupLocalState = getLocalState(group);
            const sceneLocalState = getLocalState(scene);
            const planeLocalState = getLocalState(plane);
            if (!groupLocalState || !sceneLocalState || !planeLocalState)
                return;
            groupLocalState.objects();
            sceneLocalState.objects();
            planeLocalState.objects();
            planeLocalState.nonObjects();
            group.updateWorldMatrix(false, true);
        });
        const update = createCausticsUpdate(() => {
            const { lightSource, ...rest } = this.options();
            return {
                params: Object.assign(rest, {
                    lightSource: Array.isArray(lightSource) ? new Vector3(...lightSource) : resolveRef(lightSource),
                }),
                normalTarget: this.normalTarget(),
                normalTargetB: this.normalTargetB(),
                causticsTarget: this.causticsTarget(),
                causticsTargetB: this.causticsTargetB(),
                camera: this.cameraRef().nativeElement,
                scene: this.sceneRef().nativeElement,
                group: this.groupRef().nativeElement,
                plane: this.planeRef().nativeElement,
                helper: this.cameraHelper(),
            };
        });
        injectBeforeRender(({ gl }) => update(gl));
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsCaustics, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "19.1.4", type: NgtsCaustics, isStandalone: true, selector: "ngts-caustics", inputs: { options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "groupRef", first: true, predicate: ["group"], descendants: true, isSignal: true }, { propertyName: "sceneRef", first: true, predicate: ["scene"], descendants: true, isSignal: true }, { propertyName: "cameraRef", first: true, predicate: ["camera"], descendants: true, isSignal: true }, { propertyName: "planeRef", first: true, predicate: ["plane"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-group #group [parameters]="parameters()">
			<ngt-scene #scene>
				<ngt-orthographic-camera #camera [up]="[0, 1, 0]" />
				<ng-content />
			</ngt-scene>

			<ngt-mesh #plane [renderOrder]="2" [rotation]="[-Math.PI / 2, 0, 0]">
				<ngt-plane-geometry />
				<ngt-caustics-projection-material
					[transparent]="true"
					[color]="color()"
					[causticsTexture]="causticsTarget().texture"
					[causticsTextureB]="causticsTargetB().texture"
					[blending]="CustomBlending"
					[blendSrc]="OneFactor"
					[blendDst]="SrcAlphaFactor"
					[depthWrite]="false"
				/>

				@if (debug()) {
					<ngts-edges>
						<ngt-line-basic-material color="#ffff00" [toneMapped]="false" />
					</ngts-edges>
				}
			</ngt-mesh>
		</ngt-group>
	`, isInline: true, dependencies: [{ kind: "component", type: NgtsEdges, selector: "ngts-edges", inputs: ["options"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsCaustics, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-caustics',
                    template: `
		<ngt-group #group [parameters]="parameters()">
			<ngt-scene #scene>
				<ngt-orthographic-camera #camera [up]="[0, 1, 0]" />
				<ng-content />
			</ngt-scene>

			<ngt-mesh #plane [renderOrder]="2" [rotation]="[-Math.PI / 2, 0, 0]">
				<ngt-plane-geometry />
				<ngt-caustics-projection-material
					[transparent]="true"
					[color]="color()"
					[causticsTexture]="causticsTarget().texture"
					[causticsTextureB]="causticsTargetB().texture"
					[blending]="CustomBlending"
					[blendSrc]="OneFactor"
					[blendDst]="SrcAlphaFactor"
					[depthWrite]="false"
				/>

				@if (debug()) {
					<ngts-edges>
						<ngt-line-basic-material color="#ffff00" [toneMapped]="false" />
					</ngts-edges>
				}
			</ngt-mesh>
		</ngt-group>
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    imports: [NgtsEdges],
                }]
        }], ctorParameters: () => [] });

const defaultOptions$9 = {
    precise: true,
    cacheKey: 0,
};
class NgtsCenter {
    constructor() {
        this.options = input(defaultOptions$9, { transform: mergeInputs(defaultOptions$9) });
        this.parameters = omit(this.options, [
            'top',
            'right',
            'bottom',
            'left',
            'front',
            'back',
            'disable',
            'disableX',
            'disableY',
            'disableZ',
            'precise',
            'cacheKey',
        ]);
        this.centered = output();
        this.groupRef = viewChild.required('group');
        this.outerRef = viewChild.required('outer');
        this.innerRef = viewChild.required('inner');
        this.centerOptions = pick(this.options, [
            'top',
            'right',
            'bottom',
            'left',
            'front',
            'back',
            'disable',
            'disableX',
            'disableY',
            'disableZ',
            'precise',
            'cacheKey',
        ]);
        extend({ Group });
        effect(() => {
            const inner = this.innerRef().nativeElement;
            const localState = getLocalState(inner);
            if (!localState)
                return;
            const children = [localState.objects(), localState.nonObjects()];
            if (!children?.length)
                return;
            const [{ precise, top, bottom, right, left, front, back, disable, disableZ, disableY, disableX }, group, outer] = [this.centerOptions(), this.groupRef().nativeElement, this.outerRef().nativeElement];
            outer.matrixWorld.identity();
            const box3 = new Box3().setFromObject(inner, precise);
            const center = new Vector3();
            const sphere = new Sphere();
            const width = box3.max.x - box3.min.x;
            const height = box3.max.y - box3.min.y;
            const depth = box3.max.z - box3.min.z;
            box3.getCenter(center);
            box3.getBoundingSphere(sphere);
            const vAlign = top ? height / 2 : bottom ? -height / 2 : 0;
            const hAlign = left ? -width / 2 : right ? width / 2 : 0;
            const dAlign = front ? depth / 2 : back ? -depth / 2 : 0;
            outer.position.set(disable || disableX ? 0 : -center.x + hAlign, disable || disableY ? 0 : -center.y + vAlign, disable || disableZ ? 0 : -center.z + dAlign);
            this.centered.emit({
                parent: group.parent,
                container: group,
                width,
                height,
                depth,
                boundingBox: box3,
                boundingSphere: sphere,
                center,
                verticalAlignment: vAlign,
                horizontalAlignment: hAlign,
                depthAlignment: dAlign,
            });
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsCenter, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsCenter, isStandalone: true, selector: "ngts-center", inputs: { options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, outputs: { centered: "centered" }, viewQueries: [{ propertyName: "groupRef", first: true, predicate: ["group"], descendants: true, isSignal: true }, { propertyName: "outerRef", first: true, predicate: ["outer"], descendants: true, isSignal: true }, { propertyName: "innerRef", first: true, predicate: ["inner"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-group #group [parameters]="parameters()">
			<ngt-group #outer>
				<ngt-group #inner>
					<ng-content />
				</ngt-group>
			</ngt-group>
		</ngt-group>
	`, isInline: true, changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsCenter, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-center',
                    template: `
		<ngt-group #group [parameters]="parameters()">
			<ngt-group #outer>
				<ngt-group #inner>
					<ng-content />
				</ngt-group>
			</ngt-group>
		</ngt-group>
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }], ctorParameters: () => [] });

const defaultOptions$8 = {
    scale: 10,
    frames: Infinity,
    opacity: 1,
    width: 1,
    height: 1,
    blur: 1,
    near: 0,
    far: 10,
    resolution: 512,
    smooth: true,
    color: '#000000',
    depthWrite: false,
};
class NgtsContactShadows {
    constructor() {
        this.Math = Math;
        this.options = input(defaultOptions$8, { transform: mergeInputs(defaultOptions$8) });
        this.parameters = omit(this.options, [
            'scale',
            'frames',
            'opacity',
            'width',
            'height',
            'blur',
            'near',
            'far',
            'resolution',
            'smooth',
            'color',
            'depthWrite',
            'renderOrder',
        ]);
        this.contactShadowsRef = viewChild.required('contactShadows');
        this.shadowsCameraRef = viewChild('shadowsCamera');
        this.store = injectStore();
        this.scene = this.store.select('scene');
        this.gl = this.store.select('gl');
        this.scaledWidth = computed(() => {
            const { width, scale } = this.options();
            return width * (Array.isArray(scale) ? scale[0] : scale);
        });
        this.scaledHeight = computed(() => {
            const { height, scale } = this.options();
            return height * (Array.isArray(scale) ? scale[1] : scale);
        });
        this.resolution = pick(this.options, 'resolution');
        this.color = pick(this.options, 'color');
        this.near = pick(this.options, 'near');
        this.far = pick(this.options, 'far');
        this.smooth = pick(this.options, 'smooth');
        this.frames = pick(this.options, 'frames');
        this.blur = pick(this.options, 'blur');
        this.renderTarget = computed(() => this.createRenderTarget(this.resolution()));
        this.renderTargetBlur = computed(() => this.createRenderTarget(this.resolution()));
        this.planeGeometry = computed(() => new PlaneGeometry(this.scaledWidth(), this.scaledHeight()).rotateX(Math.PI / 2));
        this.blurPlane = computed(() => new Mesh(this.planeGeometry()));
        this.depthMaterial = computed(() => {
            const color = new Color(this.color());
            const material = new MeshDepthMaterial();
            material.depthTest = material.depthWrite = false;
            material.onBeforeCompile = (shader) => {
                shader.uniforms = { ...shader.uniforms, ucolor: { value: color } };
                shader.fragmentShader = shader.fragmentShader.replace(`void main() {`, //
                `uniform vec3 ucolor;
         void main() {
        `);
                shader.fragmentShader = shader.fragmentShader.replace('vec4( vec3( 1.0 - fragCoordZ ), opacity );', 
                // Colorize the shadow, multiply by the falloff so that the center can remain darker
                'vec4( ucolor * fragCoordZ * 2.0, ( 1.0 - fragCoordZ ) * 1.0 );');
            };
            return material;
        });
        this.horizontalBlurMaterial = new ShaderMaterial({ ...HorizontalBlurShader, depthTest: false });
        this.verticalBlurMaterial = new ShaderMaterial({ ...VerticalBlurShader, depthTest: false });
        this.renderOrder = pick(this.options, 'renderOrder');
        this.opacity = pick(this.options, 'opacity');
        this.depthWrite = pick(this.options, 'depthWrite');
        this.texture = pick(this.renderTarget, 'texture');
        this.cameraArgs = computed(() => {
            const [width, height, near, far] = [this.scaledWidth(), this.scaledHeight(), this.near(), this.far()];
            return [-width / 2, width / 2, height / 2, -height / 2, near, far];
        });
        extend({ Group, Mesh, MeshBasicMaterial, OrthographicCamera });
        let count = 0;
        injectBeforeRender(() => {
            const shadowsCamera = this.shadowsCameraRef()?.nativeElement;
            if (!shadowsCamera)
                return;
            const frames = this.frames();
            if (frames === Infinity || count < frames) {
                this.renderShadows();
                count++;
            }
        });
    }
    renderShadows() {
        const shadowsCamera = this.shadowsCameraRef()?.nativeElement;
        if (!shadowsCamera)
            return;
        const [blur, smooth, gl, scene, contactShadows, depthMaterial, renderTarget] = [
            this.blur(),
            this.smooth(),
            this.gl(),
            this.scene(),
            this.contactShadowsRef().nativeElement,
            this.depthMaterial(),
            this.renderTarget(),
        ];
        const initialBackground = scene.background;
        const initialOverrideMaterial = scene.overrideMaterial;
        const initialClearAlpha = gl.getClearAlpha();
        contactShadows.visible = false;
        scene.background = null;
        scene.overrideMaterial = depthMaterial;
        gl.setClearAlpha(0);
        // render to the render target to get the depths
        gl.setRenderTarget(renderTarget);
        gl.render(scene, shadowsCamera);
        this.blurShadows(blur);
        if (smooth)
            this.blurShadows(blur * 0.4);
        // reset
        gl.setRenderTarget(null);
        contactShadows.visible = true;
        scene.overrideMaterial = initialOverrideMaterial;
        scene.background = initialBackground;
        gl.setClearAlpha(initialClearAlpha);
    }
    blurShadows(blur) {
        const shadowsCamera = this.shadowsCameraRef()?.nativeElement;
        if (!shadowsCamera)
            return;
        const [blurPlane, horizontalBlurMaterial, verticalBlurMaterial, renderTargetBlur, renderTarget, gl] = [
            this.blurPlane(),
            this.horizontalBlurMaterial,
            this.verticalBlurMaterial,
            this.renderTargetBlur(),
            this.renderTarget(),
            this.gl(),
        ];
        blurPlane.visible = true;
        blurPlane.material = horizontalBlurMaterial;
        horizontalBlurMaterial.uniforms['tDiffuse'].value = renderTarget.texture;
        horizontalBlurMaterial.uniforms['h'].value = blur / 256;
        gl.setRenderTarget(renderTargetBlur);
        gl.render(blurPlane, shadowsCamera);
        blurPlane.material = verticalBlurMaterial;
        verticalBlurMaterial.uniforms['tDiffuse'].value = renderTargetBlur.texture;
        verticalBlurMaterial.uniforms['v'].value = blur / 256;
        gl.setRenderTarget(renderTarget);
        gl.render(blurPlane, shadowsCamera);
        blurPlane.visible = false;
    }
    createRenderTarget(resolution) {
        const renderTarget = new WebGLRenderTarget(resolution, resolution);
        renderTarget.texture.generateMipmaps = false;
        return renderTarget;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsContactShadows, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsContactShadows, isStandalone: true, selector: "ngts-contact-shadows", inputs: { options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "contactShadowsRef", first: true, predicate: ["contactShadows"], descendants: true, isSignal: true }, { propertyName: "shadowsCameraRef", first: true, predicate: ["shadowsCamera"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-group #contactShadows [rotation]="[Math.PI / 2, 0, 0]" [parameters]="parameters()">
			<ngt-mesh
				[scale]="[1, -1, 1]"
				[rotation]="[-Math.PI / 2, 0, 0]"
				[renderOrder]="renderOrder() ?? 0"
				[geometry]="planeGeometry()"
			>
				<ngt-mesh-basic-material
					[transparent]="true"
					[opacity]="opacity()"
					[depthWrite]="depthWrite()"
					[map]="texture()"
				/>
			</ngt-mesh>
			<ngt-orthographic-camera *args="cameraArgs()" #shadowsCamera />
		</ngt-group>
	`, isInline: true, dependencies: [{ kind: "directive", type: NgtArgs, selector: "ng-template[args]", inputs: ["args"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsContactShadows, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-contact-shadows',
                    template: `
		<ngt-group #contactShadows [rotation]="[Math.PI / 2, 0, 0]" [parameters]="parameters()">
			<ngt-mesh
				[scale]="[1, -1, 1]"
				[rotation]="[-Math.PI / 2, 0, 0]"
				[renderOrder]="renderOrder() ?? 0"
				[geometry]="planeGeometry()"
			>
				<ngt-mesh-basic-material
					[transparent]="true"
					[opacity]="opacity()"
					[depthWrite]="depthWrite()"
					[map]="texture()"
				/>
			</ngt-mesh>
			<ngt-orthographic-camera *args="cameraArgs()" #shadowsCamera />
		</ngt-group>
	`,
                    imports: [NgtArgs],
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }], ctorParameters: () => [] });

const ENVIRONMENT_PRESETS = {
    apartment: 'lebombo_1k.hdr',
    city: 'potsdamer_platz_1k.hdr',
    dawn: 'kiara_1_dawn_1k.hdr',
    forest: 'forest_slope_1k.hdr',
    lobby: 'st_fagans_interior_1k.hdr',
    night: 'dikhololo_night_1k.hdr',
    park: 'rooitou_park_1k.hdr',
    studio: 'studio_small_03_1k.hdr',
    sunset: 'venice_sunset_1k.hdr',
    warehouse: 'empty_warehouse_01_1k.hdr',
};
const CUBEMAP_ROOT = 'https://raw.githack.com/pmndrs/drei-assets/456060a26bbeb8fdf79326f224b6d99b8bcce736/hdri/';
const defaultFiles = ['/px.png', '/nx.png', '/py.png', '/ny.png', '/pz.png', '/nz.png'];
function injectEnvironment(options = () => ({}), { injector } = {}) {
    return assertInjector(injectEnvironment, injector, () => {
        const adjustedOptions = computed(() => {
            const { preset, extensions, encoding, ...rest } = options();
            let { files, path } = rest;
            if (files == null) {
                files = defaultFiles;
            }
            if (path == null) {
                path = '';
            }
            if (preset) {
                validatePreset(preset);
                files = ENVIRONMENT_PRESETS[preset];
                path = CUBEMAP_ROOT;
            }
            return { files, preset, encoding, path, extensions };
        });
        const files = pick(adjustedOptions, 'files');
        const multiFile = computed(() => Array.isArray(files()));
        const resultOptions = computed(() => getExtension(files()));
        const extension = pick(resultOptions, 'extension');
        const loader = computed(() => getLoader(extension()));
        const store = injectStore();
        const gl = store.select('gl');
        const texture = signal(null);
        effect(() => {
            const [_extension, _multiFile, _files] = [untracked(extension), untracked(multiFile), files()];
            if (_extension !== 'webp' && _extension !== 'jpg' && _extension !== 'jpeg')
                return;
            gl().domElement.addEventListener('webglcontextlost', () => {
                // @ts-expect-error - files is correctly passed
                injectLoader.clear(multiFile ? [_files] : _files);
            }, { once: true });
        });
        const result = injectLoader(loader, 
        // @ts-expect-error - ensure the files is an array
        () => {
            const { files } = adjustedOptions();
            return Array.isArray(files) ? [files] : files;
        }, {
            extensions: (loader) => {
                const { extensions, path } = adjustedOptions();
                const { extension } = resultOptions();
                if (extension === 'webp' || extension === 'jpg' || extension === 'jpeg') {
                    // @ts-expect-error - Gainmap requires a renderer
                    loader.setRenderer(gl());
                }
                loader.setPath?.(path);
                if (extensions)
                    extensions(loader);
            },
        });
        effect(() => {
            const loaderResult = result();
            if (!loaderResult)
                return;
            const { extension, isCubeMap } = untracked(resultOptions);
            const _multiFile = untracked(multiFile);
            const { encoding } = untracked(adjustedOptions);
            // @ts-expect-error - ensure textureResult is a Texture or CubeTexture
            let textureResult = (_multiFile ? loaderResult[0] : loaderResult);
            // NOTE: racing condition, we can skip this
            //  we just said above that if multiFile is false, it is a single Texture
            if (!_multiFile && Array.isArray(textureResult) && textureResult[0] instanceof CubeTexture) {
                return;
            }
            if (!(textureResult instanceof CubeTexture) &&
                (extension === 'jpg' || extension === 'jpeg' || extension === 'webp')) {
                textureResult = textureResult.renderTarget?.texture;
            }
            textureResult.mapping = isCubeMap ? CubeReflectionMapping : EquirectangularReflectionMapping;
            if ('colorSpace' in textureResult)
                textureResult.colorSpace = encoding ?? (isCubeMap ? 'srgb' : 'srgb-linear');
            else
                textureResult.encoding = encoding ?? (isCubeMap ? sRGBEncoding : LinearEncoding);
            texture.set(textureResult);
        });
        return texture.asReadonly();
    });
}
injectEnvironment.preload = (options = () => ({})) => {
    const _options = options();
    let { files, path } = _options;
    const { preset, extensions } = _options;
    if (files == null) {
        files = defaultFiles;
    }
    if (path == null) {
        path = '';
    }
    if (preset) {
        validatePreset(preset);
        files = ENVIRONMENT_PRESETS[preset];
        path = CUBEMAP_ROOT;
    }
    const { extension } = getExtension(files);
    if (extension === 'webp' || extension === 'jpg' || extension === 'jpeg') {
        throw new Error('injectEnvironment: Preloading gainmaps is not supported');
    }
    const loader = getLoader(extension);
    if (!loader)
        throw new Error('injectEnvironment: Unrecognized file extension: ' + files);
    injectLoader.preload(() => loader, 
    // @ts-expect-error - files is correctly passed
    () => (Array.isArray(files) ? [files] : files), (loader) => {
        loader.setPath?.(path);
        if (extensions)
            extensions(loader);
    });
};
injectEnvironment.clear = (clearOptions) => {
    const options = { files: defaultFiles, ...clearOptions };
    let { files } = options;
    const preset = options.preset;
    if (preset) {
        validatePreset(preset);
        files = ENVIRONMENT_PRESETS[preset];
    }
    injectLoader.clear(files);
};
function validatePreset(preset) {
    if (!(preset in ENVIRONMENT_PRESETS))
        throw new Error('Preset must be one of: ' + Object.keys(ENVIRONMENT_PRESETS).join(', '));
}
function getExtension(files) {
    const isCubeMap = Array.isArray(files) && files.length === 6;
    const isGainmap = Array.isArray(files) && files.length === 3 && files.some((file) => file.endsWith('json'));
    const firstEntry = Array.isArray(files) ? files[0] : files;
    // Everything else
    const extension = isCubeMap
        ? 'cube'
        : isGainmap
            ? 'webp'
            : firstEntry.startsWith('data:application/exr')
                ? 'exr'
                : firstEntry.startsWith('data:application/hdr')
                    ? 'hdr'
                    : firstEntry.startsWith('data:image/jpeg')
                        ? 'jpg'
                        : firstEntry.split('.').pop()?.split('?')?.shift()?.toLowerCase();
    return { extension, isCubeMap, isGainmap };
}
function getLoader(extension) {
    const loader = extension === 'cube'
        ? CubeTextureLoader
        : extension === 'hdr'
            ? RGBELoader
            : extension === 'exr'
                ? EXRLoader
                : extension === 'jpg' || extension === 'jpeg'
                    ? HDRJPGLoader
                    : extension === 'webp'
                        ? GainMapLoader
                        : null;
    if (!loader) {
        throw new Error('injectEnvironment: Unrecognized file extension: ' + extension);
    }
    return loader;
}

function resolveScene(scene) {
    return is.ref(scene) ? scene.nativeElement : scene;
}
function setEnvProps(background, scene, defaultScene, texture, sceneProps = {}) {
    sceneProps.backgroundBlurriness ??= sceneProps.blur ?? 0;
    sceneProps.backgroundIntensity ??= 1;
    // @ts-expect-error - it's ok, we're sending through applyProps
    sceneProps.backgroundRotation ??= [0, 0, 0];
    sceneProps.environmentIntensity ??= 1;
    // @ts-expect-error - it's ok, we're sending through applyProps
    sceneProps.environmentRotation ??= [0, 0, 0];
    const target = resolveScene(scene || defaultScene);
    const oldbg = target.background;
    const oldenv = target.environment;
    const oldSceneProps = {
        backgroundBlurriness: target.backgroundBlurriness,
        backgroundIntensity: target.backgroundIntensity,
        backgroundRotation: target.backgroundRotation?.clone?.() ?? [0, 0, 0],
        environmentIntensity: target.environmentIntensity,
        environmentRotation: target.environmentRotation?.clone?.() ?? [0, 0, 0],
    };
    if (background !== 'only')
        target.environment = texture;
    if (background)
        target.background = texture;
    applyProps(target, sceneProps);
    return () => {
        if (background !== 'only')
            target.environment = oldenv;
        if (background)
            target.background = oldbg;
        applyProps(target, oldSceneProps);
    };
}
const defaultBackground = {
    background: false,
};
class NgtsEnvironmentMap {
    constructor() {
        this.options = input(defaultBackground, { transform: mergeInputs(defaultBackground) });
        this.envSet = output();
        const store = injectStore();
        const defaultScene = store.select('scene');
        const _map = pick(this.options, 'map');
        const _envConfig = computed(() => {
            const { background = false, scene, blur, backgroundBlurriness, backgroundIntensity, backgroundRotation, environmentIntensity, environmentRotation, } = this.options();
            return {
                background,
                scene,
                blur,
                backgroundBlurriness,
                backgroundIntensity,
                backgroundRotation,
                environmentIntensity,
                environmentRotation,
            };
        });
        effect((onCleanup) => {
            const map = _map();
            if (!map)
                return;
            const { background = false, scene, ...config } = _envConfig();
            const cleanup = setEnvProps(background, scene, defaultScene(), map, config);
            this.envSet.emit();
            onCleanup(() => cleanup());
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsEnvironmentMap, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "17.1.0", version: "19.1.4", type: NgtsEnvironmentMap, isStandalone: true, selector: "ngts-environment-map", inputs: { options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, outputs: { envSet: "envSet" }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsEnvironmentMap, decorators: [{
            type: Directive,
            args: [{ selector: 'ngts-environment-map' }]
        }], ctorParameters: () => [] });
class NgtsEnvironmentCube {
    constructor() {
        this.options = input(defaultBackground, { transform: mergeInputs(defaultBackground) });
        this.envSet = output();
        this.store = injectStore();
        this.defaultScene = this.store.select('scene');
        this.envConfig = computed(() => {
            const { background = false, scene, blur, backgroundBlurriness, backgroundIntensity, backgroundRotation, environmentIntensity, environmentRotation, } = this.options();
            return {
                background,
                scene,
                blur,
                backgroundBlurriness,
                backgroundIntensity,
                backgroundRotation,
                environmentIntensity,
                environmentRotation,
            };
        });
        const _texture = injectEnvironment(this.options);
        effect((onCleanup) => {
            const texture = _texture();
            if (!texture)
                return;
            const { background = false, scene, ...config } = this.envConfig();
            const cleanup = setEnvProps(background, scene, this.defaultScene(), texture, config);
            this.envSet.emit();
            onCleanup(() => cleanup());
        });
        inject(DestroyRef).onDestroy(() => {
            _texture()?.dispose();
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsEnvironmentCube, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "17.1.0", version: "19.1.4", type: NgtsEnvironmentCube, isStandalone: true, selector: "ngts-environment-cube", inputs: { options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, outputs: { envSet: "envSet" }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsEnvironmentCube, decorators: [{
            type: Directive,
            args: [{ selector: 'ngts-environment-cube' }]
        }], ctorParameters: () => [] });
class NgtsEnvironmentPortal {
    constructor() {
        this.defaultOptions = {
            near: 1,
            far: 1000,
            resolution: 256,
            frames: 1,
            background: false,
        };
        this.options = input(this.defaultOptions, { transform: mergeInputs(this.defaultOptions) });
        this.content = input.required();
        this.envSet = output();
        this.injector = inject(Injector);
        this.store = injectStore();
        this.defaultScene = this.store.select('scene');
        this.gl = this.store.select('gl');
        this.cameraRef = viewChild('cubeCamera');
        this.map = pick(this.options, 'map');
        this.files = pick(this.options, 'files');
        this.preset = pick(this.options, 'preset');
        this.extensions = pick(this.options, 'extensions');
        this.path = pick(this.options, 'path');
        this.envMapOptions = computed(() => ({ background: true, map: this.map(), extensions: this.extensions() }));
        this.envCubeOptions = computed(() => ({
            background: true,
            files: this.files(),
            preset: this.preset(),
            extensions: this.extensions(),
            path: this.path(),
        }));
        this.near = pick(this.options, 'near');
        this.far = pick(this.options, 'far');
        this.resolution = pick(this.options, 'resolution');
        this.fbo = computed(() => {
            const fbo = new WebGLCubeRenderTarget(this.resolution());
            fbo.texture.type = HalfFloatType;
            return fbo;
        });
        this.cameraArgs = computed(() => [this.near(), this.far(), this.fbo()]);
        this.virtualScene = prepare(new Scene());
        extend({ CubeCamera });
        effect((onCleanup) => {
            const [files, preset, map] = [this.files(), this.preset(), this.map()];
            // NOTE: when there's none of this, we don't render cube or map so we need to setEnv here
            if (!!files || !!preset || !!map)
                return;
            const cleanup = this.setPortalEnv();
            onCleanup(() => cleanup?.());
        });
        let count = 1;
        injectBeforeRender(() => {
            const frames = this.options().frames;
            if (frames === Infinity || (frames != null && count < frames)) {
                const camera = this.cameraRef()?.nativeElement;
                if (camera) {
                    camera.update(this.gl(), this.virtualScene);
                    count++;
                }
            }
        });
        inject(DestroyRef).onDestroy(() => {
            if (this.setEnvEffectRef)
                this.setEnvEffectRef.destroy();
        });
    }
    // NOTE: we use onEnvSet here to ensure that EnvironmentCube or EnvironmentMap sets the env before the portal
    onEnvSet() {
        if (this.setEnvEffectRef)
            this.setEnvEffectRef.destroy();
        this.setEnvEffectRef = effect((onCleanup) => {
            const cleanup = this.setPortalEnv();
            onCleanup(() => cleanup?.());
        }, { manualCleanup: true, injector: this.injector });
    }
    setPortalEnv() {
        const camera = this.cameraRef();
        if (!camera?.nativeElement)
            return;
        const [{ frames, background = false, scene, blur, backgroundBlurriness, backgroundIntensity, backgroundRotation, environmentIntensity, environmentRotation, }, gl, fbo, defaultScene,] = [this.options(), this.gl(), this.fbo(), this.defaultScene()];
        if (frames === 1)
            camera.nativeElement.update(gl, this.virtualScene);
        const cleanup = setEnvProps(background, scene, defaultScene, fbo.texture, {
            blur,
            backgroundBlurriness,
            backgroundIntensity,
            backgroundRotation,
            environmentIntensity,
            environmentRotation,
        });
        this.envSet.emit();
        return cleanup;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsEnvironmentPortal, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "19.1.4", type: NgtsEnvironmentPortal, isStandalone: true, selector: "ngts-environment-portal", inputs: { options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null }, content: { classPropertyName: "content", publicName: "content", isSignal: true, isRequired: true, transformFunction: null } }, outputs: { envSet: "envSet" }, viewQueries: [{ propertyName: "cameraRef", first: true, predicate: ["cubeCamera"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-portal [container]="virtualScene">
			<ng-template portalContent let-injector="injector" let-container="container">
				<ng-container
					[ngTemplateOutlet]="content()"
					[ngTemplateOutletInjector]="injector"
					[ngTemplateOutletContext]="{ injector, container }"
				/>

				<ngt-cube-camera #cubeCamera *args="cameraArgs()" />

				@if (files() || preset()) {
					<ngts-environment-cube [options]="envCubeOptions()" (envSet)="onEnvSet()" />
				} @else if (map()) {
					<ngts-environment-map [options]="envMapOptions()" (envSet)="onEnvSet()" />
				}
			</ng-template>
		</ngt-portal>
	`, isInline: true, dependencies: [{ kind: "directive", type: NgtsEnvironmentCube, selector: "ngts-environment-cube", inputs: ["options"], outputs: ["envSet"] }, { kind: "directive", type: NgtsEnvironmentMap, selector: "ngts-environment-map", inputs: ["options"], outputs: ["envSet"] }, { kind: "directive", type: NgtArgs, selector: "ng-template[args]", inputs: ["args"] }, { kind: "component", type: NgtPortal, selector: "ngt-portal", inputs: ["container", "state", "autoRender", "autoRenderPriority"] }, { kind: "directive", type: NgtPortalContent, selector: "ng-template[portalContent]" }, { kind: "directive", type: NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet", "ngTemplateOutletInjector"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsEnvironmentPortal, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-environment-portal',
                    template: `
		<ngt-portal [container]="virtualScene">
			<ng-template portalContent let-injector="injector" let-container="container">
				<ng-container
					[ngTemplateOutlet]="content()"
					[ngTemplateOutletInjector]="injector"
					[ngTemplateOutletContext]="{ injector, container }"
				/>

				<ngt-cube-camera #cubeCamera *args="cameraArgs()" />

				@if (files() || preset()) {
					<ngts-environment-cube [options]="envCubeOptions()" (envSet)="onEnvSet()" />
				} @else if (map()) {
					<ngts-environment-map [options]="envMapOptions()" (envSet)="onEnvSet()" />
				}
			</ng-template>
		</ngt-portal>
	`,
                    imports: [NgtsEnvironmentCube, NgtsEnvironmentMap, NgtArgs, NgtPortal, NgtPortalContent, NgTemplateOutlet],
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }], ctorParameters: () => [] });
class NgtsEnvironmentGround {
    constructor() {
        this.options = input({});
        this.envSet = output();
        this.defaultTexture = injectEnvironment(this.options);
        this.height = computed(() => this.options().ground?.height);
        this.radius = computed(() => this.options().ground?.radius);
        this.scale = computed(() => this.options().ground?.scale ?? 1000);
        this.args = computed(() => [this.options().map || this.defaultTexture()]);
        this.envMapOptions = computed(() => {
            const { map: _, ...options } = this.options();
            const [map] = this.args();
            return Object.assign(options, { map });
        });
        extend({ GroundProjectedEnv });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsEnvironmentGround, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.1.0", version: "19.1.4", type: NgtsEnvironmentGround, isStandalone: true, selector: "ngts-environment-ground", inputs: { options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, outputs: { envSet: "envSet" }, ngImport: i0, template: `
		<ngts-environment-map [options]="envMapOptions()" (envSet)="envSet.emit()" />
		<ngt-ground-projected-env *args="args()" [scale]="scale()" [height]="height()" [radius]="radius()" />
	`, isInline: true, dependencies: [{ kind: "directive", type: NgtsEnvironmentMap, selector: "ngts-environment-map", inputs: ["options"], outputs: ["envSet"] }, { kind: "directive", type: NgtArgs, selector: "ng-template[args]", inputs: ["args"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsEnvironmentGround, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-environment-ground',
                    template: `
		<ngts-environment-map [options]="envMapOptions()" (envSet)="envSet.emit()" />
		<ngt-ground-projected-env *args="args()" [scale]="scale()" [height]="height()" [radius]="radius()" />
	`,
                    imports: [NgtsEnvironmentMap, NgtArgs],
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }], ctorParameters: () => [] });
class NgtsEnvironment {
    constructor() {
        this.options = input({});
        this.content = contentChild(TemplateRef);
        this.envSet = output();
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsEnvironment, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "19.1.4", type: NgtsEnvironment, isStandalone: true, selector: "ngts-environment", inputs: { options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, outputs: { envSet: "envSet" }, queries: [{ propertyName: "content", first: true, predicate: TemplateRef, descendants: true, isSignal: true }], ngImport: i0, template: `
		@let _options = options();
		@let _content = content();

		@if (_options.ground) {
			<ngts-environment-ground [options]="_options" (envSet)="envSet.emit()" />
		} @else if (_options.map) {
			<ngts-environment-map [options]="_options" (envSet)="envSet.emit()" />
		} @else if (_content) {
			<ngts-environment-portal [options]="_options" [content]="_content" (envSet)="envSet.emit()" />
		} @else {
			<ngts-environment-cube [options]="_options" (envSet)="envSet.emit()" />
		}
	`, isInline: true, dependencies: [{ kind: "directive", type: NgtsEnvironmentCube, selector: "ngts-environment-cube", inputs: ["options"], outputs: ["envSet"] }, { kind: "directive", type: NgtsEnvironmentMap, selector: "ngts-environment-map", inputs: ["options"], outputs: ["envSet"] }, { kind: "component", type: NgtsEnvironmentPortal, selector: "ngts-environment-portal", inputs: ["options", "content"], outputs: ["envSet"] }, { kind: "component", type: NgtsEnvironmentGround, selector: "ngts-environment-ground", inputs: ["options"], outputs: ["envSet"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsEnvironment, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-environment',
                    template: `
		@let _options = options();
		@let _content = content();

		@if (_options.ground) {
			<ngts-environment-ground [options]="_options" (envSet)="envSet.emit()" />
		} @else if (_options.map) {
			<ngts-environment-map [options]="_options" (envSet)="envSet.emit()" />
		} @else if (_content) {
			<ngts-environment-portal [options]="_options" [content]="_content" (envSet)="envSet.emit()" />
		} @else {
			<ngts-environment-cube [options]="_options" (envSet)="envSet.emit()" />
		}
	`,
                    imports: [NgtsEnvironmentCube, NgtsEnvironmentMap, NgtsEnvironmentPortal, NgtsEnvironmentGround],
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }] });

const defaultOptions$7 = {
    enabled: true,
    speed: 1,
    rotationIntensity: 1,
    floatIntensity: 1,
    floatingRange: [-0.1, 0.1],
    autoInvalidate: false,
};
class NgtsFloat {
    constructor() {
        this.options = input(defaultOptions$7, { transform: mergeInputs(defaultOptions$7) });
        this.parameters = omit(this.options, [
            'enabled',
            'speed',
            'rotationIntensity',
            'floatIntensity',
            'floatingRange',
            'autoInvalidate',
        ]);
        this.floatRef = viewChild.required('float');
        extend({ Group });
        const offset = Math.random() * 10000;
        injectBeforeRender(({ clock, invalidate }) => {
            const [{ enabled, speed, rotationIntensity, floatingRange, floatIntensity, autoInvalidate }] = [this.options()];
            if (!enabled || speed === 0)
                return;
            if (autoInvalidate)
                invalidate();
            const container = this.floatRef().nativeElement;
            const offsetTime = offset + clock.getElapsedTime();
            container.rotation.x = (Math.cos((offsetTime / 4) * speed) / 8) * rotationIntensity;
            container.rotation.y = (Math.sin((offsetTime / 4) * speed) / 8) * rotationIntensity;
            container.rotation.z = (Math.sin((offsetTime / 4) * speed) / 20) * rotationIntensity;
            let yPosition = Math.sin((offsetTime / 4) * speed) / 10;
            yPosition = MathUtils.mapLinear(yPosition, -0.1, 0.1, floatingRange[0] ?? -0.1, floatingRange[1] ?? 0.1);
            container.position.y = yPosition * floatIntensity;
            container.updateMatrix();
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsFloat, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsFloat, isStandalone: true, selector: "ngts-float", inputs: { options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "floatRef", first: true, predicate: ["float"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-group [parameters]="parameters()">
			<ngt-group #float [matrixAutoUpdate]="false">
				<ng-content />
			</ngt-group>
		</ngt-group>
	`, isInline: true, changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsFloat, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-float',
                    template: `
		<ngt-group [parameters]="parameters()">
			<ngt-group #float [matrixAutoUpdate]="false">
				<ng-content />
			</ngt-group>
		</ngt-group>
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }], ctorParameters: () => [] });

const defaultOptions$6 = {
    toneMapped: false,
    color: 'white',
    form: 'rect',
    scale: 1,
    intensity: 1,
};
class NgtsLightformer {
    constructor() {
        this.side = DoubleSide;
        this.options = input(defaultOptions$6, { transform: mergeInputs(defaultOptions$6) });
        this.parameters = omit(this.options, ['map', 'toneMapped', 'color', 'form', 'scale', 'intensity', 'target']);
        this.intensity = pick(this.options, 'intensity');
        this.color = pick(this.options, 'color');
        this.target = vector3(this.options, 'target', true);
        this.scale = pick(this.options, 'scale');
        this.fixedScale = computed(() => {
            const scale = this.scale();
            return Array.isArray(scale) && scale.length === 2 ? [scale[0], scale[1], 1] : scale;
        });
        this.form = pick(this.options, 'form');
        this.toneMapped = pick(this.options, 'toneMapped');
        this.map = pick(this.options, 'map');
        this.meshRef = viewChild.required('mesh');
        this.defaultMaterialRef = viewChild('defaultMaterial');
        extend({ Mesh, MeshBasicMaterial, RingGeometry, PlaneGeometry });
        effect(() => {
            const material = this.defaultMaterialRef()?.nativeElement;
            if (!material)
                return;
            applyProps(material, { color: this.color() });
            material.color.multiplyScalar(this.intensity());
        });
        effect(() => {
            const target = this.target();
            if (!target)
                return;
            const mesh = this.meshRef().nativeElement;
            mesh.lookAt(target);
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsLightformer, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "19.1.4", type: NgtsLightformer, isStandalone: true, selector: "ngts-lightformer", inputs: { options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "meshRef", first: true, predicate: ["mesh"], descendants: true, isSignal: true }, { propertyName: "defaultMaterialRef", first: true, predicate: ["defaultMaterial"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-mesh #mesh [scale]="fixedScale()" [parameters]="parameters()">
			<ng-content select="[data-lightformer-geometry]">
				@switch (form()) {
					@case ('circle') {
						<ngt-ring-geometry *args="[0, 1, 64]" />
					}
					@case ('ring') {
						<ngt-ring-geometry *args="[0.5, 1, 64]" />
					}
					@case ('rect') {
						<ngt-plane-geometry />
					}
				}
			</ng-content>
			<ng-content select="[data-lightformer-material]">
				<ngt-mesh-basic-material #defaultMaterial [toneMapped]="toneMapped()" [map]="map()" [side]="side" />
			</ng-content>

			<ng-content />
		</ngt-mesh>
	`, isInline: true, dependencies: [{ kind: "directive", type: NgtArgs, selector: "ng-template[args]", inputs: ["args"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsLightformer, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-lightformer',
                    template: `
		<ngt-mesh #mesh [scale]="fixedScale()" [parameters]="parameters()">
			<ng-content select="[data-lightformer-geometry]">
				@switch (form()) {
					@case ('circle') {
						<ngt-ring-geometry *args="[0, 1, 64]" />
					}
					@case ('ring') {
						<ngt-ring-geometry *args="[0.5, 1, 64]" />
					}
					@case ('rect') {
						<ngt-plane-geometry />
					}
				}
			</ng-content>
			<ng-content select="[data-lightformer-material]">
				<ngt-mesh-basic-material #defaultMaterial [toneMapped]="toneMapped()" [map]="map()" [side]="side" />
			</ng-content>

			<ng-content />
		</ngt-mesh>
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    imports: [NgtArgs],
                }]
        }], ctorParameters: () => [] });

const defaultOptions$5 = {
    colorWrite: false,
    depthWrite: false,
};
class NgtsMask {
    constructor() {
        this.id = input(1, { transform: numberAttribute });
        this.options = input(defaultOptions$5, { transform: mergeInputs(defaultOptions$5) });
        this.parameters = omit(this.options, ['colorWrite', 'depthWrite']);
        this.meshRef = viewChild.required('mesh');
        this.colorWrite = pick(this.options, 'colorWrite');
        this.depthWrite = pick(this.options, 'depthWrite');
        this.spread = computed(() => {
            const [id, colorWrite, depthWrite] = [this.id(), this.colorWrite(), this.depthWrite()];
            return {
                colorWrite,
                depthWrite,
                stencilWrite: true,
                stencilRef: id,
                stencilFunc: AlwaysStencilFunc,
                stencilFail: ReplaceStencilOp,
                stencilZFail: ReplaceStencilOp,
                stencilZPass: ReplaceStencilOp,
            };
        });
        extend({ Mesh });
        effect(() => {
            const [mesh, spread] = [this.meshRef().nativeElement, this.spread()];
            Object.assign(mesh.material, spread);
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsMask, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsMask, isStandalone: true, selector: "ngts-mask", inputs: { id: { classPropertyName: "id", publicName: "id", isSignal: true, isRequired: false, transformFunction: null }, options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "meshRef", first: true, predicate: ["mesh"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-mesh #mesh [renderOrder]="-id()" [parameters]="parameters()">
			<ng-content />
		</ngt-mesh>
	`, isInline: true, changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsMask, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-mask',
                    template: `
		<ngt-mesh #mesh [renderOrder]="-id()" [parameters]="parameters()">
			<ng-content />
		</ngt-mesh>
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }], ctorParameters: () => [] });
function injectMask(id, inverse = () => false, { injector } = {}) {
    return assertInjector(injectMask, injector, () => {
        return computed(() => {
            const [_id, _inverse] = [id(), inverse()];
            return {
                stencilWrite: true,
                stencilRef: _id,
                stencilFunc: _inverse ? NotEqualStencilFunc : EqualStencilFunc,
                stencilFail: KeepStencilOp,
                stencilZFail: KeepStencilOp,
                stencilZPass: KeepStencilOp,
            };
        });
    });
}

function getFormatString(format) {
    switch (format) {
        case 64:
            return '-64px';
        case 128:
            return '-128px';
        case 256:
            return '-256px';
        case 512:
            return '-512px';
        default:
            return '';
    }
}
const LIST_URL$1 = 'https://cdn.jsdelivr.net/gh/pmndrs/drei-assets@master/matcaps.json';
const MATCAP_ROOT = 'https://rawcdn.githack.com/emmelleppi/matcaps/9b36ccaaf0a24881a39062d05566c9e92be4aa0d';
function injectMatcapTexture(id = () => 0, { format = () => 1024, onLoad, injector, } = {}) {
    return assertInjector(injectMatcapTexture, injector, () => {
        const matcapList = signal({});
        fetch(LIST_URL$1)
            .then((res) => res.json())
            .then((list) => {
            matcapList.set(list);
        });
        const DEFAULT_MATCAP = computed(() => matcapList()[0]);
        const numTot = computed(() => Object.keys(matcapList()).length);
        const fileHash = computed(() => {
            const idValue = id();
            if (typeof idValue === 'string') {
                return idValue;
            }
            if (typeof idValue === 'number') {
                return matcapList()[idValue];
            }
            return null;
        });
        const fileName = computed(() => `${fileHash() || DEFAULT_MATCAP()}${getFormatString(format())}.png`);
        const url = computed(() => `${MATCAP_ROOT}/${format()}/${fileName()}`);
        const matcapTexture = injectTexture(url, { onLoad });
        return { url, texture: matcapTexture, numTot };
    });
}
class NgtsMatcapTexture {
    constructor() {
        this.matcapTexture = input();
        this.matcapTextureLoaded = output();
        this.template = inject(TemplateRef);
        this.vcr = inject(ViewContainerRef);
        this.id = computed(() => this.matcapTexture()?.id ?? 0);
        this.format = computed(() => this.matcapTexture()?.format ?? 1024);
        const { texture } = injectMatcapTexture(this.id, {
            format: this.format,
            onLoad: this.matcapTextureLoaded.emit.bind(this.matcapTextureLoaded),
        });
        effect(() => {
            this.ref = this.vcr.createEmbeddedView(this.template, { $implicit: texture });
            this.ref.detectChanges();
        });
        inject(DestroyRef).onDestroy(() => {
            this.ref?.destroy();
        });
    }
    static ngTemplateContextGuard(_, ctx) {
        return true;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsMatcapTexture, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "17.1.0", version: "19.1.4", type: NgtsMatcapTexture, isStandalone: true, selector: "ng-template[matcapTexture]", inputs: { matcapTexture: { classPropertyName: "matcapTexture", publicName: "matcapTexture", isSignal: true, isRequired: false, transformFunction: null } }, outputs: { matcapTextureLoaded: "matcapTextureLoaded" }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsMatcapTexture, decorators: [{
            type: Directive,
            args: [{ selector: 'ng-template[matcapTexture]' }]
        }], ctorParameters: () => [] });

const NORMAL_ROOT = 'https://rawcdn.githack.com/pmndrs/drei-assets/7a3104997e1576f83472829815b00880d88b32fb';
const LIST_URL = 'https://cdn.jsdelivr.net/gh/pmndrs/drei-assets@master/normals/normals.json';
function injectNormalTexture(id = () => 0, { settings = () => ({}), onLoad, injector, }) {
    return assertInjector(injectNormalTexture, injector, () => {
        const normalList = signal({});
        fetch(LIST_URL)
            .then((res) => res.json())
            .then((list) => {
            normalList.set(list);
        });
        const DEFAULT_NORMAL = computed(() => normalList()[0]);
        const numTot = computed(() => Object.keys(normalList()).length);
        const fileHash = computed(() => {
            const idValue = id();
            if (typeof idValue === 'string') {
                return idValue;
            }
            if (typeof idValue === 'number') {
                return normalList()[idValue];
            }
            return null;
        });
        const imageName = computed(() => fileHash() || DEFAULT_NORMAL());
        const url = computed(() => `${NORMAL_ROOT}/normals/${imageName()}`);
        const normalTexture = injectTexture(url, { onLoad });
        effect(() => {
            const texture = normalTexture();
            if (!texture)
                return;
            const { anisotropy = 1, repeat = [1, 1], offset = [0, 0] } = settings();
            texture.wrapS = texture.wrapT = RepeatWrapping;
            texture.repeat = new Vector2(repeat[0], repeat[1]);
            texture.offset = new Vector2(offset[0], offset[1]);
            texture.anisotropy = anisotropy;
        });
        return { url, texture: normalTexture, numTot };
    });
}
class NgtsNormalTexture {
    constructor() {
        this.normalTexture = input();
        this.normalTextureLoaded = output();
        this.template = inject(TemplateRef);
        this.vcr = inject(ViewContainerRef);
        this.id = computed(() => this.normalTexture()?.id ?? 0);
        this.settings = computed(() => {
            const { id: _, ...settings } = this.normalTexture() || {};
            return settings;
        });
        const { texture } = injectNormalTexture(this.id, {
            settings: this.settings,
            onLoad: this.normalTextureLoaded.emit.bind(this.normalTextureLoaded),
        });
        effect(() => {
            this.ref = this.vcr.createEmbeddedView(this.template, { $implicit: texture });
            this.ref.detectChanges();
        });
        inject(DestroyRef).onDestroy(() => {
            this.ref?.destroy();
        });
    }
    static ngTemplateContextGuard(_, ctx) {
        return true;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsNormalTexture, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "17.1.0", version: "19.1.4", type: NgtsNormalTexture, isStandalone: true, selector: "ng-template[normalTexture]", inputs: { normalTexture: { classPropertyName: "normalTexture", publicName: "normalTexture", isSignal: true, isRequired: false, transformFunction: null } }, outputs: { normalTextureLoaded: "normalTextureLoaded" }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsNormalTexture, decorators: [{
            type: Directive,
            args: [{ selector: 'ng-template[normalTexture]' }]
        }], ctorParameters: () => [] });

const defaultOptions$4 = {
    castShadow: true,
    bias: 0.001,
    mapSize: 512,
    size: 5,
    near: 0.5,
    far: 500,
    frames: 1,
    position: [0, 0, 0],
    radius: 1,
    amount: 8,
    intensity: getVersion() >= 155 ? Math.PI : 1,
    ambient: 0.5,
};
class NgtsRandomizedLights {
    constructor() {
        this.options = input(defaultOptions$4, { transform: mergeInputs(defaultOptions$4) });
        this.parameters = omit(this.options, Object.keys(defaultOptions$4));
        this.lightsRef = viewChild.required('lights');
        this.accumulativeShadows = inject(NgtsAccumulativeShadows);
        this.castShadow = pick(this.options, 'castShadow');
        this.bias = pick(this.options, 'bias');
        this.intensity = pick(this.options, 'intensity');
        this.amount = pick(this.options, 'amount');
        this.mapSize = pick(this.options, 'mapSize');
        this.shadowMapSize = computed(() => [this.mapSize(), this.mapSize()]);
        this.position = pick(this.options, 'position');
        this.length = computed(() => new Vector3(...this.position()).length());
        this.count = computed(() => Array.from({ length: this.amount() }, (_, index) => index));
        this.size = pick(this.options, 'size');
        this.near = pick(this.options, 'near');
        this.far = pick(this.options, 'far');
        this.cameraArgs = computed(() => [-this.size(), this.size(), this.size(), -this.size(), this.near(), this.far()]);
        extend({ Group, DirectionalLight, OrthographicCamera, Vector2 });
        effect((onCleanup) => {
            const lights = this.lightsRef().nativeElement;
            this.accumulativeShadows.lightsMap.set(lights.uuid, this.update.bind(this));
            onCleanup(() => this.accumulativeShadows.lightsMap.delete(lights.uuid));
        });
    }
    update() {
        let light;
        const lights = this.lightsRef().nativeElement;
        if (lights) {
            const [{ ambient, radius, position }, length] = [this.options(), this.length()];
            for (let i = 0; i < lights.children.length; i++) {
                light = lights.children[i];
                if (Math.random() > ambient) {
                    light.position.set(position[0] + MathUtils.randFloatSpread(radius), position[1] + MathUtils.randFloatSpread(radius), position[2] + MathUtils.randFloatSpread(radius));
                }
                else {
                    const lambda = Math.acos(2 * Math.random() - 1) - Math.PI / 2.0;
                    const phi = 2 * Math.PI * Math.random();
                    light.position.set(Math.cos(lambda) * Math.cos(phi) * length, Math.abs(Math.cos(lambda) * Math.sin(phi) * length), Math.sin(lambda) * length);
                }
            }
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsRandomizedLights, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "19.1.4", type: NgtsRandomizedLights, isStandalone: true, selector: "ngts-randomized-lights", inputs: { options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "lightsRef", first: true, predicate: ["lights"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-group #lights [parameters]="parameters()">
			@for (i of count(); track $index) {
				<ngt-directional-light [castShadow]="castShadow()" [intensity]="intensity() / amount()">
					<ngt-value [rawValue]="bias()" attach="shadow.bias" />
					<ngt-vector2 *args="shadowMapSize()" attach="shadow.mapSize" />
					<ngt-orthographic-camera *args="cameraArgs()" attach="shadow.camera" />
				</ngt-directional-light>
			}
		</ngt-group>
	`, isInline: true, dependencies: [{ kind: "directive", type: NgtArgs, selector: "ng-template[args]", inputs: ["args"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsRandomizedLights, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-randomized-lights',
                    template: `
		<ngt-group #lights [parameters]="parameters()">
			@for (i of count(); track $index) {
				<ngt-directional-light [castShadow]="castShadow()" [intensity]="intensity() / amount()">
					<ngt-value [rawValue]="bias()" attach="shadow.bias" />
					<ngt-vector2 *args="shadowMapSize()" attach="shadow.mapSize" />
					<ngt-orthographic-camera *args="cameraArgs()" attach="shadow.camera" />
				</ngt-directional-light>
			}
		</ngt-group>
	`,
                    imports: [NgtArgs],
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }], ctorParameters: () => [] });

extend({ Group });
class NgtsRenderTextureContainer {
    constructor() {
        this.fbo = input.required();
        this.renderPriority = input.required();
        this.frames = input.required();
        this.injector = input.required();
        this.store = injectStore();
        effect((onCleanup) => {
            const [renderPriority, { internal }] = [this.renderPriority(), this.store.state()];
            let count = 0;
            let oldAutoClear;
            let oldXrEnabled;
            let oldRenderTarget;
            let oldIsPresenting;
            const cleanup = internal.subscribe(({ gl, scene, camera }) => {
                const [fbo, frames] = [this.fbo(), this.frames()];
                // NOTE: render  the frames ^ 2
                //  due to some race condition, we want to render double the frames here.
                if (frames === Infinity || count < frames * frames) {
                    oldAutoClear = gl.autoClear;
                    oldXrEnabled = gl.xr.enabled;
                    oldRenderTarget = gl.getRenderTarget();
                    oldIsPresenting = gl.xr.isPresenting;
                    gl.autoClear = true;
                    gl.xr.enabled = false;
                    gl.xr.isPresenting = false;
                    gl.setRenderTarget(fbo);
                    gl.render(scene, camera);
                    gl.setRenderTarget(oldRenderTarget);
                    gl.autoClear = oldAutoClear;
                    gl.xr.enabled = oldXrEnabled;
                    gl.xr.isPresenting = oldIsPresenting;
                    count++;
                }
            }, renderPriority, this.store);
            onCleanup(() => {
                cleanup();
            });
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsRenderTextureContainer, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "17.1.0", version: "19.1.4", type: NgtsRenderTextureContainer, isStandalone: true, selector: "[ngtsRenderTextureContainer]", inputs: { fbo: { classPropertyName: "fbo", publicName: "fbo", isSignal: true, isRequired: true, transformFunction: null }, renderPriority: { classPropertyName: "renderPriority", publicName: "renderPriority", isSignal: true, isRequired: true, transformFunction: null }, frames: { classPropertyName: "frames", publicName: "frames", isSignal: true, isRequired: true, transformFunction: null }, injector: { classPropertyName: "injector", publicName: "injector", isSignal: true, isRequired: true, transformFunction: null } }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsRenderTextureContainer, decorators: [{
            type: Directive,
            args: [{ selector: '[ngtsRenderTextureContainer]' }]
        }], ctorParameters: () => [] });
const defaultOptions$3 = {
    samples: 8,
    renderPriority: 0,
    eventPriority: 0,
    frames: Infinity,
    stencilBuffer: false,
    depthBuffer: true,
    generateMipmaps: false,
};
class NgtsRenderTextureContent {
    static ngTemplateContextGuard(_, ctx) {
        return true;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsRenderTextureContent, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.1.4", type: NgtsRenderTextureContent, isStandalone: true, selector: "ng-template[renderTextureContent]", ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsRenderTextureContent, decorators: [{
            type: Directive,
            args: [{ selector: 'ng-template[renderTextureContent]' }]
        }] });
let incrementId = 0;
class NgtsRenderTexture {
    constructor() {
        this.attach = input('map');
        this.options = input(defaultOptions$3, { transform: mergeInputs(defaultOptions$3) });
        this.parameters = omit(this.options, [
            'samples',
            'renderPriority',
            'eventPriority',
            'frames',
            'stencilBuffer',
            'depthBuffer',
            'generateMipmaps',
            'compute',
            'width',
            'height',
        ]);
        this.content = contentChild.required(NgtsRenderTextureContent, { read: TemplateRef });
        this.store = injectStore();
        this.size = this.store.select('size');
        this.viewport = this.store.select('viewport');
        this.fboParams = computed(() => ({
            width: (this.options().width || this.size().width) * this.viewport().dpr,
            height: (this.options().height || this.size().height) * this.viewport().dpr,
            settings: {
                samples: this.options().samples,
                depthBuffer: this.options().depthBuffer,
                stencilBuffer: this.options().stencilBuffer,
                generateMipmaps: this.options().generateMipmaps,
            },
        }));
        this.renderPriority = pick(this.options, 'renderPriority');
        this.frames = pick(this.options, 'frames');
        this.fbo = injectFBO(this.fboParams);
        this.virtualScene = computed(() => {
            const scene = prepare(new Scene());
            scene.name = `ngts-render-texture-virtual-scene-${incrementId++}`;
            return scene;
        });
        this.eventPriority = pick(this.options, 'eventPriority');
        this.compute = computed(() => this.options().compute || this.uvCompute);
        this.uvCompute = (event, root, previous) => {
            const fbo = this.fbo();
            if (!fbo)
                return;
            const state = root.snapshot;
            const previousState = previous?.snapshot;
            // Since this is only a texture it does not have an easy way to obtain the parent, which we
            // need to transform event coordinates to local coordinates. We use r3f internals to find the
            // next Object3D.
            let parent = getLocalState(fbo.texture)?.parent();
            while (parent && !(parent instanceof Object3D)) {
                parent = getLocalState(parent)?.parent();
            }
            if (!parent)
                return;
            // First we call the previous state-onion-layers compute, this is what makes it possible to nest portals
            if (previousState && !previousState.raycaster.camera) {
                previousState.events.compute?.(event, previous, previous.snapshot.previousRoot);
            }
            // We run a quick check against the parent, if it isn't hit there's no need to raycast at all
            const [intersection] = previousState?.raycaster.intersectObject(parent) || [];
            if (!intersection)
                return;
            // We take that hits uv coords, set up this layers raycaster, et voilà, we have raycasting on arbitrary surfaces
            const uv = intersection.uv;
            if (!uv)
                return;
            state.raycaster.setFromCamera(state.pointer.set(uv.x * 2 - 1, uv.y * 2 - 1), state.camera);
        };
    }
    onPointerOver() {
        /* noop */
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsRenderTexture, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsRenderTexture, isStandalone: true, selector: "ngts-render-texture", inputs: { attach: { classPropertyName: "attach", publicName: "attach", isSignal: true, isRequired: false, transformFunction: null }, options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, queries: [{ propertyName: "content", first: true, predicate: NgtsRenderTextureContent, descendants: true, read: TemplateRef, isSignal: true }], ngImport: i0, template: `
		<ngt-portal [container]="virtualScene()" [state]="{ events: { compute: compute(), priority: eventPriority() } }">
			<ng-template portalContent let-injector="injector" let-container="container">
				<ng-container
					ngtsRenderTextureContainer
					[fbo]="fbo()"
					[renderPriority]="renderPriority()"
					[frames]="frames()"
					[injector]="injector"
					[ngTemplateOutlet]="content()"
					[ngTemplateOutletInjector]="injector"
					[ngTemplateOutletContext]="{ container, injector }"
				>
					<ngt-group (pointerover)="onPointerOver()" />
				</ng-container>
			</ng-template>
		</ngt-portal>

		<ngt-primitive *args="[fbo().texture]" [attach]="attach()" [parameters]="parameters()" />
	`, isInline: true, dependencies: [{ kind: "component", type: NgtPortal, selector: "ngt-portal", inputs: ["container", "state", "autoRender", "autoRenderPriority"] }, { kind: "directive", type: NgtsRenderTextureContainer, selector: "[ngtsRenderTextureContainer]", inputs: ["fbo", "renderPriority", "frames", "injector"] }, { kind: "directive", type: NgtPortalContent, selector: "ng-template[portalContent]" }, { kind: "directive", type: NgtArgs, selector: "ng-template[args]", inputs: ["args"] }, { kind: "directive", type: NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet", "ngTemplateOutletInjector"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsRenderTexture, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-render-texture',
                    template: `
		<ngt-portal [container]="virtualScene()" [state]="{ events: { compute: compute(), priority: eventPriority() } }">
			<ng-template portalContent let-injector="injector" let-container="container">
				<ng-container
					ngtsRenderTextureContainer
					[fbo]="fbo()"
					[renderPriority]="renderPriority()"
					[frames]="frames()"
					[injector]="injector"
					[ngTemplateOutlet]="content()"
					[ngTemplateOutletInjector]="injector"
					[ngTemplateOutletContext]="{ container, injector }"
				>
					<ngt-group (pointerover)="onPointerOver()" />
				</ng-container>
			</ng-template>
		</ngt-portal>

		<ngt-primitive *args="[fbo().texture]" [attach]="attach()" [parameters]="parameters()" />
	`,
                    imports: [NgtPortal, NgtsRenderTextureContainer, NgtPortalContent, NgtArgs, NgTemplateOutlet],
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }] });

function calcPosFromAngles(inclination, azimuth, vector = new Vector3()) {
    const theta = Math.PI * (inclination - 0.5);
    const phi = 2 * Math.PI * (azimuth - 0.5);
    vector.x = Math.cos(phi);
    vector.y = Math.sin(theta);
    vector.z = Math.sin(phi);
    return vector;
}
const defaultOptions$2 = {
    distance: 1000,
    inclination: 0.6,
    azimuth: 0.1,
    mieCoefficient: 0.005,
    mieDirectionalG: 0.8,
    rayleigh: 0.5,
    turbidity: 10,
};
class NgtsSky {
    constructor() {
        this.options = input(defaultOptions$2, { transform: mergeInputs(defaultOptions$2) });
        this.parameters = omit(this.options, [
            'distance',
            'inclination',
            'azimuth',
            'sunPosition',
            'turbidity',
            'mieCoefficient',
            'mieDirectionalG',
            'sunPosition',
        ]);
        this.distance = pick(this.options, 'distance');
        this.turbidity = pick(this.options, 'turbidity');
        this.mieCoefficient = pick(this.options, 'mieCoefficient');
        this.mieDirectionalG = pick(this.options, 'mieDirectionalG');
        this.rayleigh = pick(this.options, 'rayleigh');
        this.scale = computed(() => new Vector3().setScalar(this.distance()));
        this.inclination = pick(this.options, 'inclination');
        this.azimuth = pick(this.options, 'azimuth');
        this.sunPosition = vector3(this.options, 'sunPosition', true);
        this.calculatedSunPosition = computed(() => {
            const sunPosition = this.sunPosition();
            if (sunPosition)
                return sunPosition;
            return calcPosFromAngles(this.inclination(), this.azimuth());
        });
        this.sky = new Sky();
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsSky, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.1.0", version: "19.1.4", type: NgtsSky, isStandalone: true, selector: "ngts-sky", inputs: { options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, ngImport: i0, template: `
		<ngt-primitive *args="[sky]" [parameters]="parameters()" [scale]="scale()">
			<ngt-value attach="material.uniforms.mieCoefficient.value" [rawValue]="mieCoefficient()" />
			<ngt-value attach="material.uniforms.mieDirectionalG.value" [rawValue]="mieDirectionalG()" />
			<ngt-value attach="material.uniforms.rayleigh.value" [rawValue]="rayleigh()" />
			<ngt-value attach="material.uniforms.sunPosition.value" [rawValue]="calculatedSunPosition()" />
			<ngt-value attach="material.uniforms.turbidity.value" [rawValue]="turbidity()" />
			<ng-content />
		</ngt-primitive>
	`, isInline: true, dependencies: [{ kind: "directive", type: NgtArgs, selector: "ng-template[args]", inputs: ["args"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsSky, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-sky',
                    template: `
		<ngt-primitive *args="[sky]" [parameters]="parameters()" [scale]="scale()">
			<ngt-value attach="material.uniforms.mieCoefficient.value" [rawValue]="mieCoefficient()" />
			<ngt-value attach="material.uniforms.mieDirectionalG.value" [rawValue]="mieDirectionalG()" />
			<ngt-value attach="material.uniforms.rayleigh.value" [rawValue]="rayleigh()" />
			<ngt-value attach="material.uniforms.sunPosition.value" [rawValue]="calculatedSunPosition()" />
			<ngt-value attach="material.uniforms.turbidity.value" [rawValue]="turbidity()" />
			<ng-content />
		</ngt-primitive>
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    imports: [NgtArgs],
                }]
        }] });

function isSpotLight(child) {
    return child?.isSpotLight;
}
const defaultVolumetricMeshOptions = {
    opacity: 1,
    distance: 5,
    angle: 0.15,
    attenuation: 5,
    anglePower: 5,
    color: 'white',
};
class NgtsVolumetricMesh {
    constructor() {
        this.options = input(defaultVolumetricMeshOptions, { transform: mergeInputs(defaultVolumetricMeshOptions) });
        this.meshRef = viewChild.required('mesh');
        this.store = injectStore();
        this.size = this.store.select('size');
        this.dpr = this.store.select('viewport', 'dpr');
        this.camera = this.store.select('camera');
        this.radiusTop = pick(this.options, 'radiusTop');
        this.radiusBottom = pick(this.options, 'radiusBottom');
        this.angle = pick(this.options, 'angle');
        this.distance = pick(this.options, 'distance');
        this.normalizedRadiusTop = computed(() => (this.radiusTop() === undefined ? 0.1 : this.radiusTop()));
        this.normalizedRadiusBottom = computed(() => this.radiusBottom() === undefined ? this.angle() * 7 : this.radiusBottom());
        this.material = new SpotLightMaterial();
        this.opacity = pick(this.options, 'opacity');
        this.color = pick(this.options, 'color');
        this.attenuation = pick(this.options, 'attenuation');
        this.anglePower = pick(this.options, 'anglePower');
        this.depthBuffer = pick(this.options, 'depthBuffer');
        this.geometry = computed(() => {
            const geometry = new CylinderGeometry(this.normalizedRadiusTop(), this.normalizedRadiusBottom(), this.distance(), 128, 64, true);
            geometry.applyMatrix4(new Matrix4().makeTranslation(0, -this.distance() / 2, 0));
            geometry.applyMatrix4(new Matrix4().makeRotationX(-Math.PI / 2));
            return geometry;
        });
        extend({ Mesh });
        const vec = new Vector3();
        injectBeforeRender(() => {
            const mesh = this.meshRef().nativeElement;
            const material = this.material;
            material.uniforms['spotPosition'].value.copy(mesh.getWorldPosition(vec));
            if (mesh.parent) {
                mesh.lookAt(mesh.parent.target.getWorldPosition(vec));
            }
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsVolumetricMesh, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsVolumetricMesh, isStandalone: true, selector: "ngts-volumetric-mesh", inputs: { options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "meshRef", first: true, predicate: ["mesh"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-mesh #mesh [geometry]="geometry()" [raycast]="null">
			<ngt-primitive *args="[material]" attach="material">
				<ngt-value attach="uniforms.opacity.value" [rawValue]="opacity()" />
				<ngt-value attach="uniforms.lightColor.value" [rawValue]="color()" />
				<ngt-value attach="uniforms.attenuation.value" [rawValue]="attenuation()" />
				<ngt-value attach="uniforms.anglePower.value" [rawValue]="anglePower()" />
				<ngt-value attach="uniforms.depth.value" [rawValue]="depthBuffer()" />
				<ngt-value attach="uniforms.cameraNear.value" [rawValue]="camera().near" />
				<ngt-value attach="uniforms.cameraFar.value" [rawValue]="camera().far" />
				<ngt-value
					attach="uniforms.resolution.value"
					[rawValue]="depthBuffer() ? [size().width * dpr(), size().height * dpr()] : [0, 0]"
				/>
			</ngt-primitive>
		</ngt-mesh>
	`, isInline: true, dependencies: [{ kind: "directive", type: NgtArgs, selector: "ng-template[args]", inputs: ["args"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsVolumetricMesh, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-volumetric-mesh',
                    template: `
		<ngt-mesh #mesh [geometry]="geometry()" [raycast]="null">
			<ngt-primitive *args="[material]" attach="material">
				<ngt-value attach="uniforms.opacity.value" [rawValue]="opacity()" />
				<ngt-value attach="uniforms.lightColor.value" [rawValue]="color()" />
				<ngt-value attach="uniforms.attenuation.value" [rawValue]="attenuation()" />
				<ngt-value attach="uniforms.anglePower.value" [rawValue]="anglePower()" />
				<ngt-value attach="uniforms.depth.value" [rawValue]="depthBuffer()" />
				<ngt-value attach="uniforms.cameraNear.value" [rawValue]="camera().near" />
				<ngt-value attach="uniforms.cameraFar.value" [rawValue]="camera().far" />
				<ngt-value
					attach="uniforms.resolution.value"
					[rawValue]="depthBuffer() ? [size().width * dpr(), size().height * dpr()] : [0, 0]"
				/>
			</ngt-primitive>
		</ngt-mesh>
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    imports: [NgtArgs],
                }]
        }], ctorParameters: () => [] });
function injectSpotLightCommon(spotLight, mesh, width, height, distance, injector) {
    assertInjector(injectSpotLightCommon, injector, () => {
        const pos = new Vector3();
        const dir = new Vector3();
        effect(() => {
            const [_spotLight, _width, _height] = [spotLight()?.nativeElement, width(), height()];
            if (!_spotLight)
                return;
            if (isSpotLight(_spotLight)) {
                _spotLight.shadow.mapSize.set(_width, _height);
                if (_spotLight.shadow.map) {
                    _spotLight.shadow.map.setSize(_width, _height);
                }
                _spotLight.shadow.needsUpdate = true;
            }
            else {
                throw new Error('NgtsSpotLightShadow must be a child of a NgtsSpotLight');
            }
        });
        injectBeforeRender(() => {
            const [_spotLight, _mesh] = [spotLight()?.nativeElement, mesh()?.nativeElement];
            if (!_spotLight || !_mesh)
                return;
            const A = _spotLight.position;
            const B = _spotLight.target.position;
            dir.copy(B).sub(A);
            const len = dir.length();
            dir.normalize().multiplyScalar(len * distance());
            pos.copy(A).add(dir);
            _mesh.position.copy(pos);
            _mesh.lookAt(_spotLight.target.position);
        });
    });
}
const defaultSpotLightShadowOptions = {
    distance: 0.4,
    alphaTest: 0.5,
    scale: 1,
    width: 512,
    height: 512,
};
class NgtsSpotLightShadowShader {
    constructor() {
        this.DoubleSide = DoubleSide;
        this.shader = input.required();
        this.options = input(defaultSpotLightShadowOptions, { transform: mergeInputs(defaultSpotLightShadowOptions) });
        this.mesh = viewChild.required('mesh');
        this.spotLight = inject(NgtsSpotLight);
        this.debug = this.spotLight.debug;
        this.scale = pick(this.options, 'scale');
        this.alphaTest = pick(this.options, 'alphaTest');
        this.width = pick(this.options, 'width');
        this.height = pick(this.options, 'height');
        this.map = pick(this.options, 'map');
        this.distance = pick(this.options, 'distance');
        this.renderTarget = computed(() => {
            return new WebGLRenderTarget(this.width(), this.height(), {
                format: RGBAFormat,
                stencilBuffer: false,
                // depthTexture: null!
            });
        });
        this.texture = computed(() => {
            const renderTarget = this.renderTarget();
            renderTarget.texture.wrapT = renderTarget.texture.wrapS = RepeatWrapping;
            return renderTarget.texture;
        });
        this.fsQuad = computed(() => {
            return new FullScreenQuad(new ShaderMaterial({
                uniforms: this.uniforms,
                vertexShader: /* language=glsl glsl */ `
          varying vec2 vUv;

          void main() {
            vUv = uv;
            gl_Position = projectionMatrix * modelViewMatrix * vec4(position, 1.0);
          }
        `,
                fragmentShader: this.shader(),
            }));
        });
        this.uniforms = {
            uShadowMap: { value: undefined },
            uTime: { value: 0 },
        };
        extend({ Mesh, PlaneGeometry, MeshBasicMaterial });
        injectSpotLightCommon(this.spotLight.spotLight, this.mesh, this.width, this.height, this.distance);
        effect(() => {
            this.uniforms.uShadowMap.value = this.map();
        });
        effect((onCleanup) => {
            const fsQuad = this.fsQuad();
            onCleanup(() => {
                fsQuad.material.dispose();
                fsQuad.dispose();
            });
        });
        effect((onCleanup) => {
            const renderTarget = this.renderTarget();
            onCleanup(() => {
                renderTarget.dispose();
            });
        });
        injectBeforeRender(({ gl, delta }) => {
            this.uniforms.uTime.value += delta;
            gl.setRenderTarget(this.renderTarget());
            this.fsQuad().render(gl);
            gl.setRenderTarget(null);
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsSpotLightShadowShader, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsSpotLightShadowShader, isStandalone: true, selector: "ngts-spot-light-shadow-shader", inputs: { shader: { classPropertyName: "shader", publicName: "shader", isSignal: true, isRequired: true, transformFunction: null }, options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "mesh", first: true, predicate: ["mesh"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-mesh #mesh [scale]="scale()" [castShadow]="true">
			<ngt-plane-geometry />
			<ngt-mesh-basic-material
				[transparent]="true"
				[side]="DoubleSide"
				[alphaTest]="alphaTest()"
				[alphaMap]="texture()"
				[opacity]="debug() ? 1 : 0"
			>
				<ng-content />
			</ngt-mesh-basic-material>
		</ngt-mesh>
	`, isInline: true, changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsSpotLightShadowShader, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-spot-light-shadow-shader',
                    template: `
		<ngt-mesh #mesh [scale]="scale()" [castShadow]="true">
			<ngt-plane-geometry />
			<ngt-mesh-basic-material
				[transparent]="true"
				[side]="DoubleSide"
				[alphaTest]="alphaTest()"
				[alphaMap]="texture()"
				[opacity]="debug() ? 1 : 0"
			>
				<ng-content />
			</ngt-mesh-basic-material>
		</ngt-mesh>
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }], ctorParameters: () => [] });
class NgtsSpotLightShadowNoShader {
    constructor() {
        this.DoubleSide = DoubleSide;
        this.options = input(defaultSpotLightShadowOptions, { transform: mergeInputs(defaultSpotLightShadowOptions) });
        this.mesh = viewChild.required('mesh');
        this.spotLight = inject(NgtsSpotLight);
        this.debug = this.spotLight.debug;
        this.alphaTest = pick(this.options, 'alphaTest');
        this.scale = pick(this.options, 'scale');
        this.map = computed(() => {
            const map = this.options().map;
            if (map) {
                map.wrapS = map.wrapT = RepeatWrapping;
            }
            return map;
        });
        this.width = pick(this.options, 'width');
        this.height = pick(this.options, 'height');
        this.distance = pick(this.options, 'distance');
        extend({ Mesh, PlaneGeometry, MeshBasicMaterial });
        injectSpotLightCommon(this.spotLight.spotLight, this.mesh, this.width, this.height, this.distance);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsSpotLightShadowNoShader, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.2.0", version: "19.1.4", type: NgtsSpotLightShadowNoShader, isStandalone: true, selector: "ngts-spot-light-shadow-no-shader", inputs: { options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "mesh", first: true, predicate: ["mesh"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-mesh #mesh [scale]="scale()" [castShadow]="true">
			<ngt-plane-geometry />
			<ngt-mesh-basic-material
				[transparent]="true"
				[side]="DoubleSide"
				[alphaTest]="alphaTest()"
				[alphaMap]="map()"
				[opacity]="debug() ? 1 : 0"
			>
				<ng-content />
			</ngt-mesh-basic-material>
		</ngt-mesh>
	`, isInline: true, changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsSpotLightShadowNoShader, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-spot-light-shadow-no-shader',
                    template: `
		<ngt-mesh #mesh [scale]="scale()" [castShadow]="true">
			<ngt-plane-geometry />
			<ngt-mesh-basic-material
				[transparent]="true"
				[side]="DoubleSide"
				[alphaTest]="alphaTest()"
				[alphaMap]="map()"
				[opacity]="debug() ? 1 : 0"
			>
				<ng-content />
			</ngt-mesh-basic-material>
		</ngt-mesh>
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }], ctorParameters: () => [] });
class NgtsSpotLightShadow {
    constructor() {
        this.shader = input();
        this.options = input(defaultSpotLightShadowOptions, { transform: mergeInputs(defaultSpotLightShadowOptions) });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsSpotLightShadow, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "19.1.4", type: NgtsSpotLightShadow, isStandalone: true, selector: "ngts-spot-light-shadow", inputs: { shader: { classPropertyName: "shader", publicName: "shader", isSignal: true, isRequired: false, transformFunction: null }, options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, ngImport: i0, template: `
		@if (shader(); as shader) {
			<ngts-spot-light-shadow-shader [shader]="shader" [options]="options()" />
		} @else {
			<ngts-spot-light-shadow-no-shader [options]="options()" />
		}
	`, isInline: true, dependencies: [{ kind: "component", type: NgtsSpotLightShadowShader, selector: "ngts-spot-light-shadow-shader", inputs: ["shader", "options"] }, { kind: "component", type: NgtsSpotLightShadowNoShader, selector: "ngts-spot-light-shadow-no-shader", inputs: ["options"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsSpotLightShadow, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-spot-light-shadow',
                    template: `
		@if (shader(); as shader) {
			<ngts-spot-light-shadow-shader [shader]="shader" [options]="options()" />
		} @else {
			<ngts-spot-light-shadow-no-shader [options]="options()" />
		}
	`,
                    imports: [NgtsSpotLightShadowShader, NgtsSpotLightShadowNoShader],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                }]
        }] });
const defaultOptions$1 = {
    opacity: 1,
    color: 'white',
    distance: 5,
    angle: 0.15,
    attenuation: 5,
    anglePower: 5,
    volumetric: true,
    debug: false,
};
class NgtsSpotLight {
    constructor() {
        this.SpotLightHelper = SpotLightHelper;
        this.options = input(defaultOptions$1, { transform: mergeInputs(defaultOptions$1) });
        this.parameters = omit(this.options, [
            'opacity',
            'radiusTop',
            'radiusBottom',
            'depthBuffer',
            'color',
            'distance',
            'angle',
            'attenuation',
            'anglePower',
            'volumetric',
            'debug',
        ]);
        this.volumetricOptions = pick(this.options, [
            'opacity',
            'radiusTop',
            'radiusBottom',
            'depthBuffer',
            'color',
            'distance',
            'angle',
            'attenuation',
            'anglePower',
            'debug',
        ]);
        this.spotLight = viewChild.required('spotLight');
        this.debug = pick(this.options, 'debug');
        this.angle = pick(this.options, 'angle');
        this.color = pick(this.options, 'color');
        this.distance = pick(this.options, 'distance');
        this.volumetric = pick(this.options, 'volumetric');
        extend({ Group, SpotLight });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsSpotLight, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "19.1.4", type: NgtsSpotLight, isStandalone: true, selector: "ngts-spot-light", inputs: { options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "spotLight", first: true, predicate: ["spotLight"], descendants: true, isSignal: true }], ngImport: i0, template: `
		<ngt-group>
			<ngt-spot-light
				#spotLight
				[angle]="angle()"
				[color]="color()"
				[distance]="distance()"
				[castShadow]="true"
				[parameters]="parameters()"
			>
				@if (volumetric()) {
					<ngts-volumetric-mesh [options]="volumetricOptions()" />
				}

				@if (debug()) {
					<ngts-helper [type]="SpotLightHelper" />
				}
			</ngt-spot-light>
			<ng-content />
		</ngt-group>
	`, isInline: true, dependencies: [{ kind: "component", type: NgtsVolumetricMesh, selector: "ngts-volumetric-mesh", inputs: ["options"] }, { kind: "component", type: NgtsHelper, selector: "ngts-helper", inputs: ["type", "options"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsSpotLight, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-spot-light',
                    template: `
		<ngt-group>
			<ngt-spot-light
				#spotLight
				[angle]="angle()"
				[color]="color()"
				[distance]="distance()"
				[castShadow]="true"
				[parameters]="parameters()"
			>
				@if (volumetric()) {
					<ngts-volumetric-mesh [options]="volumetricOptions()" />
				}

				@if (debug()) {
					<ngts-helper [type]="SpotLightHelper" />
				}
			</ngt-spot-light>
			<ng-content />
		</ngt-group>
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    imports: [NgtsVolumetricMesh, NgtsHelper],
                }]
        }], ctorParameters: () => [] });

const presets = {
    rembrandt: {
        main: [1, 2, 1],
        fill: [-2, -0.5, -2],
    },
    portrait: {
        main: [-1, 2, 0.5],
        fill: [-1, 0.5, -1.5],
    },
    upfront: {
        main: [0, 2, 1],
        fill: [-1, 0.5, -1.5],
    },
    soft: {
        main: [-2, 4, 4],
        fill: [-1, 0.5, -1.5],
    },
};
const defaultOptions = {
    preset: 'rembrandt',
    shadows: 'contact',
    adjustCamera: true,
    environment: 'city',
    intensity: 0.5,
};
class NgtsStageRefit {
    constructor() {
        this.radius = input.required();
        this.adjustCamera = input.required();
        const bounds = inject(NgtsBounds);
        effect(() => {
            const [, adjustCamera] = [this.radius(), this.adjustCamera()];
            if (adjustCamera)
                bounds.refresh().clip().fit();
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsStageRefit, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "17.1.0", version: "19.1.4", type: NgtsStageRefit, isStandalone: true, selector: "ngts-stage-refit", inputs: { radius: { classPropertyName: "radius", publicName: "radius", isSignal: true, isRequired: true, transformFunction: null }, adjustCamera: { classPropertyName: "adjustCamera", publicName: "adjustCamera", isSignal: true, isRequired: true, transformFunction: null } }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsStageRefit, decorators: [{
            type: Directive,
            args: [{ selector: 'ngts-stage-refit' }]
        }], ctorParameters: () => [] });
class NgtsStage {
    constructor() {
        this.options = input(defaultOptions, { transform: mergeInputs(defaultOptions) });
        this.parameters = omit(this.options, ['preset', 'shadows', 'adjustCamera', 'environment', 'intensity', 'center']);
        this.centered = output();
        this.dims = signal({ radius: 0, width: 0, height: 0, depth: 0 });
        this.radius = pick(this.dims, 'radius');
        this.height = pick(this.dims, 'height');
        this.center = pick(this.options, 'center');
        this.adjustCamera = pick(this.options, 'adjustCamera');
        this.margin = computed(() => Number(this.adjustCamera()));
        this.intensity = pick(this.options, 'intensity');
        this.shadows = pick(this.options, 'shadows');
        this.environment = pick(this.options, 'environment');
        this.preset = pick(this.options, 'preset');
        this.config = computed(() => {
            const preset = this.preset();
            return typeof preset === 'string' ? presets[preset] : preset;
        });
        this.shadowBias = computed(() => this.shadows().bias ?? -0.0001);
        this.normalBias = computed(() => this.shadows().normalBias ?? 0);
        this.shadowSize = computed(() => this.shadows().size ?? 1024);
        this.shadowOffset = computed(() => this.shadows().offset ?? 0);
        this.contactShadow = computed(() => this.shadows() === 'contact' || this.shadows()?.type === 'contact');
        this.accumulativeShadow = computed(() => this.shadows() === 'accumulative' || this.shadows()?.type === 'accumulative');
        this.shadowOptions = computed(() => {
            const shadows = this.shadows();
            if (this.contactShadow()) {
                return { scale: this.radius() * 4, far: this.radius(), blur: 2, ...shadows };
            }
            if (this.accumulativeShadow()) {
                return { temporal: true, frames: 100, alphaTest: 0.9, toneMapped: true, scale: this.radius() * 4, ...shadows };
            }
            return typeof shadows === 'object' ? shadows : {};
        });
        this.randomizedLightsOptions = computed(() => {
            if (!this.accumulativeShadow())
                return {};
            const shadows = this.shadows();
            return {
                amount: shadows.amount ?? 8,
                radius: shadows.radius ?? this.radius(),
                ambient: shadows.ambient ?? 0.5,
                intensity: shadows.intensity ?? 1,
                position: [
                    this.config().main[0] * this.radius(),
                    this.config().main[1] * this.radius(),
                    this.config().main[2] * this.radius(),
                ],
                size: this.radius() * 4,
                bias: -this.shadowBias(),
                mapSize: this.shadowSize(),
            };
        });
        this.environmentOptions = computed(() => {
            const environment = this.environment();
            return !environment ? {} : typeof environment === 'string' ? { preset: environment } : environment;
        });
        this.boundsOptions = computed(() => ({
            fit: !!this.adjustCamera(),
            clip: !!this.adjustCamera(),
            margin: Number(this.adjustCamera()),
            observe: true,
            ...this.parameters(),
        }));
        this.centerOptions = computed(() => ({
            ...(this.center() || {}),
            position: [0, this.shadowOffset() / 2, 0],
        }));
        extend({ AmbientLight, SpotLight, PointLight, Group, Vector2 });
    }
    onCenter($event) {
        const { width, height, depth, boundingSphere } = $event;
        this.dims.set({ radius: boundingSphere.radius, width, height, depth });
        this.centered.emit($event);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsStage, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "19.1.4", type: NgtsStage, isStandalone: true, selector: "ngts-stage", inputs: { options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, outputs: { centered: "centered" }, ngImport: i0, template: `
		<ngt-ambient-light [intensity]="intensity() / 3" />
		<ngt-spot-light
			[penumbra]="1"
			[position]="[config().main[0] * radius(), config().main[1] * radius(), config().main[2] * radius()]"
			[intensity]="intensity() * 2"
			[castShadow]="!!shadows()"
		>
			<ngt-value [rawValue]="shadowBias()" attach="shadow.bias" />
			<ngt-value [rawValue]="normalBias()" attach="shadow.normalBias" />
			<ngt-vector2 *args="[shadowSize(), shadowSize()]" attach="shadow.mapSize" />
		</ngt-spot-light>
		<ngt-point-light
			[position]="[config().fill[0] * radius(), config().fill[1] * radius(), config().fill[2] * radius()]"
			[intensity]="intensity()"
		/>

		<ngts-bounds [options]="boundsOptions()">
			<ngts-stage-refit [radius]="radius()" [adjustCamera]="!!adjustCamera()" />
			<ngts-center [options]="centerOptions()" (centered)="onCenter($event)">
				<ng-content />
			</ngts-center>
		</ngts-bounds>

		<ngt-group [position]="[0, -height() / 2 - shadowOffset() / 2, 0]">
			@if (contactShadow()) {
				<ngts-contact-shadows [options]="shadowOptions()" />
			}

			@if (accumulativeShadow()) {
				<ngts-accumulative-shadows [options]="shadowOptions()">
					<ngts-randomized-lights [options]="randomizedLightsOptions()" />
				</ngts-accumulative-shadows>
			}
		</ngt-group>

		@if (environment()) {
			<ngts-environment [options]="environmentOptions()" />
		}
	`, isInline: true, dependencies: [{ kind: "directive", type: NgtArgs, selector: "ng-template[args]", inputs: ["args"] }, { kind: "component", type: NgtsBounds, selector: "ngts-bounds", inputs: ["options"] }, { kind: "directive", type: NgtsStageRefit, selector: "ngts-stage-refit", inputs: ["radius", "adjustCamera"] }, { kind: "component", type: NgtsCenter, selector: "ngts-center", inputs: ["options"], outputs: ["centered"] }, { kind: "component", type: NgtsContactShadows, selector: "ngts-contact-shadows", inputs: ["options"] }, { kind: "component", type: NgtsAccumulativeShadows, selector: "ngts-accumulative-shadows", inputs: ["options"] }, { kind: "component", type: NgtsEnvironment, selector: "ngts-environment", inputs: ["options"], outputs: ["envSet"] }, { kind: "component", type: NgtsRandomizedLights, selector: "ngts-randomized-lights", inputs: ["options"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsStage, decorators: [{
            type: Component,
            args: [{
                    selector: 'ngts-stage',
                    template: `
		<ngt-ambient-light [intensity]="intensity() / 3" />
		<ngt-spot-light
			[penumbra]="1"
			[position]="[config().main[0] * radius(), config().main[1] * radius(), config().main[2] * radius()]"
			[intensity]="intensity() * 2"
			[castShadow]="!!shadows()"
		>
			<ngt-value [rawValue]="shadowBias()" attach="shadow.bias" />
			<ngt-value [rawValue]="normalBias()" attach="shadow.normalBias" />
			<ngt-vector2 *args="[shadowSize(), shadowSize()]" attach="shadow.mapSize" />
		</ngt-spot-light>
		<ngt-point-light
			[position]="[config().fill[0] * radius(), config().fill[1] * radius(), config().fill[2] * radius()]"
			[intensity]="intensity()"
		/>

		<ngts-bounds [options]="boundsOptions()">
			<ngts-stage-refit [radius]="radius()" [adjustCamera]="!!adjustCamera()" />
			<ngts-center [options]="centerOptions()" (centered)="onCenter($event)">
				<ng-content />
			</ngts-center>
		</ngts-bounds>

		<ngt-group [position]="[0, -height() / 2 - shadowOffset() / 2, 0]">
			@if (contactShadow()) {
				<ngts-contact-shadows [options]="shadowOptions()" />
			}

			@if (accumulativeShadow()) {
				<ngts-accumulative-shadows [options]="shadowOptions()">
					<ngts-randomized-lights [options]="randomizedLightsOptions()" />
				</ngts-accumulative-shadows>
			}
		</ngt-group>

		@if (environment()) {
			<ngts-environment [options]="environmentOptions()" />
		}
	`,
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    imports: [
                        NgtArgs,
                        NgtsBounds,
                        NgtsStageRefit,
                        NgtsCenter,
                        NgtsContactShadows,
                        NgtsAccumulativeShadows,
                        NgtsEnvironment,
                        NgtsRandomizedLights,
                    ],
                }]
        }], ctorParameters: () => [] });

/**
 * Generated bundle index. Do not edit.
 */

export { ENVIRONMENT_PRESETS, NgtsAccumulativeShadows, NgtsBBAnchor, NgtsBackdrop, NgtsBounds, NgtsCameraShake, NgtsCaustics, NgtsCenter, NgtsContactShadows, NgtsEnvironment, NgtsEnvironmentCube, NgtsEnvironmentGround, NgtsEnvironmentMap, NgtsEnvironmentPortal, NgtsFloat, NgtsLightformer, NgtsMask, NgtsMatcapTexture, NgtsNormalTexture, NgtsRandomizedLights, NgtsRenderTexture, NgtsRenderTextureContainer, NgtsRenderTextureContent, NgtsSky, NgtsSpotLight, NgtsSpotLightShadow, NgtsStage, NgtsStageRefit, calcPosFromAngles, injectEnvironment, injectMask, injectMatcapTexture, injectNormalTexture };
//# sourceMappingURL=angular-three-soba-staging.mjs.map
