import { DOCUMENT } from '@angular/common';
import * as i0 from '@angular/core';
import { input, inject, computed, untracked, effect, Directive } from '@angular/core';
import { omit, pick, injectStore, resolveRef, addAfterEffect } from 'angular-three';
import { mergeInputs } from 'ngxtension/inject-inputs';
import Stats from 'stats-gl';

const defaultOptions = {
    domClass: '',
    parent: null,
};
class NgtsStats {
    constructor() {
        this.options = input(defaultOptions, { transform: mergeInputs(defaultOptions), alias: 'stats' });
        const statsOptions = omit(this.options, ['parent', 'domClass']);
        const parent = pick(this.options, 'parent');
        const domClass = pick(this.options, 'domClass');
        const document = inject(DOCUMENT);
        const store = injectStore();
        const gl = store.select('gl');
        const stats = computed(() => {
            const _gl = gl();
            if (!_gl)
                return null;
            const stats = new Stats({ ...untracked(statsOptions) });
            void stats.init(_gl);
            return stats;
        });
        effect((onCleanup) => {
            const _stats = stats();
            if (!_stats)
                return;
            const [_parent, _domClass] = [resolveRef(parent()), domClass()];
            const target = _parent ?? document.body;
            target.appendChild(_stats.dom);
            const classList = _domClass.split(' ').filter(Boolean);
            if (classList.length)
                _stats.dom.classList.add(...classList);
            const end = addAfterEffect(() => _stats.update());
            onCleanup(() => {
                if (classList.length)
                    _stats.dom.classList.remove(...classList);
                target.removeChild(_stats.dom);
                end();
            });
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsStats, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "17.1.0", version: "19.1.4", type: NgtsStats, isStandalone: true, selector: "ngt-canvas[stats]", inputs: { options: { classPropertyName: "options", publicName: "stats", isSignal: true, isRequired: false, transformFunction: null } }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsStats, decorators: [{
            type: Directive,
            args: [{ selector: 'ngt-canvas[stats]' }]
        }], ctorParameters: () => [] });

/**
 * Generated bundle index. Do not edit.
 */

export { NgtsStats };
//# sourceMappingURL=angular-three-soba-stats.mjs.map
