import { ElementRef } from '@angular/core';
import { NgtAnyRecord } from 'angular-three';
import { Group, Matrix4, Vector3 } from 'three';
import * as i0 from "@angular/core";
export interface OnDragStartParameters {
    component: 'Arrow' | 'Slider' | 'Rotator' | 'Sphere';
    axis: 0 | 1 | 2;
    origin: Vector3;
    directions: Vector3[];
}
export interface OnDragParameters {
    l: Matrix4;
    deltaL: Matrix4;
    w: Matrix4;
    deltaW: Matrix4;
}
export interface NgtsPivotControlsOptions {
    /** Enables/disables the control, true */
    enabled: boolean;
    /** Scale of the gizmo, 1 */
    scale: number;
    /** Width of the gizmo lines, this is a THREE.Line2 prop, 2.5 */
    lineWidth: number;
    /** If fixed is true is remains constant in size, scale is now in pixels, false */
    fixed: boolean;
    /** Pivot does not act as a group, it won't shift contents but can offset in position */
    offset: [number, number, number];
    /** Starting rotation */
    rotation: [number, number, number];
    /** Starting matrix */
    matrix?: Matrix4;
    /** BBAnchor, each axis can be between -1/0/+1 */
    anchor?: [number, number, number];
    /** If autoTransform is true, automatically apply the local transform on drag, true */
    autoTransform: boolean;
    /** Allows you to switch individual axes off */
    activeAxes: [boolean, boolean, boolean];
    /** Allows you to switch individual transformations off */
    disableAxes: boolean;
    disableSliders: boolean;
    disableRotations: boolean;
    disableScaling: boolean;
    /** Limits */
    translationLimits?: [[number, number] | undefined, [number, number] | undefined, [number, number] | undefined];
    rotationLimits?: [[number, number] | undefined, [number, number] | undefined, [number, number] | undefined];
    scaleLimits?: [[number, number] | undefined, [number, number] | undefined, [number, number] | undefined];
    /** RGB colors */
    axisColors: [string | number, string | number, string | number];
    /** Color of the hovered item */
    hoveredColor: string | number;
    /** HTML value annotations, default: false */
    annotations: boolean;
    /** CSS Classname applied to the HTML annotations */
    annotationsClass?: string;
    /** Set this to false if you want the gizmo to be visible through faces */
    depthTest: boolean;
    opacity?: number;
    visible: boolean;
    userData?: NgtAnyRecord;
}
export declare class NgtsPivotControls {
    options: import("@angular/core").InputSignalWithTransform<NgtsPivotControlsOptions, "" | Partial<NgtsPivotControlsOptions>>;
    parameters: import("@angular/core").Signal<Omit<NgtsPivotControlsOptions, "scale" | "rotation" | "matrix" | "anchor" | "fixed" | "visible" | "userData" | "enabled" | "autoTransform" | "disableAxes" | "disableSliders" | "disableRotations" | "disableScaling" | "activeAxes" | "offset" | "lineWidth" | "translationLimits" | "rotationLimits" | "scaleLimits" | "depthTest" | "axisColors" | "hoveredColor" | "annotations" | "annotationsClass" | "opacity">>;
    dragStarted: import("@angular/core").OutputEmitterRef<OnDragStartParameters>;
    dragEnded: import("@angular/core").OutputEmitterRef<void>;
    dragged: import("@angular/core").OutputEmitterRef<OnDragParameters>;
    matrix: import("@angular/core").Signal<Matrix4 | undefined>;
    annotations: import("@angular/core").Signal<boolean>;
    annotationsClass: import("@angular/core").Signal<string | undefined>;
    translationLimits: import("@angular/core").Signal<[[number, number] | undefined, [number, number] | undefined, [number, number] | undefined] | undefined>;
    rotationLimits: import("@angular/core").Signal<[[number, number] | undefined, [number, number] | undefined, [number, number] | undefined] | undefined>;
    scaleLimits: import("@angular/core").Signal<[[number, number] | undefined, [number, number] | undefined, [number, number] | undefined] | undefined>;
    autoTransform: import("@angular/core").Signal<boolean>;
    fixed: import("@angular/core").Signal<boolean>;
    hoveredColor: import("@angular/core").Signal<string | number>;
    axisColors: import("@angular/core").Signal<[string | number, string | number, string | number]>;
    lineWidth: import("@angular/core").Signal<number>;
    scale: import("@angular/core").Signal<number>;
    userData: import("@angular/core").Signal<NgtAnyRecord | undefined>;
    opacity: import("@angular/core").Signal<number | undefined>;
    depthTest: import("@angular/core").Signal<boolean>;
    offset: import("@angular/core").Signal<[number, number, number]>;
    rotation: import("@angular/core").Signal<[number, number, number]>;
    visible: import("@angular/core").Signal<boolean>;
    enabled: import("@angular/core").Signal<boolean>;
    disableAxes: import("@angular/core").Signal<boolean>;
    disableSliders: import("@angular/core").Signal<boolean>;
    disableRotations: import("@angular/core").Signal<boolean>;
    disableScaling: import("@angular/core").Signal<boolean>;
    activeAxes: import("@angular/core").Signal<[boolean, boolean, boolean]>;
    xDir: Vector3;
    yDir: Vector3;
    zDir: Vector3;
    parentRef: import("@angular/core").Signal<ElementRef<Group<import("three").Object3DEventMap>>>;
    groupRef: import("@angular/core").Signal<ElementRef<Group<import("three").Object3DEventMap>>>;
    gizmoRef: import("@angular/core").Signal<ElementRef<Group<import("three").Object3DEventMap>>>;
    childrenRef: import("@angular/core").Signal<ElementRef<Group<import("three").Object3DEventMap>>>;
    private store;
    private invalidate;
    translation: [number, number, number];
    private anchor;
    private cameraScale;
    private gizmoScale;
    constructor();
    onDragStart(parameters: OnDragStartParameters): void;
    onDrag(mdW: Matrix4): void;
    onDragEnd(): void;
    static ɵfac: i0.ɵɵFactoryDeclaration<NgtsPivotControls, never>;
    static ɵcmp: i0.ɵɵComponentDeclaration<NgtsPivotControls, "ngts-pivot-controls", never, { "options": { "alias": "options"; "required": false; "isSignal": true; }; }, { "dragStarted": "dragStarted"; "dragEnded": "dragEnded"; "dragged": "dragged"; }, never, ["*"], true, never>;
}
