import { ElementRef, Injector, Signal } from '@angular/core';
import { AnimationAction, AnimationClip, AnimationMixer, Object3D } from 'three';
/**
 * name: any to allow consumers to pass in type-safe animation clips
 */
type NgtsAnimationClipWithoutName = Omit<AnimationClip, 'name'> & {
    name: any;
};
export type NgtsAnimationClip = Omit<NgtsAnimationClipWithoutName, 'clone'> & {
    clone: () => NgtsAnimationClip;
};
export type NgtsAnimationClips<TAnimationNames extends string> = {
    [Name in TAnimationNames]: Omit<NgtsAnimationClip, 'name'> & {
        name: Name;
    };
}[TAnimationNames];
export type NgtsAnimationApi<T extends NgtsAnimationClip> = {
    /**
     * Whether or not the animations finishes initialized
     *
     * @deprecated 3.5.0 - use `isReady` getter for better type-narrow instead. Will be removed in 4.0.0
     */
    ready: Signal<boolean>;
    clips: T[];
    mixer: AnimationMixer;
    names: T['name'][];
} & ({
    get isReady(): true;
    actions: {
        [key in T['name']]: AnimationAction;
    };
} | {
    get isReady(): false;
});
export type NgtsAnimation<TAnimation extends NgtsAnimationClip = NgtsAnimationClip> = TAnimation[] | {
    animations: TAnimation[];
};
/**
 * Use afterNextRender
 */
export declare function injectAnimations<TAnimation extends NgtsAnimationClip>(animations: () => NgtsAnimation<TAnimation> | undefined | null, object: ElementRef<Object3D> | Object3D | (() => ElementRef<Object3D> | Object3D | undefined | null), { injector }?: {
    injector?: Injector;
}): NgtsAnimationApi<TAnimation>;
export {};
