import { ElementRef, Injector } from '@angular/core';
import { NgtGroup } from 'angular-three';
import { BufferGeometry, Color, Group, InstancedBufferAttribute, InstancedMesh, Mesh, Object3D, Vector3 } from 'three';
import * as i0 from "@angular/core";
interface SamplePayload {
    /**
     * The position of the sample.
     */
    position: Vector3;
    /**
     * The normal of the mesh at the sampled position.
     */
    normal: Vector3;
    /**
     * The vertex color of the mesh at the sampled position.
     */
    color: Color;
}
export type TransformFn = (payload: TransformPayload, i: number) => void;
interface TransformPayload extends SamplePayload {
    /**
     * The dummy object used to transform each instance.
     * This object's matrix will be updated after transforming & it will be used
     * to set the instance's matrix.
     */
    dummy: Object3D;
    /**
     * The mesh that's initially passed to the sampler.
     * Use this if you need to apply transforms from your mesh to your instances
     * or if you need to grab attributes from the geometry.
     */
    sampledMesh: Mesh;
}
export declare function injectSurfaceSampler(mesh: () => ElementRef<Mesh> | Mesh | null | undefined, options?: () => {
    count?: number;
    transform?: TransformFn;
    weight?: string;
    instanceMesh?: ElementRef<InstancedMesh> | InstancedMesh | null;
}, { injector }?: {
    injector?: Injector;
}): import("@angular/core").Signal<InstancedBufferAttribute>;
export interface NgtsSamplerOptions extends Partial<NgtGroup> {
    /**
     * The NAME of the weight attribute to use when sampling.
     *
     * @see https://threejs.org/docs/#examples/en/math/MeshSurfaceSampler.setWeightAttribute
     */
    weight?: string;
    /**
     * Transformation to be applied to each instance.
     * Receives a dummy object3D with all the sampled data ( @see TransformPayload ).
     * It should mutate `transformPayload.dummy`.
     *
     * @see ( @todo add link to example )
     *
     * There is no need to update the dummy's matrix
     */
    transform?: TransformFn;
    count: number;
}
export declare class NgtsSampler {
    mesh: import("@angular/core").InputSignal<Mesh<BufferGeometry<import("three").NormalBufferAttributes>, import("three").Material | import("three").Material[], import("three").Object3DEventMap> | ElementRef<Mesh<BufferGeometry<import("three").NormalBufferAttributes>, import("three").Material | import("three").Material[], import("three").Object3DEventMap>> | null>;
    instances: import("@angular/core").InputSignal<InstancedMesh<BufferGeometry<import("three").NormalBufferAttributes>, import("three").Material | import("three").Material[], import("three").InstancedMeshEventMap> | ElementRef<InstancedMesh<BufferGeometry<import("three").NormalBufferAttributes>, import("three").Material | import("three").Material[], import("three").InstancedMeshEventMap>> | null>;
    options: import("@angular/core").InputSignalWithTransform<NgtsSamplerOptions, "" | Partial<NgtsSamplerOptions>>;
    parameters: import("@angular/core").Signal<Omit<NgtsSamplerOptions, "transform" | "weight" | "count">>;
    groupRef: import("@angular/core").Signal<ElementRef<Group<import("three").Object3DEventMap>> | undefined>;
    constructor();
    static ɵfac: i0.ɵɵFactoryDeclaration<NgtsSampler, never>;
    static ɵcmp: i0.ɵɵComponentDeclaration<NgtsSampler, "ngts-sampler", never, { "mesh": { "alias": "mesh"; "required": false; "isSignal": true; }; "instances": { "alias": "instances"; "required": false; "isSignal": true; }; "options": { "alias": "options"; "required": false; "isSignal": true; }; }, {}, never, ["*"], true, never>;
}
export {};
