import { ElementRef } from '@angular/core';
import { NgtGroup } from 'angular-three';
import { ColorRepresentation, Group, Object3D } from 'three';
import * as i0 from "@angular/core";
export interface NgtsCausticsOptions extends Partial<NgtGroup> {
    /** How many frames it will render, set it to Infinity for runtime, default: 1 */
    frames: number;
    /** Enables visual cues to help you stage your scene, default: false */
    debug: boolean;
    /** Will display caustics only and skip the models, default: false */
    causticsOnly: boolean;
    /** Will include back faces and enable the backsideIOR prop, default: false */
    backside: boolean;
    /** The IOR refraction index, default: 1.1 */
    ior: number;
    /** The IOR refraction index for back faces (only available when backside is enabled), default: 1.1 */
    backsideIOR: number;
    /** The texel size, default: 0.3125 */
    worldRadius: number;
    /** Intensity of the prjected caustics, default: 0.05 */
    intensity: number;
    /** Caustics color, default: white */
    color: ColorRepresentation;
    /** Buffer resolution, default: 2048 */
    resolution: number;
    /** Camera position, it will point towards the contents bounds center, default: [5, 5, 5] */
    lightSource: [x: number, y: number, z: number] | ElementRef<Object3D> | Object3D;
}
export declare class NgtsCaustics {
    protected readonly Math: Math;
    protected readonly CustomBlending: 5;
    protected readonly OneFactor: 201;
    protected readonly SrcAlphaFactor: 204;
    options: import("@angular/core").InputSignalWithTransform<NgtsCausticsOptions, "" | Partial<NgtsCausticsOptions>>;
    parameters: import("@angular/core").Signal<Omit<NgtsCausticsOptions, "intensity" | "frames" | "color" | "resolution" | "debug" | "causticsOnly" | "backside" | "ior" | "backsideIOR" | "worldRadius" | "lightSource">>;
    debug: import("@angular/core").Signal<boolean>;
    color: import("@angular/core").Signal<ColorRepresentation>;
    private resolution;
    groupRef: import("@angular/core").Signal<ElementRef<Group<import("three").Object3DEventMap>>>;
    private sceneRef;
    private cameraRef;
    private planeRef;
    private normalTargetParams;
    private normalTarget;
    private normalTargetB;
    private causticsTargetParams;
    causticsTarget: import("@angular/core").Signal<import("three").WebGLRenderTarget<import("three").Texture>>;
    causticsTargetB: import("@angular/core").Signal<import("three").WebGLRenderTarget<import("three").Texture>>;
    private cameraHelper;
    constructor();
    static ɵfac: i0.ɵɵFactoryDeclaration<NgtsCaustics, never>;
    static ɵcmp: i0.ɵɵComponentDeclaration<NgtsCaustics, "ngts-caustics", never, { "options": { "alias": "options"; "required": false; "isSignal": true; }; }, {}, never, ["*"], true, never>;
}
