import { ElementRef } from '@angular/core';
import { NgtSpotLight } from 'angular-three';
import { ColorRepresentation, CylinderGeometry, DepthTexture, Mesh, ShaderMaterial, SpotLight, SpotLightHelper, Texture, Vector3 } from 'three';
import * as i0 from "@angular/core";
export interface NgtsSpotLightOptions extends Partial<NgtSpotLight> {
    depthBuffer?: DepthTexture;
    attenuation?: number;
    anglePower?: number;
    radiusTop?: number;
    radiusBottom?: number;
    opacity?: number;
    color?: ColorRepresentation;
    volumetric?: boolean;
    debug?: boolean;
}
export type NgtsVolumetricMeshOptions = Omit<NgtsSpotLightOptions, 'volumetric'>;
export declare class NgtsVolumetricMesh {
    options: import("@angular/core").InputSignalWithTransform<NgtsVolumetricMeshOptions, "" | Partial<NgtsVolumetricMeshOptions>>;
    meshRef: import("@angular/core").Signal<ElementRef<Mesh<import("three").BufferGeometry<import("three").NormalBufferAttributes>, import("three").Material | import("three").Material[], import("three").Object3DEventMap>>>;
    private store;
    size: import("@angular/core").Signal<import("angular-three").NgtSize>;
    dpr: import("@angular/core").Signal<number>;
    camera: import("@angular/core").Signal<import("angular-three").NgtCameraManual>;
    private radiusTop;
    private radiusBottom;
    private angle;
    private distance;
    private normalizedRadiusTop;
    private normalizedRadiusBottom;
    material: ShaderMaterial & {
        depth: Texture | null;
        opacity: number;
        attenuation: number;
        anglePower: number;
        spotPosition: Vector3;
        lightColor: import("three").Color;
        cameraNear: number;
        cameraFar: number;
        resolution: import("three").Vector2;
        transparent: boolean;
        depthWrite: boolean;
    };
    opacity: import("@angular/core").Signal<number | undefined>;
    color: import("@angular/core").Signal<ColorRepresentation | undefined>;
    attenuation: import("@angular/core").Signal<number | undefined>;
    anglePower: import("@angular/core").Signal<number | undefined>;
    depthBuffer: import("@angular/core").Signal<DepthTexture | undefined>;
    geometry: import("@angular/core").Signal<CylinderGeometry>;
    constructor();
    static ɵfac: i0.ɵɵFactoryDeclaration<NgtsVolumetricMesh, never>;
    static ɵcmp: i0.ɵɵComponentDeclaration<NgtsVolumetricMesh, "ngts-volumetric-mesh", never, { "options": { "alias": "options"; "required": false; "isSignal": true; }; }, {}, never, never, true, never>;
}
interface NgtsShadowMeshOptions {
    distance: number;
    alphaTest: number;
    scale: number;
    width: number;
    height: number;
    map?: Texture;
}
export declare class NgtsSpotLightShadowShader {
    protected readonly DoubleSide: 2;
    shader: import("@angular/core").InputSignal<string>;
    options: import("@angular/core").InputSignalWithTransform<NgtsShadowMeshOptions, "" | Partial<NgtsShadowMeshOptions>>;
    mesh: import("@angular/core").Signal<ElementRef<Mesh<import("three").BufferGeometry<import("three").NormalBufferAttributes>, import("three").Material | import("three").Material[], import("three").Object3DEventMap>>>;
    private spotLight;
    debug: import("@angular/core").Signal<boolean | undefined>;
    scale: import("@angular/core").Signal<number>;
    alphaTest: import("@angular/core").Signal<number>;
    private width;
    private height;
    private map;
    private distance;
    private renderTarget;
    texture: import("@angular/core").Signal<Texture>;
    private fsQuad;
    private uniforms;
    constructor();
    static ɵfac: i0.ɵɵFactoryDeclaration<NgtsSpotLightShadowShader, never>;
    static ɵcmp: i0.ɵɵComponentDeclaration<NgtsSpotLightShadowShader, "ngts-spot-light-shadow-shader", never, { "shader": { "alias": "shader"; "required": true; "isSignal": true; }; "options": { "alias": "options"; "required": false; "isSignal": true; }; }, {}, never, ["*"], true, never>;
}
export declare class NgtsSpotLightShadowNoShader {
    protected readonly DoubleSide: 2;
    options: import("@angular/core").InputSignalWithTransform<NgtsShadowMeshOptions, "" | Partial<NgtsShadowMeshOptions>>;
    mesh: import("@angular/core").Signal<ElementRef<Mesh<import("three").BufferGeometry<import("three").NormalBufferAttributes>, import("three").Material | import("three").Material[], import("three").Object3DEventMap>>>;
    private spotLight;
    debug: import("@angular/core").Signal<boolean | undefined>;
    alphaTest: import("@angular/core").Signal<number>;
    scale: import("@angular/core").Signal<number>;
    map: import("@angular/core").Signal<Texture | undefined>;
    private width;
    private height;
    private distance;
    constructor();
    static ɵfac: i0.ɵɵFactoryDeclaration<NgtsSpotLightShadowNoShader, never>;
    static ɵcmp: i0.ɵɵComponentDeclaration<NgtsSpotLightShadowNoShader, "ngts-spot-light-shadow-no-shader", never, { "options": { "alias": "options"; "required": false; "isSignal": true; }; }, {}, never, ["*"], true, never>;
}
export declare class NgtsSpotLightShadow {
    shader: import("@angular/core").InputSignal<string | undefined>;
    options: import("@angular/core").InputSignalWithTransform<NgtsShadowMeshOptions, "" | Partial<NgtsShadowMeshOptions>>;
    static ɵfac: i0.ɵɵFactoryDeclaration<NgtsSpotLightShadow, never>;
    static ɵcmp: i0.ɵɵComponentDeclaration<NgtsSpotLightShadow, "ngts-spot-light-shadow", never, { "shader": { "alias": "shader"; "required": false; "isSignal": true; }; "options": { "alias": "options"; "required": false; "isSignal": true; }; }, {}, never, never, true, never>;
}
export declare class NgtsSpotLight {
    protected readonly SpotLightHelper: typeof SpotLightHelper;
    options: import("@angular/core").InputSignalWithTransform<NgtsSpotLightOptions, "" | Partial<NgtsSpotLightOptions>>;
    parameters: import("@angular/core").Signal<Omit<NgtsSpotLightOptions, "opacity" | "color" | "volumetric" | "distance" | "angle" | "depthBuffer" | "attenuation" | "anglePower" | "radiusTop" | "radiusBottom" | "debug">>;
    volumetricOptions: import("@angular/core").Signal<Pick<NgtsSpotLightOptions, "opacity" | "color" | "distance" | "angle" | "depthBuffer" | "attenuation" | "anglePower" | "radiusTop" | "radiusBottom" | "debug">>;
    spotLight: import("@angular/core").Signal<ElementRef<SpotLight>>;
    debug: import("@angular/core").Signal<boolean | undefined>;
    angle: import("@angular/core").Signal<number | undefined>;
    color: import("@angular/core").Signal<ColorRepresentation | undefined>;
    distance: import("@angular/core").Signal<number | undefined>;
    volumetric: import("@angular/core").Signal<boolean | undefined>;
    constructor();
    static ɵfac: i0.ɵɵFactoryDeclaration<NgtsSpotLight, never>;
    static ɵcmp: i0.ɵɵComponentDeclaration<NgtsSpotLight, "ngts-spot-light", never, { "options": { "alias": "options"; "required": false; "isSignal": true; }; }, {}, never, ["*"], true, never>;
}
export {};
