import { NgtGroup } from 'angular-three';
import { Group } from 'three';
import { NgtsAccumulativeShadowsOptions } from './accumulative-shadows';
import { NgtsCenterOptions, NgtsCenterState } from './center';
import { NgtsContactShadowsOptions } from './contact-shadows';
import { NgtsEnvironmentOptions } from './environment/environment';
import { NgtsEnvironmentPresets } from './environment/inject-environment';
import { NgtsRandomizedLightsOptions } from './randomized-lights';
import * as i0 from "@angular/core";
type NgtsStageShadowsOptions = Partial<NgtsAccumulativeShadowsOptions> & Partial<NgtsRandomizedLightsOptions> & Partial<NgtsContactShadowsOptions> & {
    type: 'contact' | 'accumulative';
    /** Shadow plane offset, default: 0 */
    offset: number;
    /** Shadow bias, default: -0.0001 */
    bias: number;
    /** Shadow normal bias, default: 0 */
    normalBias: number;
    /** Shadow map size, default: 1024 */
    size: number;
};
export interface NgtsStageOptions extends Partial<NgtGroup> {
    /** Lighting setup, default: "rembrandt" */
    preset: 'rembrandt' | 'portrait' | 'upfront' | 'soft' | {
        main: [x: number, y: number, z: number];
        fill: [x: number, y: number, z: number];
    };
    /** Controls the ground shadows, default: "contact" */
    shadows: boolean | 'contact' | 'accumulative' | NgtsStageShadowsOptions;
    /** Optionally wraps and thereby centers the models using <Bounds>, can also be a margin, default: true */
    adjustCamera: boolean | number;
    /** The default environment, default: "city" */
    environment: NgtsEnvironmentPresets | Partial<NgtsEnvironmentOptions> | null;
    /** The lighting intensity, default: 0.5 */
    intensity: number;
    /** To adjust centering, default: undefined */
    center?: Partial<NgtsCenterOptions>;
}
export declare class NgtsStageRefit {
    radius: import("@angular/core").InputSignal<number>;
    adjustCamera: import("@angular/core").InputSignal<boolean>;
    constructor();
    static ɵfac: i0.ɵɵFactoryDeclaration<NgtsStageRefit, never>;
    static ɵdir: i0.ɵɵDirectiveDeclaration<NgtsStageRefit, "ngts-stage-refit", never, { "radius": { "alias": "radius"; "required": true; "isSignal": true; }; "adjustCamera": { "alias": "adjustCamera"; "required": true; "isSignal": true; }; }, {}, never, never, true, never>;
}
export declare class NgtsStage {
    options: import("@angular/core").InputSignalWithTransform<NgtsStageOptions, "" | Partial<NgtsStageOptions>>;
    private parameters;
    centered: import("@angular/core").OutputEmitterRef<NgtsCenterState>;
    private dims;
    radius: import("@angular/core").Signal<number>;
    height: import("@angular/core").Signal<number>;
    center: import("@angular/core").Signal<Partial<NgtsCenterOptions> | undefined>;
    adjustCamera: import("@angular/core").Signal<number | boolean>;
    margin: import("@angular/core").Signal<number>;
    intensity: import("@angular/core").Signal<number>;
    shadows: import("@angular/core").Signal<boolean | "contact" | "accumulative" | NgtsStageShadowsOptions>;
    environment: import("@angular/core").Signal<"apartment" | "city" | "dawn" | "forest" | "lobby" | "night" | "park" | "studio" | "sunset" | "warehouse" | Partial<NgtsEnvironmentOptions> | null>;
    preset: import("@angular/core").Signal<"rembrandt" | "portrait" | "upfront" | "soft" | {
        main: [x: number, y: number, z: number];
        fill: [x: number, y: number, z: number];
    }>;
    config: import("@angular/core").Signal<{
        main: [x: number, y: number, z: number];
        fill: [x: number, y: number, z: number];
    } | {
        readonly main: readonly [1, 2, 1];
        readonly fill: readonly [-2, -0.5, -2];
    } | {
        readonly main: readonly [-1, 2, 0.5];
        readonly fill: readonly [-1, 0.5, -1.5];
    } | {
        readonly main: readonly [0, 2, 1];
        readonly fill: readonly [-1, 0.5, -1.5];
    } | {
        readonly main: readonly [-2, 4, 4];
        readonly fill: readonly [-1, 0.5, -1.5];
    }>;
    shadowBias: import("@angular/core").Signal<number>;
    normalBias: import("@angular/core").Signal<number>;
    shadowSize: import("@angular/core").Signal<number>;
    shadowOffset: import("@angular/core").Signal<number>;
    contactShadow: import("@angular/core").Signal<boolean>;
    accumulativeShadow: import("@angular/core").Signal<boolean>;
    shadowOptions: import("@angular/core").Signal<{}>;
    randomizedLightsOptions: import("@angular/core").Signal<Partial<NgtsRandomizedLightsOptions>>;
    environmentOptions: import("@angular/core").Signal<Partial<NgtsEnvironmentOptions>>;
    boundsOptions: import("@angular/core").Signal<{
        position?: import("angular-three").NgtVector3 | undefined;
        up?: import("angular-three").NgtVector3 | undefined;
        scale?: import("angular-three").NgtVector3 | undefined;
        rotation?: import("angular-three").NgtEuler | undefined;
        matrix?: import("angular-three").NgtMatrix4 | undefined;
        quaternion?: import("angular-three").NgtQuaternion | undefined;
        layers?: import("angular-three").NgtLayers | undefined;
        dispose?: (() => void) | null | undefined;
        raycast?: ((raycaster: import("three").Raycaster, intersects: import("three").Intersection[]) => void) | null | undefined;
        addEventListener?: ((<TEventKey extends keyof import("angular-three").NgtNodeEventMap<TOriginal>>(type: TEventKey, listener: (this: import("angular-three").NgtNodeElement<Group<import("three").Object3DEventMap>, typeof Group>, ev: import("angular-three").NgtNodeEventMap<Group<import("three").Object3DEventMap>>[TEventKey]) => any) => void) & (<TEventKey extends keyof import("angular-three").NgtAllObject3DEventsMap>(type: TEventKey, listener: (this: {
            addEventListener: (<TEventKey_1 extends keyof import("angular-three").NgtNodeEventMap<TOriginal>>(type: TEventKey_1, listener: (this: import("angular-three").NgtNodeElement<Group<import("three").Object3DEventMap>, typeof Group>, ev: import("angular-three").NgtNodeEventMap<Group<import("three").Object3DEventMap>>[TEventKey_1]) => any) => void) & /*elided*/ any;
            removeEventListener: (<TEventKey_1 extends keyof import("angular-three").NgtNodeEventMap<TOriginal>>(type: TEventKey_1, listener: (this: import("angular-three").NgtNodeElement<Group<import("three").Object3DEventMap>, typeof Group>, ev: import("angular-three").NgtNodeEventMap<Group<import("three").Object3DEventMap>>[TEventKey_1]) => any) => void) & (<TEventKey_1 extends keyof import("angular-three").NgtAllObject3DEventsMap>(type: TEventKey_1, listener: (this: /*elided*/ any, ev: import("angular-three").NgtAllObject3DEventsMap[TEventKey_1]) => any) => void);
            attach: string | string[] | import("angular-three").NgtAttachFunction;
            __ngt_args__: [];
            readonly isGroup?: true | undefined;
            readonly isObject3D?: true | undefined;
            readonly id?: number | undefined;
            uuid?: string | undefined;
            name?: string | undefined;
            readonly type?: string | undefined;
            parent?: import("three").Object3D<import("three").Object3DEventMap> | null | undefined;
            children?: import("three").Object3D<import("three").Object3DEventMap>[] | undefined;
            readonly modelViewMatrix?: import("three").Matrix4 | undefined;
            readonly normalMatrix?: import("three").Matrix3 | undefined;
            matrixWorld?: import("three").Matrix4 | undefined;
            matrixAutoUpdate?: boolean | undefined;
            matrixWorldAutoUpdate?: boolean | undefined;
            matrixWorldNeedsUpdate?: boolean | undefined;
            visible?: boolean | undefined;
            castShadow?: boolean | undefined;
            receiveShadow?: boolean | undefined;
            frustumCulled?: boolean | undefined;
            renderOrder?: number | undefined;
            animations?: import("three").AnimationClip[] | undefined;
            userData?: Record<string, any> | undefined;
            customDepthMaterial?: import("three").Material | undefined;
            customDistanceMaterial?: import("three").Material | undefined;
            onBeforeShadow?: ((renderer: import("three").WebGLRenderer, scene: import("three").Scene, camera: import("three").Camera, shadowCamera: import("three").Camera, geometry: import("three").BufferGeometry, depthMaterial: import("three").Material, group: Group) => void) | undefined;
            onAfterShadow?: ((renderer: import("three").WebGLRenderer, scene: import("three").Scene, camera: import("three").Camera, shadowCamera: import("three").Camera, geometry: import("three").BufferGeometry, depthMaterial: import("three").Material, group: Group) => void) | undefined;
            onBeforeRender?: ((renderer: import("three").WebGLRenderer, scene: import("three").Scene, camera: import("three").Camera, geometry: import("three").BufferGeometry, material: import("three").Material, group: Group) => void) | undefined;
            onAfterRender?: ((renderer: import("three").WebGLRenderer, scene: import("three").Scene, camera: import("three").Camera, geometry: import("three").BufferGeometry, material: import("three").Material, group: Group) => void) | undefined;
            applyMatrix4?: ((matrix: import("three").Matrix4) => void) | undefined;
            applyQuaternion?: ((quaternion: import("three").Quaternion) => Group<import("three").Object3DEventMap>) | undefined;
            setRotationFromAxisAngle?: ((axis: import("three").Vector3, angle: number) => void) | undefined;
            setRotationFromEuler?: ((euler: import("three").Euler) => void) | undefined;
            setRotationFromMatrix?: ((m: import("three").Matrix4) => void) | undefined;
            setRotationFromQuaternion?: ((q: import("three").Quaternion) => void) | undefined;
            rotateOnAxis?: ((axis: import("three").Vector3, angle: number) => Group<import("three").Object3DEventMap>) | undefined;
            rotateOnWorldAxis?: ((axis: import("three").Vector3, angle: number) => Group<import("three").Object3DEventMap>) | undefined;
            rotateX?: ((angle: number) => Group<import("three").Object3DEventMap>) | undefined;
            rotateY?: ((angle: number) => Group<import("three").Object3DEventMap>) | undefined;
            rotateZ?: ((angle: number) => Group<import("three").Object3DEventMap>) | undefined;
            translateOnAxis?: ((axis: import("three").Vector3, distance: number) => Group<import("three").Object3DEventMap>) | undefined;
            translateX?: ((distance: number) => Group<import("three").Object3DEventMap>) | undefined;
            translateY?: ((distance: number) => Group<import("three").Object3DEventMap>) | undefined;
            translateZ?: ((distance: number) => Group<import("three").Object3DEventMap>) | undefined;
            localToWorld?: ((vector: import("three").Vector3) => import("three").Vector3) | undefined;
            worldToLocal?: ((vector: import("three").Vector3) => import("three").Vector3) | undefined;
            lookAt?: {
                (vector: import("three").Vector3): void;
                (x: number, y: number, z: number): void;
            } | undefined;
            add?: ((...object: import("three").Object3D[]) => Group<import("three").Object3DEventMap>) | undefined;
            remove?: ((...object: import("three").Object3D[]) => Group<import("three").Object3DEventMap>) | undefined;
            removeFromParent?: (() => Group<import("three").Object3DEventMap>) | undefined;
            clear?: (() => Group<import("three").Object3DEventMap>) | undefined;
            getObjectById?: ((id: number) => import("three").Object3D | undefined) | undefined;
            getObjectByName?: ((name: string) => import("three").Object3D | undefined) | undefined;
            getObjectByProperty?: ((name: string, value: any) => import("three").Object3D | undefined) | undefined;
            getObjectsByProperty?: ((name: string, value: any, optionalTarget?: import("three").Object3D[]) => import("three").Object3D[]) | undefined;
            getWorldPosition?: ((target: import("three").Vector3) => import("three").Vector3) | undefined;
            getWorldQuaternion?: ((target: import("three").Quaternion) => import("three").Quaternion) | undefined;
            getWorldScale?: ((target: import("three").Vector3) => import("three").Vector3) | undefined;
            getWorldDirection?: ((target: import("three").Vector3) => import("three").Vector3) | undefined;
            traverse?: ((callback: (object: import("three").Object3D) => any) => void) | undefined;
            traverseVisible?: ((callback: (object: import("three").Object3D) => any) => void) | undefined;
            traverseAncestors?: ((callback: (object: import("three").Object3D) => any) => void) | undefined;
            updateMatrix?: (() => void) | undefined;
            updateMatrixWorld?: ((force?: boolean) => void) | undefined;
            updateWorldMatrix?: ((updateParents: boolean, updateChildren: boolean) => void) | undefined;
            toJSON?: ((meta?: import("three").JSONMeta) => import("three").Object3DJSON) | undefined;
            clone?: ((recursive?: boolean) => Group<import("three").Object3DEventMap>) | undefined;
            copy?: ((object: import("three").Object3D, recursive?: boolean) => Group<import("three").Object3DEventMap>) | undefined;
            hasEventListener?: (<T extends keyof import("three").Object3DEventMap>(type: T, listener: import("three").EventListener<import("three").Object3DEventMap[T], T, Group<import("three").Object3DEventMap>>) => boolean) | undefined;
            dispatchEvent?: (<T extends keyof import("three").Object3DEventMap>(event: import("three").BaseEvent<T> & import("three").Object3DEventMap[T]) => void) | undefined;
            position: import("angular-three").NgtVector3;
            up: import("angular-three").NgtVector3;
            scale: import("angular-three").NgtVector3;
            rotation: import("angular-three").NgtEuler;
            matrix: import("angular-three").NgtMatrix4;
            quaternion: import("angular-three").NgtQuaternion;
            layers: import("angular-three").NgtLayers;
            dispose: (() => void) | null;
            raycast: import("three").Object3D["raycast"] | null;
        }, ev: import("angular-three").NgtAllObject3DEventsMap[TEventKey]) => any) => void)) | undefined;
        removeEventListener?: ((<TEventKey extends keyof import("angular-three").NgtNodeEventMap<TOriginal>>(type: TEventKey, listener: (this: import("angular-three").NgtNodeElement<Group<import("three").Object3DEventMap>, typeof Group>, ev: import("angular-three").NgtNodeEventMap<Group<import("three").Object3DEventMap>>[TEventKey]) => any) => void) & (<TEventKey extends keyof import("angular-three").NgtAllObject3DEventsMap>(type: TEventKey, listener: (this: {
            addEventListener: (<TEventKey_1 extends keyof import("angular-three").NgtNodeEventMap<TOriginal>>(type: TEventKey_1, listener: (this: import("angular-three").NgtNodeElement<Group<import("three").Object3DEventMap>, typeof Group>, ev: import("angular-three").NgtNodeEventMap<Group<import("three").Object3DEventMap>>[TEventKey_1]) => any) => void) & (<TEventKey_1 extends keyof import("angular-three").NgtAllObject3DEventsMap>(type: TEventKey_1, listener: (this: /*elided*/ any, ev: import("angular-three").NgtAllObject3DEventsMap[TEventKey_1]) => any) => void);
            removeEventListener: (<TEventKey_1 extends keyof import("angular-three").NgtNodeEventMap<TOriginal>>(type: TEventKey_1, listener: (this: import("angular-three").NgtNodeElement<Group<import("three").Object3DEventMap>, typeof Group>, ev: import("angular-three").NgtNodeEventMap<Group<import("three").Object3DEventMap>>[TEventKey_1]) => any) => void) & /*elided*/ any;
            attach: string | string[] | import("angular-three").NgtAttachFunction;
            __ngt_args__: [];
            readonly isGroup?: true | undefined;
            readonly isObject3D?: true | undefined;
            readonly id?: number | undefined;
            uuid?: string | undefined;
            name?: string | undefined;
            readonly type?: string | undefined;
            parent?: import("three").Object3D<import("three").Object3DEventMap> | null | undefined;
            children?: import("three").Object3D<import("three").Object3DEventMap>[] | undefined;
            readonly modelViewMatrix?: import("three").Matrix4 | undefined;
            readonly normalMatrix?: import("three").Matrix3 | undefined;
            matrixWorld?: import("three").Matrix4 | undefined;
            matrixAutoUpdate?: boolean | undefined;
            matrixWorldAutoUpdate?: boolean | undefined;
            matrixWorldNeedsUpdate?: boolean | undefined;
            visible?: boolean | undefined;
            castShadow?: boolean | undefined;
            receiveShadow?: boolean | undefined;
            frustumCulled?: boolean | undefined;
            renderOrder?: number | undefined;
            animations?: import("three").AnimationClip[] | undefined;
            userData?: Record<string, any> | undefined;
            customDepthMaterial?: import("three").Material | undefined;
            customDistanceMaterial?: import("three").Material | undefined;
            onBeforeShadow?: ((renderer: import("three").WebGLRenderer, scene: import("three").Scene, camera: import("three").Camera, shadowCamera: import("three").Camera, geometry: import("three").BufferGeometry, depthMaterial: import("three").Material, group: Group) => void) | undefined;
            onAfterShadow?: ((renderer: import("three").WebGLRenderer, scene: import("three").Scene, camera: import("three").Camera, shadowCamera: import("three").Camera, geometry: import("three").BufferGeometry, depthMaterial: import("three").Material, group: Group) => void) | undefined;
            onBeforeRender?: ((renderer: import("three").WebGLRenderer, scene: import("three").Scene, camera: import("three").Camera, geometry: import("three").BufferGeometry, material: import("three").Material, group: Group) => void) | undefined;
            onAfterRender?: ((renderer: import("three").WebGLRenderer, scene: import("three").Scene, camera: import("three").Camera, geometry: import("three").BufferGeometry, material: import("three").Material, group: Group) => void) | undefined;
            applyMatrix4?: ((matrix: import("three").Matrix4) => void) | undefined;
            applyQuaternion?: ((quaternion: import("three").Quaternion) => Group<import("three").Object3DEventMap>) | undefined;
            setRotationFromAxisAngle?: ((axis: import("three").Vector3, angle: number) => void) | undefined;
            setRotationFromEuler?: ((euler: import("three").Euler) => void) | undefined;
            setRotationFromMatrix?: ((m: import("three").Matrix4) => void) | undefined;
            setRotationFromQuaternion?: ((q: import("three").Quaternion) => void) | undefined;
            rotateOnAxis?: ((axis: import("three").Vector3, angle: number) => Group<import("three").Object3DEventMap>) | undefined;
            rotateOnWorldAxis?: ((axis: import("three").Vector3, angle: number) => Group<import("three").Object3DEventMap>) | undefined;
            rotateX?: ((angle: number) => Group<import("three").Object3DEventMap>) | undefined;
            rotateY?: ((angle: number) => Group<import("three").Object3DEventMap>) | undefined;
            rotateZ?: ((angle: number) => Group<import("three").Object3DEventMap>) | undefined;
            translateOnAxis?: ((axis: import("three").Vector3, distance: number) => Group<import("three").Object3DEventMap>) | undefined;
            translateX?: ((distance: number) => Group<import("three").Object3DEventMap>) | undefined;
            translateY?: ((distance: number) => Group<import("three").Object3DEventMap>) | undefined;
            translateZ?: ((distance: number) => Group<import("three").Object3DEventMap>) | undefined;
            localToWorld?: ((vector: import("three").Vector3) => import("three").Vector3) | undefined;
            worldToLocal?: ((vector: import("three").Vector3) => import("three").Vector3) | undefined;
            lookAt?: {
                (vector: import("three").Vector3): void;
                (x: number, y: number, z: number): void;
            } | undefined;
            add?: ((...object: import("three").Object3D[]) => Group<import("three").Object3DEventMap>) | undefined;
            remove?: ((...object: import("three").Object3D[]) => Group<import("three").Object3DEventMap>) | undefined;
            removeFromParent?: (() => Group<import("three").Object3DEventMap>) | undefined;
            clear?: (() => Group<import("three").Object3DEventMap>) | undefined;
            getObjectById?: ((id: number) => import("three").Object3D | undefined) | undefined;
            getObjectByName?: ((name: string) => import("three").Object3D | undefined) | undefined;
            getObjectByProperty?: ((name: string, value: any) => import("three").Object3D | undefined) | undefined;
            getObjectsByProperty?: ((name: string, value: any, optionalTarget?: import("three").Object3D[]) => import("three").Object3D[]) | undefined;
            getWorldPosition?: ((target: import("three").Vector3) => import("three").Vector3) | undefined;
            getWorldQuaternion?: ((target: import("three").Quaternion) => import("three").Quaternion) | undefined;
            getWorldScale?: ((target: import("three").Vector3) => import("three").Vector3) | undefined;
            getWorldDirection?: ((target: import("three").Vector3) => import("three").Vector3) | undefined;
            traverse?: ((callback: (object: import("three").Object3D) => any) => void) | undefined;
            traverseVisible?: ((callback: (object: import("three").Object3D) => any) => void) | undefined;
            traverseAncestors?: ((callback: (object: import("three").Object3D) => any) => void) | undefined;
            updateMatrix?: (() => void) | undefined;
            updateMatrixWorld?: ((force?: boolean) => void) | undefined;
            updateWorldMatrix?: ((updateParents: boolean, updateChildren: boolean) => void) | undefined;
            toJSON?: ((meta?: import("three").JSONMeta) => import("three").Object3DJSON) | undefined;
            clone?: ((recursive?: boolean) => Group<import("three").Object3DEventMap>) | undefined;
            copy?: ((object: import("three").Object3D, recursive?: boolean) => Group<import("three").Object3DEventMap>) | undefined;
            hasEventListener?: (<T extends keyof import("three").Object3DEventMap>(type: T, listener: import("three").EventListener<import("three").Object3DEventMap[T], T, Group<import("three").Object3DEventMap>>) => boolean) | undefined;
            dispatchEvent?: (<T extends keyof import("three").Object3DEventMap>(event: import("three").BaseEvent<T> & import("three").Object3DEventMap[T]) => void) | undefined;
            position: import("angular-three").NgtVector3;
            up: import("angular-three").NgtVector3;
            scale: import("angular-three").NgtVector3;
            rotation: import("angular-three").NgtEuler;
            matrix: import("angular-three").NgtMatrix4;
            quaternion: import("angular-three").NgtQuaternion;
            layers: import("angular-three").NgtLayers;
            dispose: (() => void) | null;
            raycast: import("three").Object3D["raycast"] | null;
        }, ev: import("angular-three").NgtAllObject3DEventsMap[TEventKey]) => any) => void)) | undefined;
        attach?: string | string[] | import("angular-three").NgtAttachFunction | undefined;
        __ngt_args__?: [] | undefined;
        isGroup?: true | undefined;
        isObject3D?: true | undefined;
        id?: number | undefined;
        uuid?: string | undefined;
        name?: string | undefined;
        type?: string | undefined;
        parent?: import("three").Object3D<import("three").Object3DEventMap> | null | undefined;
        children?: import("three").Object3D<import("three").Object3DEventMap>[] | undefined;
        modelViewMatrix?: import("three").Matrix4 | undefined;
        normalMatrix?: import("three").Matrix3 | undefined;
        matrixWorld?: import("three").Matrix4 | undefined;
        matrixAutoUpdate?: boolean | undefined;
        matrixWorldAutoUpdate?: boolean | undefined;
        matrixWorldNeedsUpdate?: boolean | undefined;
        visible?: boolean | undefined;
        castShadow?: boolean | undefined;
        receiveShadow?: boolean | undefined;
        frustumCulled?: boolean | undefined;
        renderOrder?: number | undefined;
        animations?: import("three").AnimationClip[] | undefined;
        userData?: Record<string, any> | undefined;
        customDepthMaterial?: import("three").Material | undefined;
        customDistanceMaterial?: import("three").Material | undefined;
        onBeforeShadow?: ((renderer: import("three").WebGLRenderer, scene: import("three").Scene, camera: import("three").Camera, shadowCamera: import("three").Camera, geometry: import("three").BufferGeometry, depthMaterial: import("three").Material, group: Group) => void) | undefined;
        onAfterShadow?: ((renderer: import("three").WebGLRenderer, scene: import("three").Scene, camera: import("three").Camera, shadowCamera: import("three").Camera, geometry: import("three").BufferGeometry, depthMaterial: import("three").Material, group: Group) => void) | undefined;
        onBeforeRender?: ((renderer: import("three").WebGLRenderer, scene: import("three").Scene, camera: import("three").Camera, geometry: import("three").BufferGeometry, material: import("three").Material, group: Group) => void) | undefined;
        onAfterRender?: ((renderer: import("three").WebGLRenderer, scene: import("three").Scene, camera: import("three").Camera, geometry: import("three").BufferGeometry, material: import("three").Material, group: Group) => void) | undefined;
        applyMatrix4?: ((matrix: import("three").Matrix4) => void) | undefined;
        applyQuaternion?: ((quaternion: import("three").Quaternion) => Group<import("three").Object3DEventMap>) | undefined;
        setRotationFromAxisAngle?: ((axis: import("three").Vector3, angle: number) => void) | undefined;
        setRotationFromEuler?: ((euler: import("three").Euler) => void) | undefined;
        setRotationFromMatrix?: ((m: import("three").Matrix4) => void) | undefined;
        setRotationFromQuaternion?: ((q: import("three").Quaternion) => void) | undefined;
        rotateOnAxis?: ((axis: import("three").Vector3, angle: number) => Group<import("three").Object3DEventMap>) | undefined;
        rotateOnWorldAxis?: ((axis: import("three").Vector3, angle: number) => Group<import("three").Object3DEventMap>) | undefined;
        rotateX?: ((angle: number) => Group<import("three").Object3DEventMap>) | undefined;
        rotateY?: ((angle: number) => Group<import("three").Object3DEventMap>) | undefined;
        rotateZ?: ((angle: number) => Group<import("three").Object3DEventMap>) | undefined;
        translateOnAxis?: ((axis: import("three").Vector3, distance: number) => Group<import("three").Object3DEventMap>) | undefined;
        translateX?: ((distance: number) => Group<import("three").Object3DEventMap>) | undefined;
        translateY?: ((distance: number) => Group<import("three").Object3DEventMap>) | undefined;
        translateZ?: ((distance: number) => Group<import("three").Object3DEventMap>) | undefined;
        localToWorld?: ((vector: import("three").Vector3) => import("three").Vector3) | undefined;
        worldToLocal?: ((vector: import("three").Vector3) => import("three").Vector3) | undefined;
        lookAt?: {
            (vector: import("three").Vector3): void;
            (x: number, y: number, z: number): void;
        } | undefined;
        add?: ((...object: import("three").Object3D[]) => Group<import("three").Object3DEventMap>) | undefined;
        remove?: ((...object: import("three").Object3D[]) => Group<import("three").Object3DEventMap>) | undefined;
        removeFromParent?: (() => Group<import("three").Object3DEventMap>) | undefined;
        clear?: (() => Group<import("three").Object3DEventMap>) | undefined;
        getObjectById?: ((id: number) => import("three").Object3D | undefined) | undefined;
        getObjectByName?: ((name: string) => import("three").Object3D | undefined) | undefined;
        getObjectByProperty?: ((name: string, value: any) => import("three").Object3D | undefined) | undefined;
        getObjectsByProperty?: ((name: string, value: any, optionalTarget?: import("three").Object3D[]) => import("three").Object3D[]) | undefined;
        getWorldPosition?: ((target: import("three").Vector3) => import("three").Vector3) | undefined;
        getWorldQuaternion?: ((target: import("three").Quaternion) => import("three").Quaternion) | undefined;
        getWorldScale?: ((target: import("three").Vector3) => import("three").Vector3) | undefined;
        getWorldDirection?: ((target: import("three").Vector3) => import("three").Vector3) | undefined;
        traverse?: ((callback: (object: import("three").Object3D) => any) => void) | undefined;
        traverseVisible?: ((callback: (object: import("three").Object3D) => any) => void) | undefined;
        traverseAncestors?: ((callback: (object: import("three").Object3D) => any) => void) | undefined;
        updateMatrix?: (() => void) | undefined;
        updateMatrixWorld?: ((force?: boolean) => void) | undefined;
        updateWorldMatrix?: ((updateParents: boolean, updateChildren: boolean) => void) | undefined;
        toJSON?: ((meta?: import("three").JSONMeta) => import("three").Object3DJSON) | undefined;
        clone?: ((recursive?: boolean) => Group<import("three").Object3DEventMap>) | undefined;
        copy?: ((object: import("three").Object3D, recursive?: boolean) => Group<import("three").Object3DEventMap>) | undefined;
        hasEventListener?: (<T extends keyof import("three").Object3DEventMap>(type: T, listener: import("three").EventListener<import("three").Object3DEventMap[T], T, Group<import("three").Object3DEventMap>>) => boolean) | undefined;
        dispatchEvent?: (<T extends keyof import("three").Object3DEventMap>(event: import("three").BaseEvent<T> & import("three").Object3DEventMap[T]) => void) | undefined;
        fit: boolean;
        clip: boolean;
        margin: number;
        observe: boolean;
    }>;
    centerOptions: import("@angular/core").Signal<Partial<NgtsCenterOptions>>;
    constructor();
    onCenter($event: NgtsCenterState): void;
    static ɵfac: i0.ɵɵFactoryDeclaration<NgtsStage, never>;
    static ɵcmp: i0.ɵɵComponentDeclaration<NgtsStage, "ngts-stage", never, { "options": { "alias": "options"; "required": false; "isSignal": true; }; }, { "centered": "centered"; }, never, ["*"], true, never>;
}
export {};
